@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -17,6 +17,9 @@
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
+#include <qregexp.h>
+#include <qstringlist.h>
+
 #include <comment.h>
 #include <xsltransform.h>
 
@@ -39,7 +42,7 @@ namespace BibTeX
         // nothing
     }
 
-    QString Comment::text()
+    QString Comment::text() const
     {
         return m_text;
     }
@@ -49,7 +52,7 @@ namespace BibTeX
         m_text = text;
     }
 
-    bool Comment::useCommand()
+    bool Comment::useCommand() const
     {
         return m_useCommand;
     }
@@ -59,9 +62,28 @@ namespace BibTeX
         m_useCommand = useCommand;
     }
 
-    bool Comment::containsPattern( const QString& pattern, EntryField::FieldType fieldType, bool caseSensitive )
+    bool Comment::containsPattern( const QString& pattern, EntryField::FieldType fieldType, FilterType filterType, bool caseSensitive ) const
     {
-        return fieldType == EntryField::ftUnknown && m_text.contains( pattern, caseSensitive );
+        if ( filterType == ftExact )
+        {
+            /** check for exact match */
+            return fieldType == EntryField::ftUnknown && m_text.contains( pattern, caseSensitive );
+        }
+        else
+        {
+            /** for each word in the search pattern ... */
+            QStringList words = QStringList::split( QRegExp( "\\s+" ), pattern );
+            unsigned int hits = 0;
+            for ( QStringList::Iterator it = words.begin(); it != words.end(); ++it )
+            {
+                /** check if word is contained in text */
+                if ( fieldType == EntryField::ftUnknown && m_text.contains( *it, caseSensitive ) )
+                    ++hits;
+            }
+
+            /** return success depending on filter type and number of hits */
+            return ( filterType == ftAnyWord && hits > 0 || filterType == ftEveryWord && hits == words.count() );
+        }
     }
 
     Element* Comment::clone()
