@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -17,9 +17,14 @@
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <qevent.h>
 #include <qdragobject.h>
 #include <qfile.h>
+#include <qvaluelist.h>
 #include <qcursor.h>
 #include <qbuffer.h>
 #include <qlistview.h>
@@ -33,13 +38,11 @@
 #include <ktempfile.h>
 #include <klocale.h>
 #include <kurl.h>
-#include <kdebug.h>
 #include <kurldrag.h>
 #include <kactionclasses.h>
 #include <kaction.h>
 #include <kpopupmenu.h>
 #include <kmessagebox.h>
-#include <kdebug.h>
 #include <kprogress.h>
 #include <kiconloader.h>
 #include <krun.h>
@@ -47,6 +50,11 @@
 #include <kbibtex_part.h>
 #include <documentlistviewitem.h>
 #include <file.h>
+#ifdef HAVE_LIBBIBUTILS
+#include <fileimporterbibutils.h>
+#else // HAVE_LIBBIBUTILS
+#include <fileimporterris.h>
+#endif // HAVE_LIBBIBUTILS
 #include <fileimporterbibtex.h>
 #include <fileexporterbibtex.h>
 #include <element.h>
@@ -75,7 +83,7 @@ namespace KBibTeX
         header() ->setMovingEnabled( TRUE );
         buildColumns();
 
-        //         setDragEnabled ( true );
+        setDragEnabled( true );
         //         setDragAutoScroll( true );
         setAcceptDrops( TRUE );
         setDropVisualizer( TRUE );
@@ -121,73 +129,105 @@ namespace KBibTeX
         bool update = viewport()->isUpdatesEnabled();
         viewport()->setUpdatesEnabled( FALSE );
 
+        if ( m_bibtexFile->count() > 250 )
+            qDebug( "This BibTeX source code contains many elements (%d) which thwarts KBibTeX when updating the main list.", m_bibtexFile->count() );
+
         clear();
         for ( unsigned int i = 0; i < m_bibtexFile->count(); i++ )
         {
             BibTeX::Element *element = m_bibtexFile->at( i );
-            QListViewItem *item = new DocumentListViewItem( m_bibtexFile, element, this );
-            item->setVisible( m_filter.isEmpty() || element->containsPattern( m_filter, m_filterFieldType ) );
+            new DocumentListViewItem( m_bibtexFile, element, this );
             progress->setProgress( i );
 
             if ( i % 43 == 23 )
                 kapp->processEvents();
         }
 
-        //setUpdatesEnabled( TRUE );
         viewport()->setUpdatesEnabled( update );
         triggerUpdate();
         delete prgDlg;
 
+        updateVisiblity();
+
         QApplication::restoreOverrideCursor();
     }
 
-    bool DocumentListView::insertItems( BibTeX::File *items, KBibTeX::DocumentListViewItem *after )
+    void DocumentListView::insertItems( BibTeX::File *items, KBibTeX::DocumentListViewItem *after )
+    {
+        for ( BibTeX::File::ElementList::iterator it = items->begin(); it != items->end(); it++ )
+            after = insertItem( *it, after );
+    }
+
+    KBibTeX::DocumentListViewItem * DocumentListView::insertItem( BibTeX::Element *item, KBibTeX::DocumentListViewItem *after )
     {
         if ( m_bibtexFile == NULL )
             m_bibtexFile = new BibTeX::File();
 
-        for ( BibTeX::File::ElementList::iterator it = items->begin(); it != items->end(); it++ )
+        BibTeX::Element *element = NULL;
+        BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry *>( item );
+        if ( entry != NULL )
         {
-            BibTeX::Element *element = BibTeX::File::cloneElement( *it );
-
-            m_bibtexFile->appendElement( element, after == NULL ? NULL : after->element() );
-            after = new DocumentListViewItem( m_bibtexFile, element, this, after );
-            after->setUnreadStatus( TRUE );
-            m_unreadItems.append( after );
+            BibTeX::Entry *newEntry = new BibTeX::Entry( entry );
+            if ( m_bibtexFile->containsKey( newEntry->id() ) )
+            {
+                int counter = 0;
+                QString newId = QString( newEntry->id() ).append( '_' ).append( QString::number( ++counter ) );
+                while ( m_bibtexFile->containsKey( newId ) )
+                    newId = QString( newEntry->id() ).append( '_' ).append( QString::number( ++counter ) );
+                newEntry->setId( newId );
+            }
+            element = newEntry;
         }
+        else
+            element = item->clone();
 
-        QTimer::singleShot( 2500, this, SLOT( makeNewItemsUnread() ) );
 
-        return TRUE;
+        m_bibtexFile->appendElement( element, after == NULL ? NULL : after->element() );
+        after = new DocumentListViewItem( m_bibtexFile, element, this, after );
+        after->setUnreadStatus( TRUE );
+        updateVisiblity( after );
+        m_unreadItems.append( after );
+
+        QTimer::singleShot( 3500, this, SLOT( makeNewItemsUnread() ) );
+
+        return after;
+    }
+
+    void DocumentListView::insertItem( BibTeX::Element *item )
+    {
+        insertItem( item, NULL );
     }
 
     void DocumentListView::updateVisiblity()
     {
-        Settings * settings = Settings::self( m_bibtexFile );
-
         QListViewItemIterator it( this );
         while ( it.current() )
         {
             DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
-            BibTeX::Element *element = kblvi->element();
-            bool notFiltered = m_filter.isEmpty() || element ->containsPattern( m_filter, m_filterFieldType );
-
-            BibTeX::Macro *macro = dynamic_cast<BibTeX::Macro*>( element );
-            if ( macro != NULL )
-                kblvi->setVisible( notFiltered && settings->editing_ShowMacros );
-            else
-            {
-                BibTeX::Comment *comment = dynamic_cast<BibTeX::Comment*>( element );
-                if ( comment != NULL )
-                    kblvi->setVisible( notFiltered && settings->editing_ShowComments );
-                else
-                    kblvi->setVisible( notFiltered );
-            }
-
+            updateVisiblity( kblvi );
             it++;
         }
     }
 
+    void DocumentListView::updateVisiblity( KBibTeX::DocumentListViewItem *item )
+    {
+        Settings * settings = Settings::self( m_bibtexFile );
+        BibTeX::Element *element = item->element();
+        bool notFiltered = m_filter.isEmpty() || element ->containsPattern( m_filter, m_filterFieldType, m_filterType );
+
+        BibTeX::Macro *macro = dynamic_cast<BibTeX::Macro*>( element );
+        if ( macro != NULL )
+            item->setVisible( notFiltered && settings->editing_ShowMacros );
+        else
+        {
+            BibTeX::Comment *comment = dynamic_cast<BibTeX::Comment*>( element );
+            if ( comment != NULL )
+                item->setVisible( notFiltered && settings->editing_ShowComments );
+            else
+                item->setVisible( notFiltered );
+        }
+    }
+
     void DocumentListView::deferredInitialization()
     {
         connect( header(), SIGNAL( sizeChange( int, int, int ) ), this, SLOT( saveColumnWidths() ) );
@@ -235,18 +275,23 @@ namespace KBibTeX
 
     void DocumentListView::deleteSelected()
     {
-        QListViewItemIterator it( this, QListViewItemIterator::Selected );
+        QListViewItemIterator it( this, QListViewItemIterator::Selected | QListViewItemIterator::Visible );
+        if ( it.current() == NULL ) return;
+
         QListViewItem *above = it.current() ->itemAbove();
+        QValueList<DocumentListViewItem*> toBeDeleted;
         while ( it.current() )
         {
             DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
-            if ( kblvi->isVisible() )
-            {
-                m_bibtexFile->deleteElement( kblvi->element() );
-                it++;
-                takeItem( kblvi );
-                delete( kblvi );
-            }
+            toBeDeleted.append( kblvi );
+            it++;
+        }
+
+        for ( QValueList<DocumentListViewItem*>::Iterator it = toBeDeleted.begin(); it != toBeDeleted.end(); ++it )
+        {
+            m_bibtexFile->deleteElement(( *it )->element() );
+            takeItem( *it );
+            delete( *it );
         }
 
         if ( above )
@@ -255,47 +300,84 @@ namespace KBibTeX
         emit modified();
     }
 
-    void DocumentListView::copySelected()
+    const QValueList<BibTeX::Element*> DocumentListView::selectedItems()
     {
-        kapp->clipboard() ->setText( selectedAsString() );
-    }
+        QValueList<BibTeX::Element*> result;
 
-    void DocumentListView::copyRefSelected()
-    {
-        QString refs;
         QListViewItemIterator it( this, QListViewItemIterator::Selected );
         while ( it.current() )
         {
             DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
-            BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( kblvi->element() );
-            if ( entry != NULL && kblvi->isVisible() )
-            {
-                if ( !refs.isEmpty() )
-                    refs.append( "," );
-                refs.append( entry->id() );
-            }
+            if ( kblvi->isVisible() )
+                result.append( kblvi->element() );
             it++;
         }
 
-        kapp->clipboard() ->setText( QString( "\\cite{%1}" ).arg( refs ) );
+        return result;
     }
 
-    void DocumentListView::cutSelected()
+    QString DocumentListView::selectedToBibTeXText()
     {
-        copySelected();
+        BibTeX::FileExporterBibTeX *exporter = new BibTeX::FileExporterBibTeX();
+        exporter->setEncoding( BibTeX::File::encLaTeX );
+
+        QBuffer buffer;
+        buffer.open( IO_WriteOnly );
+        QValueList<BibTeX::Element*> selectedElements = selectedItems();
+        for ( QValueList<BibTeX::Element*>::iterator it = selectedElements.begin(); it != selectedElements.end(); ++it )
+            exporter->save( &buffer, *it );
+        buffer.close();
+        delete exporter;
+
+        buffer.open( IO_ReadOnly );
+        QTextStream in( &buffer );
+        in.setEncoding( QTextStream::UnicodeUTF8 );
+        QString result = in.read();
+        buffer.close();
+
+        return result;
+    }
+
+    QString DocumentListView::selectedToBibTeXRefs()
+    {
+        QString refs;
+        QValueList<BibTeX::Element*> selectedElements = selectedItems();
+        for ( QValueList<BibTeX::Element*>::iterator it = selectedElements.begin(); it != selectedElements.end(); ++it )
+        {
+            BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( *it );
+            if ( entry == NULL ) continue;
+
+            if ( !refs.isEmpty() )
+                refs.append( "," );
+            refs.append( entry->id() );
+        }
+        return QString( "\\cite{%1}" ).arg( refs );
+    }
+
+    void DocumentListView::copy()
+    {
+        kapp->clipboard() ->setText( selectedToBibTeXText() );
+    }
+
+    void DocumentListView::copyReferences()
+    {
+        kapp->clipboard() ->setText( selectedToBibTeXRefs() );
+    }
+
+    void DocumentListView::cut()
+    {
+        copy();
         deleteSelected();
     }
 
     bool DocumentListView::paste()
     {
-        QListViewItem * lvi = selectedItem();
-        if ( lvi == NULL )
-            lvi = currentItem();
-        DocumentListViewItem * dlvi = NULL;
-        if ( lvi != NULL )
-            dlvi = dynamic_cast<KBibTeX::DocumentListViewItem *>( lvi );
+        DocumentListViewItem * dlvi =  dynamic_cast<KBibTeX::DocumentListViewItem *>( selectedItem() );
+        if ( dlvi == NULL )
+            dlvi = dynamic_cast<KBibTeX::DocumentListViewItem *>( currentItem() );
 
         QString clipboardText = kapp->clipboard() ->text();
+
         /** check if clipboard contains BibTeX content */
         if ( BibTeX::FileImporterBibTeX::guessCanDecode( clipboardText ) )
         {
@@ -304,15 +386,49 @@ namespace KBibTeX
             BibTeX::File *clipboardData = importer->load( clipboardText );
             delete importer;
 
-            if ( clipboardData != NULL && clipboardData->count() > 0 )
+            if ( clipboardData != NULL )
             {
-                bool result = insertItems( clipboardData, dlvi );
+                insertItems( clipboardData, dlvi );
                 delete clipboardData;
-                return result;
+                return TRUE;
             }
             else
                 return FALSE;
         }
+#ifdef HAVE_LIBBIBUTILS
+        else if ( BibTeX::FileImporterBibUtils::guessCanDecode( clipboardText ) )
+        {
+            BibTeX::FileImporterBibUtils::InputFormat inputFormat = BibTeX::FileImporterBibUtils::guessInputFormat( clipboardText );
+            BibTeX::FileImporter *importer = new BibTeX::FileImporterBibUtils( inputFormat );
+            BibTeX::File *clipboardData = importer->load( clipboardText );
+            delete importer;
+
+            if ( clipboardData != NULL )
+            {
+                insertItems( clipboardData, dlvi );
+                delete clipboardData;
+                return TRUE;
+            }
+            else
+                return FALSE;
+        }
+#else // HAVE_LIBBIBUTILS
+        else if ( BibTeX::FileImporterRIS::guessCanDecode( clipboardText ) )
+        {
+            BibTeX::FileImporter *importer = new BibTeX::FileImporterRIS( );
+            BibTeX::File *clipboardData = importer->load( clipboardText );
+            delete importer;
+
+            if ( clipboardData != NULL )
+            {
+                insertItems( clipboardData, dlvi );
+                delete clipboardData;
+                return TRUE;
+            }
+            else
+                return FALSE;
+        }
+#endif // HAVE_LIBBIBUTILS
         else
         {
             /** Decoding the paste text as bibtex failed. Maybe the user wants
@@ -373,141 +489,50 @@ namespace KBibTeX
         }
     }
 
-    void DocumentListView::sendSelectedToLyx()
+    void DocumentListView::selectAll()
     {
-        QStringList refsToSend;
-        QListViewItemIterator it( this, QListViewItemIterator::Selected );
-        while ( it.current() )
-        {
-            DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
-            BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( kblvi->element() );
-            if ( entry != NULL && kblvi->isVisible() )
-                refsToSend.append( entry->id() );
-            it++;
-        }
-
-        Settings * settings = Settings::self( m_bibtexFile );
-        QString lyxPipeFilename = settings->detectLyXInPipe();
-        kdDebug() << "sendSelectedToLyx: lyxPipeFilename= " << lyxPipeFilename << endl;
-        QFile pipe( lyxPipeFilename );
-        if ( pipe.exists() && pipe.open( IO_WriteOnly ) )
-        {
-            QTextStream * writer = new QTextStream( &pipe );
-            QString msg = "LYXCMD:kbibtex:citation-insert:" + refsToSend.join( "," );
-            *writer << msg << endl;
-            delete writer;
-            pipe.close();
-        }
-        else
-            KMessageBox::error( this, ( lyxPipeFilename.isEmpty() ? i18n( "Cannot establish a link to LyX" ) : QString( i18n( "Cannot establish a link to LyX using the pipe \"%1\"" ) ).arg( lyxPipeFilename ) ) + i18n( "\nMaybe LyX is not running?" ), i18n( "Error communicating with LyX" ) );
+        QListView::selectAll( true );
     }
 
+    /*        void DocumentListView::sendSelectedToLyx()
+        {
+            QStringList refsToSend;
+            QListViewItemIterator it( this, QListViewItemIterator::Selected );
+            while ( it.current() )
+            {
+                DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
+                BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( kblvi->element() );
+                if ( entry != NULL && kblvi->isVisible() )
+                    refsToSend.append( entry->id() );
+                it++;
+            }
+
+            Settings * settings = Settings::self( m_bibtexFile );
+            QString lyxPipeFilename = settings->detectLyXInPipe();
+            kdDebug() << "sendSelectedToLyx: lyxPipeFilename= " << lyxPipeFilename << endl;
+            QFile pipe( lyxPipeFilename );
+            if ( pipe.exists() && pipe.open( IO_WriteOnly ) )
+            {
+                QTextStream * writer = new QTextStream( &pipe );
+                QString msg = "LYXCMD:kbibtex:citation-insert:" + refsToSend.join( "," );
+                *writer << msg << endl;
+                delete writer;
+                pipe.close();
+            }
+            else
+                KMessageBox::error( this, ( lyxPipeFilename.isEmpty() ? i18n( "Cannot establish a link to LyX" ) : QString( i18n( "Cannot establish a link to LyX using the pipe \"%1\"" ) ).arg( lyxPipeFilename ) ) + i18n( "\nMaybe LyX is not running?" ), i18n( "Error communicating with LyX" ) );
+        }*/
+
     void DocumentListView::slotDoubleClick( QListViewItem *item )
     {
-        Settings * settings = Settings::self( m_bibtexFile );
-        bool openingDocumentOK = FALSE;
-
-        if ( settings->editing_MainListDoubleClickAction == 1 )
-        {
-            if ( item == NULL )
-                item = selectedItem();
-            if ( item == NULL )
-                item = currentItem();
-
-            DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( item );
-            if ( kblvi != NULL )
-            {
-                BibTeX::Element * element = kblvi->element();
-                BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( element );
-                if ( entry != NULL )
-                {
-                    QStringList urls = entry->urls();
-                    for ( QStringList::Iterator it = urls.begin(); it != urls.end(); ++it )
-                    {
-                        KURL url = KURL( *it );
-                        if ( url.isValid() && ( !url.isLocalFile() || QFile::exists( url.path() ) ) )
-                        {
-                            new KRun( url, this );
-                            openingDocumentOK = TRUE;
-                            break;
-                        }
-                    }
-                }
-            }
-        }
-
-        if ( !openingDocumentOK )
-            m_docWidget->editElement( );
+        DocumentListViewItem *dlvi = dynamic_cast<DocumentListViewItem*>( item );
+        if ( dlvi != NULL ) emit executed( dlvi );
     }
 
-
-    BibTeX::Element* DocumentListView::editElement( QListViewItem * item )
-    {
-        QDialog::DialogCode dialogResult = QDialog::Rejected;
-        BibTeX::Element* result = NULL;
-
-        if ( item == NULL )
-            item = selectedItem();
-        if ( item == NULL )
-            item = currentItem();
-
-        if ( item != NULL )
-        {
-            DocumentListViewItem * lvi = dynamic_cast<KBibTeX::DocumentListViewItem*>( item );
-            if ( lvi != NULL )
-            {
-                BibTeX::Entry * entry = dynamic_cast<BibTeX::Entry*>( lvi->element() );
-                if ( entry )
-                {
-                    dialogResult = KBibTeX::EntryWidget::execute( entry, m_bibtexFile, m_isReadOnly );
-                    if ( dialogResult == QDialog::Accepted )
-                        result = entry;
-                }
-                else
-                {
-                    BibTeX::Comment * comment = dynamic_cast<BibTeX::Comment*>( lvi->element() );
-                    if ( comment )
-                    {
-                        dialogResult = KBibTeX::CommentWidget::execute( comment, m_isReadOnly );
-                        if ( dialogResult == QDialog::Accepted )
-                            result = comment;
-                    }
-                    else
-                    {
-                        BibTeX::Macro* macro = dynamic_cast<BibTeX::Macro*>( lvi->element() );
-                        if ( macro )
-                        {
-                            dialogResult = KBibTeX::MacroWidget::execute( macro, m_isReadOnly );
-                            if ( dialogResult == QDialog::Accepted )
-                                result = macro;
-                        }
-                        else
-                        {
-                            BibTeX::Preamble* preamble = dynamic_cast<BibTeX::Preamble*>( lvi->element() );
-                            if ( preamble )
-                            {
-                                dialogResult = KBibTeX::PreambleWidget::execute( preamble, m_isReadOnly );
-                                if ( dialogResult == QDialog::Accepted )
-                                    result = preamble;
-                            }
-                        }
-                    }
-                }
-
-                if ( dialogResult == QDialog::Accepted )
-                    lvi->updateItem();
-            }
-        }
-
-        if ( dialogResult == QDialog::Accepted )
-            emit modified();
-
-        return result;
-    }
-
-    void DocumentListView::filterText( const QString & text, BibTeX::EntryField::FieldType fieldType )
+    void DocumentListView::filter( const QString & text, BibTeX::Element::FilterType filterType, BibTeX::EntryField::FieldType fieldType )
     {
         m_filter = text;
+        m_filterType = filterType;
         m_filterFieldType = fieldType;
         updateVisiblity();
     }
@@ -550,9 +575,16 @@ namespace KBibTeX
 
     bool DocumentListView::acceptDrag( QDropEvent * event ) const
     {
+        if ( event->source() == this ) return false;
         return QTextDrag::canDecode( event ) || QUriDrag::canDecode( event );
     }
 
+    void DocumentListView::startDrag()
+    {
+        Settings * settings = Settings::self( m_bibtexFile );
+        QDragObject *d = new QTextDrag( settings->editing_DragAction == Settings::COPYREFERENCE ? selectedToBibTeXRefs() : selectedToBibTeXText(), this );
+        d->dragCopy();
+    }
     void DocumentListView::saveColumnIndex()
     {
         Settings * settings = Settings::self( m_bibtexFile );
@@ -688,32 +720,6 @@ namespace KBibTeX
     //     // do NOT delete d.
     // }
 
-    QString DocumentListView::selectedAsString()
-    {
-        QBuffer buffer;
-        buffer.open( IO_WriteOnly );
-        BibTeX::FileExporterBibTeX * exporter = new BibTeX::FileExporterBibTeX();
-        exporter->setEncoding( BibTeX::File::encLaTeX );
-        QListViewItemIterator it( this, QListViewItemIterator::Selected );
-        while ( it.current() )
-        {
-            DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
-            if ( kblvi->isVisible() )
-                exporter->save( &buffer, kblvi->element() );
-            it++;
-        }
-        delete exporter;
-        buffer.close();
-
-        buffer.open( IO_ReadOnly );
-        QTextStream in( &buffer );
-        in.setEncoding( QTextStream::UnicodeUTF8 );
-        QString result = in.read();
-        buffer.close();
-
-        return result;
-    }
-
     bool DocumentListView::eventFilter( QObject * watched, QEvent * e )
     {
         if ( watched == header() )
@@ -739,7 +745,13 @@ namespace KBibTeX
     void DocumentListView::keyPressEvent( QKeyEvent *e )
     {
         if ( e->key() == QKeyEvent::Key_Enter || e->key() == QKeyEvent::Key_Return )
-            slotDoubleClick();
+        {
+            DocumentListViewItem *item = dynamic_cast<DocumentListViewItem*>( selectedItem() );
+            if ( item == NULL )
+                item = dynamic_cast<DocumentListViewItem*>( currentItem() );
+            if ( item != NULL )
+                emit executed( item );
+        }
         else
             KListView::keyPressEvent( e );
     }
