@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -21,7 +21,7 @@
 #include <qlayout.h>
 #include <qfile.h>
 
-#include <kparts/factory.h>
+#include <klibloader.h>
 #include <kxmlguifactory.h>
 #include <kapplication.h>
 #include <klibloader.h>
@@ -46,9 +46,9 @@ namespace KBibTeX
 {
 
     DocumentSourceView::DocumentSourceView( KBibTeX::DocumentWidget *docWidget, bool isReadOnly, QWidget *parent, const char *name )
-            : QWidget( parent, name ), m_docWidget( docWidget ), m_bibtexFile( NULL ), m_document( NULL ), m_view( NULL )
+            : QWidget( parent, name ), m_isReadOnly( isReadOnly ), m_docWidget( docWidget ), m_bibtexFile( NULL ), m_document( NULL ), m_view( NULL )
     {
-        setupGUI( isReadOnly );
+        setupGUI();
 
         readConfig();
     }
@@ -56,13 +56,11 @@ namespace KBibTeX
     DocumentSourceView::~DocumentSourceView()
     {
         kapp->config() ->sync();
-
-        if ( m_view->document() ->views().count() == 1 )
-        {
-            delete m_view->document();
-        }
     }
 
+    /**
+     * Copy the selected text to the clipboard
+     */
     void DocumentSourceView::copy()
     {
         KTextEditor::ClipboardInterface * clip = dynamic_cast<KTextEditor::ClipboardInterface*>( m_view );
@@ -70,6 +68,9 @@ namespace KBibTeX
             clip->copy();
     }
 
+    /**
+     * Cut the selected text to the clipboard
+     */
     void DocumentSourceView::cut()
     {
         KTextEditor::ClipboardInterface * clip = dynamic_cast<KTextEditor::ClipboardInterface*>( m_view );
@@ -77,6 +78,9 @@ namespace KBibTeX
             clip->cut();
     }
 
+    /**
+     * Paste the clipboard's content into the document
+     */
     bool DocumentSourceView::paste()
     {
         KTextEditor::ClipboardInterface * clip = dynamic_cast<KTextEditor::ClipboardInterface*>( m_view );
@@ -89,6 +93,9 @@ namespace KBibTeX
             return FALSE;
     }
 
+    /**
+     * Paste a line of text into the document at the designated position
+     */
     void DocumentSourceView::insertLines( const QString& text, int line )
     {
         m_editInterface->insertLine( line < 0 ? m_editInterface->numLines() : ( uint ) line, text );
@@ -98,9 +105,17 @@ namespace KBibTeX
             cursorIf->setCursorPosition( line < 0 ? m_editInterface->numLines() - 1 : ( uint ) line, 0 );
     }
 
+    void DocumentSourceView::selectAll()
+    {
+        KTextEditor::SelectionInterface * selI = dynamic_cast<KTextEditor::SelectionInterface*>( m_view );
+        if ( selI )
+            selI->selectAll();
+    }
+
     void DocumentSourceView::setReadOnly( bool isReadOnly )
     {
-        m_document->setReadWrite( !isReadOnly );
+        m_isReadOnly = isReadOnly;
+        m_document->setReadWrite( !m_isReadOnly );
     }
 
     void DocumentSourceView::setFactory( KXMLGUIFactory *factory, KXMLGUIClient * client )
@@ -202,6 +217,7 @@ namespace KBibTeX
         exporter->setStringDelimiter( settings->fileIO_BibtexStringOpenDelimiter, settings->fileIO_BibtexStringCloseDelimiter );
         exporter->setKeywordCasing( settings->fileIO_KeywordCasing );
         exporter->setEncoding(( BibTeX::File::Encoding ) settings->fileIO_Encoding );
+        exporter->setEnclosingCurlyBrackets( settings->fileIO_EnclosingCurlyBrackets );
         result = exporter->save( &buffer, bibtexFile );
         delete exporter;
         buffer.close();
@@ -215,7 +231,13 @@ namespace KBibTeX
             buffer.close();
 
             if ( m_editInterface )
+            {
+                // very strange: to set the text, you have to set
+                // readwrite to TRUE...
+                m_document->setReadWrite( TRUE );
                 m_editInterface->setText( text );
+                m_document->setReadWrite( !m_isReadOnly );
+            }
 
             m_bibtexFile = bibtexFile;
         }
@@ -255,41 +277,29 @@ namespace KBibTeX
             m_view->setFocus();
     }
 
-    void DocumentSourceView::setupGUI( bool isReadOnly )
+    void DocumentSourceView::setupGUI()
     {
-        // This returns 0, if the library can not be found
-        KParts::Factory * factory = ( KParts::Factory * ) KLibLoader::self() ->factory( "libkatepart" );
+        // create the Kate::Document
+        m_document = Kate::createDocument( this, "Kate::Document" );
 
-        if ( factory )
-            m_document = ( Kate::Document* ) KTextEditor::createDocument( "libkatepart", this, "Kate::Document" );
+        m_view = ( Kate::View * ) m_document->createView( this, 0L );
+        m_editInterface = editInterface( m_document );
+        m_document->setReadWrite( !m_isReadOnly );
+        if ( !m_isReadOnly )
+            connect( m_document, SIGNAL( textChanged() ), this, SIGNAL( modified() ) );
+        QBoxLayout * layout = new QVBoxLayout( this );
+        layout->addWidget( static_cast<QWidget*>( m_view ) );
 
-        if ( m_document )
-        {
-            m_view = ( Kate::View * ) m_document->createView( this, 0L );
-            m_editInterface = editInterface( m_document );
-            m_document->setReadWrite( !isReadOnly );
-            if ( !isReadOnly )
-                connect( m_document, SIGNAL( textChanged() ), this, SIGNAL( modified() ) );
-        }
-        else
-            KMessageBox::error( this, i18n( "A KDE text-editor component could not be found;\nplease check your KDE installation." ) );
-
-        if ( m_view )
-        {
-            QBoxLayout * layout = new QVBoxLayout( this );
-            layout->addWidget( static_cast<QWidget*>( m_view ) );
-
-            int c = m_document->hlModeCount() ;
-            int hlIdx = -1;
-            for ( int i = 0; i < c; i++ )
-                if ( m_document->hlModeName( i ) .compare( "BibTeX" ) == 0 )
-                {
-                    hlIdx = i;
-                    break;
-                }
-            if ( hlIdx > -1 )
-                m_document-> setHlMode( hlIdx );
-        }
+        int c = m_document->hlModeCount() ;
+        int hlIdx = -1;
+        for ( int i = 0; i < c; i++ )
+            if ( m_document->hlModeName( i ) .compare( "BibTeX" ) == 0 )
+            {
+                hlIdx = i;
+                break;
+            }
+        if ( hlIdx > -1 )
+            m_document-> setHlMode( hlIdx );
     }
 
     void DocumentSourceView::readConfig()
@@ -346,7 +356,7 @@ namespace KBibTeX
             fromLine = 0;
             fromCol = 0;
         }
-        while ( KMessageBox::questionYesNo( this, QString( i18n( "Could not find text \"%1\" in the document.\nStart from the beginning?" ) ).arg( m_lastSearchTerm ), i18n( "Find text in source view" ), KGuiItem( i18n( "Restart search" ) ) ) == KMessageBox::Yes );
+        while ( KMessageBox::questionYesNo( this, QString( i18n( "Could not find text '%1' in the document.\nStart from the beginning?" ) ).arg( m_lastSearchTerm ), i18n( "Find text in source view" ), KGuiItem( i18n( "Restart search" ) ) ) == KMessageBox::Yes );
     }
 }
 #include "documentsourceview.moc"
