@@ -24,14 +24,17 @@
 
 #include <file.h>
 
-#include <kate/view.h>
 #include <ktexteditor/editinterface.h>
+#include <kate/view.h>
 #include <kate/document.h>
 
 namespace KBibTeX
 {
     class DocumentWidget;
 
+    /**
+     * @author Thomas Fischer <fischer@unix-ag.uni-kl.de>
+     */
     class DocumentSourceView : public QWidget
     {
         Q_OBJECT
@@ -46,6 +49,7 @@ namespace KBibTeX
         void cut();
         bool paste();
         void insertLines( const QString& text, int line = -1 );
+        void selectAll();
 
         void setReadOnly( bool isReadOnly );
         void setFactory( KXMLGUIFactory *factory, KXMLGUIClient *client );
@@ -63,6 +67,7 @@ namespace KBibTeX
         virtual void focusInEvent( QFocusEvent* event );
 
     private:
+        bool m_isReadOnly;
         KBibTeX::DocumentWidget *m_docWidget;
         BibTeX::File *m_bibtexFile;
         Kate::Document* m_document;
@@ -71,7 +76,7 @@ namespace KBibTeX
         QString m_lastSearchTerm;
         QStringList m_findHistory;
 
-        void setupGUI( bool isReadOnly );
+        void setupGUI();
         void readConfig();
         void writeConfig();
         void readConfig( KConfig *config );
