@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -33,6 +33,7 @@
 #include <qpushbutton.h>
 
 #include <krun.h>
+#include <kdebug.h>
 #include <klocale.h>
 #include <ktempfile.h>
 #include <kglobalsettings.h>
@@ -95,11 +96,14 @@ namespace KBibTeX
 {
 
     DocumentWidget::DocumentWidget( bool isReadOnly, QWidget *parent, const char *name )
-            : QTabWidget( parent, name ), m_lineEditNewKeyword( NULL ), m_isReadOnly( isReadOnly ), m_filename( QString::null ), m_progressDialog( NULL ), m_newElementCounter( 1 ), m_editMode( emList ), m_viewDocumentActionMenu( NULL ), m_assignKeywordsActionMenu( NULL ), m_searchWebsitesActionMenu( NULL ), m_find( NULL ), m_findNext( NULL ), m_copyReference( NULL ), m_dirWatch( this )
+            : QTabWidget( parent, name ), m_lineEditNewKeyword( NULL ), m_isReadOnly( isReadOnly ), m_filename( QString::null ), m_progressDialog( NULL ), m_newElementCounter( 1 ), m_editMode( emList ), m_viewDocumentActionMenu( NULL ), m_assignKeywordsActionMenu( NULL ), m_searchWebsitesActionMenu( NULL ), m_actionEditCut( NULL ), m_actionEditCopy( NULL ), m_actionEditCopyRef( NULL ), m_actionEditPaste( NULL ), m_actionEditSelectAll( NULL ), m_actionEditFind( NULL ), m_actionEditFindNext( NULL ), m_dirWatch( this )
     {
         m_bibtexfile = new BibTeX::File();
 
         setupGUI();
+
+        m_listViewElements->setBibTeXFile( m_bibtexfile );
+        m_sourceView->setBibTeXFile( m_bibtexfile );
     }
 
     DocumentWidget::~DocumentWidget()
@@ -125,7 +129,7 @@ namespace KBibTeX
         layout->addWidget( m_horSplitter );
         m_horSplitter->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding );
 
-        m_sideBar = new SideBar( m_horSplitter );
+        m_sideBar = new SideBar( m_isReadOnly, m_horSplitter );
         m_vertSplitter = new QSplitter( Qt::Vertical, m_horSplitter );
         m_vertSplitter->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding );
 
@@ -142,7 +146,9 @@ namespace KBibTeX
         m_sourceView->setFont( KGlobalSettings::fixedFont() );
 
         // Setup actions
-        connect( m_searchBar, SIGNAL( doSearch( const QString&, BibTeX::EntryField::FieldType ) ), m_listViewElements, SLOT( filterText( const QString&, BibTeX::EntryField::FieldType ) ) );
+        connect( m_searchBar, SIGNAL( onlineSearch() ), this, SLOT( onlineSearch() ) );
+        connect( m_searchBar, SIGNAL( doSearch( const QString&, BibTeX::Element::FilterType, BibTeX::EntryField::FieldType ) ), m_listViewElements, SLOT( filter( const QString&, BibTeX::Element::FilterType, BibTeX::EntryField::FieldType ) ) );
+        connect( m_listViewElements, SIGNAL( executed( DocumentListViewItem* ) ), this, SLOT( executeElement( DocumentListViewItem* ) ) );
         connect( m_listViewElements, SIGNAL( selectionChanged() ), this, SLOT( slotSelectionChanged() ) );
         connect( m_listViewElements, SIGNAL( currentChanged( QListViewItem* ) ), this, SLOT( slotPreviewElement( QListViewItem* ) ) );
         connect( m_listViewElements, SIGNAL( clicked( QListViewItem* ) ), this, SLOT( slotPreviewElement( QListViewItem* ) ) );
@@ -151,7 +157,7 @@ namespace KBibTeX
         connect( m_listViewElements, SIGNAL( modified() ), this, SLOT( slotModified() ) );
         connect( m_sideBar, SIGNAL( valueRenamed() ), this, SLOT( slotModified() ) );
         connect( m_sideBar, SIGNAL( valueRenamed() ), this, SLOT( refreshBibTeXFile() ) );
-        connect( m_sideBar, SIGNAL( selected( const QString& , BibTeX::EntryField::FieldType ) ), m_searchBar, SLOT( setSearch( const QString& , BibTeX::EntryField::FieldType ) ) );
+        connect( m_sideBar, SIGNAL( selected( const QString&, BibTeX::Element::FilterType, BibTeX::EntryField::FieldType ) ), m_searchBar, SLOT( setSearch( const QString&, BibTeX::Element::FilterType, BibTeX::EntryField::FieldType ) ) );
         connect( &m_dirWatch, SIGNAL( dirty( const QString& ) ), this, SLOT( slotFileGotDirty( const QString & ) ) );
     }
 
@@ -161,66 +167,31 @@ namespace KBibTeX
         bool result = FALSE;
         if ( !mergeOnly )
             m_dirWatch.removeFile( m_filename );
-        m_filename = fileName;
+
+        BibTeX::FileImporter *importer = fileImporterFactory( fileName );
+        if ( importer != NULL )
+        {
+            QFile file( fileName );
+            if ( file.open( IO_ReadOnly ) )
+            {
+                result = open( &file, mergeOnly, QString( i18n( "<qt>Loading file <b>%1</b></qt>" ) ).arg( fileName ), importer );
+                if ( result )
+                {
+                    m_bibtexfile->fileName = fileName;
+                    m_filename = fileName;
+                }
+                file.close();
+            }
+            else
+                kdDebug() << "Cannot open file " << fileName << endl;
+
+            delete importer;
+        }
+
+
         if ( !mergeOnly )
             m_dirWatch.addFile( m_filename );
 
-        BibTeX::File::FileFormat format = BibTeX::File::formatUndefined;
-        if ( fileName.endsWith( ".bib" ) )
-            format = BibTeX::File::formatBibTeX;
-#ifdef HAVE_LIBBIBUTILS
-        else if ( fileName.endsWith( ".enl" ) )
-            format = BibTeX::File::formatEndNote;
-#endif // HAVE_LIBBIBUTILS
-        else if ( fileName.endsWith( ".ris" ) )
-            format = BibTeX::File::formatRIS;
-
-        if ( format != BibTeX::File::formatUndefined )
-        {
-            BibTeX::FileImporter * importer = NULL;
-            switch ( format )
-            {
-            case BibTeX::File::formatBibTeX:
-                {
-                    Settings * settings = Settings::self( NULL );
-                    importer = new BibTeX::FileImporterBibTeX( settings->editing_FirstNameFirst );
-                }
-                break;
-#ifdef HAVE_LIBBIBUTILS
-            case BibTeX::File::formatEndNote:
-                importer = new BibTeX::FileImporterBibUtils( BibTeX::FileImporterBibUtils::ifEndNote );
-                break;
-            case BibTeX::File::formatRIS:
-                importer = new BibTeX::FileImporterBibUtils( BibTeX::FileImporterBibUtils::ifRIS );
-                break;
-#else
-            case BibTeX::File::formatRIS:
-                importer = new BibTeX::FileImporterRIS( );
-                break;
-#endif
-            default:
-                {
-// nothing
-                }
-            }
-
-            if ( importer != NULL )
-            {
-                QFile file( fileName );
-                if ( file.open( IO_ReadOnly ) )
-                {
-                    result = open( &file, mergeOnly, QString( i18n( "<qt>Loading file <b>%1</b></qt>" ) ).arg( fileName ), importer );
-                    if ( result )
-                        m_bibtexfile->fileName = fileName;
-                    file.close();
-                }
-                delete importer;
-            }
-        }
-        else
-            KMessageBox::sorry( this, QString( i18n( "The file ‘%1’ does not contain a known type of bibliography." ) ).arg( fileName ), i18n( "Unknown file format" ) );
-
-
         return result;
     }
 
@@ -281,8 +252,6 @@ namespace KBibTeX
     {
         bool result = FALSE;
         m_dirWatch.removeFile( m_filename );
-        m_filename = fileName;
-        m_dirWatch.addFile( m_filename );
 
         BibTeX::File::FileFormat format = BibTeX::File::formatUndefined;
         if ( fileName.endsWith( ".rtf", FALSE ) )
@@ -292,8 +261,10 @@ namespace KBibTeX
         else if ( fileName.endsWith( ".bib", FALSE ) )
             format = BibTeX::File::formatBibTeX;
 #ifdef HAVE_LIBBIBUTILS
-        else if ( fileName.endsWith( ".enl", FALSE ) )
+        else if ( fileName.endsWith( ".ref", FALSE ) || fileName.endsWith( ".refer", FALSE ) || fileName.endsWith( ".txt", FALSE ) || fileName.endsWith( ".rfr", FALSE ) )
             format = BibTeX::File::formatEndNote;
+        else if ( fileName.endsWith( ".isi", FALSE ) || fileName.endsWith( ".cgi", FALSE ) )
+            format = BibTeX::File::formatISI;
 #endif
         else if ( fileName.endsWith( ".ris", FALSE ) )
             format = BibTeX::File::formatRIS;
@@ -303,23 +274,27 @@ namespace KBibTeX
             format = BibTeX::File::formatXML;
         else if ( fileName.endsWith( ".html", FALSE ) || fileName.endsWith( ".xhtml", FALSE ) || fileName.endsWith( ".htm", FALSE ) )
             format = BibTeX::File::formatHTML;
-        if ( format == BibTeX::File::formatUndefined )
-        {
-            kdDebug() << "Unknown file format to save to" << endl;
-            return FALSE;
-        }
 
-        m_dirWatch.stopScan();
-        QFile file( fileName );
-        if ( file.open( IO_WriteOnly ) )
+        if ( format != BibTeX::File::formatUndefined )
         {
-            result = save( &file, format, QString( i18n( "<qt>Writing file <b>%1</b></qt>" ) ).arg( fileName ), errorLog );
-            if ( result )
-                m_bibtexfile->fileName = fileName;
-            file.close();
+            QFile file( fileName );
+            if ( file.open( IO_WriteOnly ) )
+            {
+                result = save( &file, format, QString( i18n( "<qt>Writing file <b>%1</b></qt>" ) ).arg( fileName ), errorLog );
+                if ( result )
+                {
+                    m_bibtexfile->fileName = fileName;
+                    m_filename = fileName;
+                }
+                file.close();
+            }
+            else
+                kdDebug() << "Cannot write to file " << fileName << endl;
         }
-        m_dirWatch.startScan();
+        else
+            kdDebug() << "Unknown file format to save to (filename is " << fileName << ")" << endl;
 
+        m_dirWatch.addFile( m_filename );
         return result;
     }
 
@@ -343,6 +318,7 @@ namespace KBibTeX
                 bibtexExporter->setStringDelimiter( settings->fileIO_BibtexStringOpenDelimiter, settings->fileIO_BibtexStringCloseDelimiter );
                 bibtexExporter->setKeywordCasing( settings->fileIO_KeywordCasing );
                 bibtexExporter->setEncoding( settings->fileIO_Encoding );
+                bibtexExporter->setEnclosingCurlyBrackets( settings->fileIO_EnclosingCurlyBrackets );
                 exporter = bibtexExporter;
             }
             break;
@@ -353,6 +329,9 @@ namespace KBibTeX
         case BibTeX::File::formatEndNote:
             exporter = new BibTeX::FileExporterBibUtils( BibTeX::FileExporterBibUtils::ofEndNote );
             break;
+        case BibTeX::File::formatISI:
+            exporter = new BibTeX::FileExporterBibUtils( BibTeX::FileExporterBibUtils::ofISI );
+            break;
 #else
         case BibTeX::File::formatRIS:
             exporter = new BibTeX::FileExporterRIS( );
@@ -406,7 +385,7 @@ namespace KBibTeX
             {
                 if ( settings->fileIO_EmbedFiles && !Settings::kpsewhich( "embedfile.sty" ) )
                 {
-                    KMessageBox::sorry( this, i18n( "Embedding files into the PDF file is enabled, but the required file \"embedfile.sty\" was not found. Embedding files will be disabled." ), i18n( "Embedding files disabled" ) );
+                    KMessageBox::sorry( this, i18n( "Embedding files into the PDF file is enabled, but the required file 'embedfile.sty' was not found. Embedding files will be disabled." ), i18n( "Embedding files disabled" ) );
                     settings->fileIO_EmbedFiles = FALSE;
                 }
 
@@ -456,7 +435,7 @@ namespace KBibTeX
         {
             if ( elementType.lower() == "macro" )
             {
-                QString name = QString( i18n( "NewMacro_%1" ) ).arg( m_newElementCounter++ );
+                QString name = QString( i18n( "May only contain ASCII characters, in case of doubt keep English form", "NewMacro%1" ) ).arg( m_newElementCounter++ );
                 BibTeX::Macro *macro = new BibTeX::Macro( name );
                 if ( MacroWidget::execute( macro, m_isReadOnly ) == QDialog::Accepted )
                 {
@@ -504,10 +483,10 @@ namespace KBibTeX
             }
             else
             {
-                QString name = QString( i18n( "NewEntry_%1" ) ).arg( m_newElementCounter++ );
+                QString name = nextNewEntry();
                 BibTeX::Entry *entry = new BibTeX::Entry( elementType, name );
                 m_dirWatch.stopScan();
-                if ( EntryWidget::execute( entry, m_bibtexfile, m_isReadOnly ) == QDialog::Accepted )
+                if ( EntryWidget::execute( entry, m_bibtexfile, m_isReadOnly, TRUE ) == QDialog::Accepted )
                 {
                     new DocumentListViewItem( m_bibtexfile, entry, m_listViewElements );
                     m_bibtexfile->appendElement( entry );
@@ -528,16 +507,20 @@ namespace KBibTeX
         {
             if ( elementType.lower() == "macro" )
             {
-                QString name = QString( i18n( "NewString_%1" ) ).arg( m_newElementCounter++ );
+                QString name = QString( i18n( "May only contain ASCII characters, in case of doubt keep English form", "NewString%1" ) ).arg( m_newElementCounter++ );
                 m_sourceView->insertLines( QString( "@string{ %1 = \"%2\" }" ).arg( name ).arg( i18n( "No text yet" ) ) );
             }
             else if ( elementType.lower() == "comment" )
             {
                 m_sourceView->insertLines( i18n( "@comment{ Put your comment here... }" ) );
             }
+            else if ( elementType.lower() == "preamble" )
+            {
+                m_sourceView->insertLines( i18n( "@preamble{\"Put your preamble here using double quotes...\"}" ), 0 );
+            }
             else
             {
-                QString name = QString( i18n( "NewEntry_%1" ) ).arg( m_newElementCounter++ );
+                QString name = nextNewEntry();
                 BibTeX::Entry *entry = new BibTeX::Entry( elementType, name );
 
                 for ( int t = 0; t < 2; t++ )
@@ -590,7 +573,8 @@ namespace KBibTeX
 
     void DocumentWidget::showStatistics()
     {
-        KMessageBox::information( this, QString( i18n( "This BibTeX file contains %1 elements." ) ).arg( m_bibtexfile->count() ), i18n( "File Statistics" ) );
+        int n = m_bibtexfile->count();
+        KMessageBox::information( this, i18n( "This BibTeX file contains 1 element.", "This BibTeX file contains %n elements.", n ), i18n( "File Statistics" ) );
     }
 
     void DocumentWidget::refreshBibTeXFile()
@@ -612,20 +596,25 @@ namespace KBibTeX
         if ( m_assignKeywordsActionMenu != NULL )
             connect( m_assignKeywordsActionMenu->popupMenu(), SIGNAL( activated( int ) ), this, SLOT( slotAssignKeywords( int ) ) );
 
-        m_copyReference = client->action( "edit_copyref" );
-        m_find = client->action( "edit_find" );
-        m_findNext = client->action( "edit_find_next" );
+        m_actionEditCut = client->action( "edit_cut" );
+        m_actionEditCopy = client->action( "edit_copy" );
+        m_actionEditCopyRef = client->action( "edit_copyref" );
+        m_actionEditPaste = client->action( "edit_paste" );
+        m_actionEditSelectAll = client->action( "edit_select_all" );
+        m_actionEditFind = client->action( "edit_find" );
+        m_actionEditFindNext = client->action( "edit_find_next" );
         m_listViewElements->setViewShowColumnsMenu( dynamic_cast<KActionMenu*>( client->action( "view_showcolumns" ) ) );
         m_searchWebsitesActionMenu = dynamic_cast<KActionMenu*>( client->action( "search_document_online" ) );
     }
 
-    void DocumentWidget::setOnlineDatabaseSearchMenu( KPopupMenu* popupMenu )
-    {
-        m_searchBar->setOnlineDatabaseSearchMenu( popupMenu );
-    }
-
     void DocumentWidget::updateViewDocumentMenu( )
     {
+        if ( m_viewDocumentActionMenu == NULL )
+        {
+            kdDebug() << "FIXME: m_viewDocumentActionMenu is not set" << endl;
+            return;
+        }
+
         KPopupMenu * popup = m_viewDocumentActionMenu->popupMenu();
         popup->clear();
         m_viewDocumentActionMenuURLs.clear();
@@ -652,7 +641,7 @@ namespace KBibTeX
                 for ( KURL::List::Iterator i = documentURLs.begin(); i != documentURLs.end(); ++i )
                 {
                     QString prettyURL = ( *i ).prettyURL();
-                    if ( prettyURL.endsWith( ".pdf", FALSE ) )
+                    if ( prettyURL.endsWith( ".pdf", FALSE ) || prettyURL.find( "/pdf/" ) > 0 )
                         popup->insertItem( SmallIcon( "pdf" ), prettyURL );
                     else if ( prettyURL.endsWith( ".ps", FALSE ) )
                         popup->insertItem( SmallIcon( "postscript" ), prettyURL );
@@ -669,6 +658,12 @@ namespace KBibTeX
 
     void DocumentWidget::updateAssignKeywords()
     {
+        if ( m_assignKeywordsActionMenu == NULL )
+        {
+            kdDebug() << "FIXME: m_assignKeywordsActionMenu is not set" << endl;
+            return;
+        }
+
         KPopupMenu * popup = m_assignKeywordsActionMenu->popupMenu();
         popup->clear();
         m_assignKeywordsActionMenuURLs.clear();
@@ -698,7 +693,7 @@ namespace KBibTeX
         /**
         * Fetch all keywords from current file into fileKeywords
         */
-        for ( QValueList<BibTeX::Element*>::ConstIterator eit = m_bibtexfile->constBegin();eit != m_bibtexfile->constEnd();++eit )
+        for ( QValueList<BibTeX::Element*>::ConstIterator eit = m_bibtexfile->constBegin(); eit != m_bibtexfile->constEnd(); ++eit )
         {
             BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( *eit );
             BibTeX::EntryField *field = NULL;
@@ -834,10 +829,55 @@ namespace KBibTeX
 
     }
 
+    void DocumentWidget::executeElement( DocumentListViewItem* item )
+    {
+        Settings * settings = Settings::self( m_bibtexfile );
+        bool openingDocumentOK = FALSE;
+
+        if ( settings->editing_MainListDoubleClickAction == 1 )
+        {
+            BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( item->element() );
+            if ( entry != NULL )
+            {
+                QStringList urls = entry->urls();
+                for ( QStringList::Iterator it = urls.begin(); it != urls.end(); ++it )
+                {
+                    KURL url = KURL( *it );
+                    if ( url.isValid() && ( !url.isLocalFile() || QFile::exists( url.path() ) ) )
+                    {
+                        new KRun( url, this );
+                        openingDocumentOK = TRUE;
+                        break;
+                    }
+                }
+            }
+        }
+
+        if ( !openingDocumentOK )
+            editElement( item );
+    }
+
     void DocumentWidget::editElement()
+    {
+        QListViewItem * item = m_listViewElements->selectedItem();
+        if ( item == NULL )
+            item = m_listViewElements->currentItem();
+        DocumentListViewItem * dlvi = dynamic_cast<DocumentListViewItem*>( item );
+        if ( dlvi )
+            editElement( dlvi );
+    }
+
+    void DocumentWidget::editElement( DocumentListViewItem*item )
+    {
+        BibTeX::Element * element = item->element();
+        editElement( element );
+        item->updateItem();
+    }
+
+    void DocumentWidget::editElement( BibTeX::Element*element )
     {
         m_dirWatch.stopScan();
-        BibTeX::Element * element = m_listViewElements->editElement( );
+        editElementDialog( element );
         m_dirWatch.startScan( );
 
         Settings * settings = Settings::self( m_bibtexfile );
@@ -845,6 +885,37 @@ namespace KBibTeX
         m_sideBar->refreshLists( m_bibtexfile );
     }
 
+    bool DocumentWidget::editElementDialog( BibTeX::Element *element )
+    {
+        QDialog::DialogCode dialogResult = QDialog::Rejected;
+        BibTeX::Entry * entry = dynamic_cast<BibTeX::Entry*>( element );
+        if ( entry )
+            dialogResult = KBibTeX::EntryWidget::execute( entry, m_bibtexfile, m_isReadOnly, FALSE );
+        else
+        {
+            BibTeX::Comment * comment = dynamic_cast<BibTeX::Comment*>( element );
+            if ( comment )
+                dialogResult = KBibTeX::CommentWidget::execute( comment, m_isReadOnly );
+            else
+            {
+                BibTeX::Macro* macro = dynamic_cast<BibTeX::Macro*>( element );
+                if ( macro )
+                    dialogResult = KBibTeX::MacroWidget::execute( macro, m_isReadOnly );
+                else
+                {
+                    BibTeX::Preamble* preamble = dynamic_cast<BibTeX::Preamble*>( element );
+                    if ( preamble )
+                        dialogResult = KBibTeX::PreambleWidget::execute( preamble, m_isReadOnly );
+                }
+            }
+        }
+
+        if ( dialogResult == QDialog::Accepted )
+            slotModified();
+
+        return dialogResult == QDialog::Accepted;
+    }
+
     void DocumentWidget::deleteElements()
     {
         if ( !m_isReadOnly )
@@ -860,7 +931,32 @@ namespace KBibTeX
     void DocumentWidget::sendSelectedToLyx()
     {
         if ( m_editMode == emList )
-            m_listViewElements->sendSelectedToLyx();
+        {
+            QValueList<BibTeX::Element*> elements = m_listViewElements->selectedItems();
+            QStringList refsToSend;
+
+            for ( QValueList<BibTeX::Element*>::Iterator it = elements.begin(); it != elements.end(); ++it )
+            {
+                BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( *it );
+                if ( entry != NULL ) refsToSend.append( entry->id() );
+            }
+
+            Settings * settings = Settings::self( m_bibtexfile );
+            QString lyxPipeFilename = settings->detectLyXInPipe();
+            kdDebug() << "sendSelectedToLyx: lyxPipeFilename= " << lyxPipeFilename << endl;
+            QFile pipe( lyxPipeFilename );
+            if ( pipe.exists() && pipe.open( IO_WriteOnly ) )
+            {
+                QTextStream * writer = new QTextStream( &pipe );
+                QString msg = "LYXCMD:kbibtex:citation-insert:" + refsToSend.join( "," );
+                *writer << msg << endl;
+                delete writer;
+                pipe.close();
+            }
+            else
+                KMessageBox::error( this, ( lyxPipeFilename.isEmpty() ? i18n( "Cannot establish a link to LyX" ) : QString( i18n( "Cannot establish a link to LyX using the pipe '%1'" ) ).arg( lyxPipeFilename ) ) + i18n( "\nMaybe LyX is not running?" ), i18n( "Error communicating with LyX" ) );
+
+        }
     }
 
     void DocumentWidget::cutElements()
@@ -869,7 +965,7 @@ namespace KBibTeX
         {
             if ( m_editMode == emList )
             {
-                m_listViewElements->cutSelected();
+                m_listViewElements->cut();
             }
             else if ( m_editMode == emSource )
             {
@@ -883,7 +979,7 @@ namespace KBibTeX
     {
         if ( m_editMode == emList )
         {
-            m_listViewElements->copySelected();
+            m_listViewElements->copy();
         }
         else if ( m_editMode == emSource )
         {
@@ -895,7 +991,11 @@ namespace KBibTeX
     {
         if ( m_editMode == emList )
         {
-            m_listViewElements->copyRefSelected();
+            m_listViewElements->copyReferences();
+        }
+        else  if ( m_editMode == emSource )
+        {
+// TODO: Get selected text, parse text into BibTeX::File and join list of entries' ids
         }
     }
 
@@ -916,11 +1016,20 @@ namespace KBibTeX
         }
     }
 
+    void DocumentWidget::selectAll()
+    {
+        if ( m_editMode == emList )
+            m_listViewElements->selectAll();
+        else if ( m_editMode == emSource )
+            m_sourceView->selectAll();
+    }
+
     void DocumentWidget::setReadOnly( bool isReadOnly )
     {
         m_isReadOnly = isReadOnly;
         m_listViewElements->setReadOnly( m_isReadOnly );
         m_sourceView->setReadOnly( m_isReadOnly );
+        m_sideBar->setReadOnly( m_isReadOnly );
     }
 
     void DocumentWidget::configureEditor()
@@ -1026,24 +1135,45 @@ namespace KBibTeX
             kapp->invokeBrowser( QString( searchURL ).arg( queryString.replace( '{', "" ).replace( '}', "" ) ) );
     }
 
-    void DocumentWidget::doWebQuery( WebQuery* wq )
+    void DocumentWidget::onlineSearch()
     {
-        if ( !m_isReadOnly && m_editMode == emList )
+        if ( !m_isReadOnly )
         {
-            BibTeX::File * bibtexFile = wq->query();
-
-            if ( bibtexFile != NULL )
+            BibTeX::FileExporter * exporter = new BibTeX::FileExporterBibTeX( );
+            QValueList<BibTeX::Entry*> list;
+            if ( WebQueryWizard::execute( this, list ) == QDialog::Accepted )
             {
-                if ( m_listViewElements->insertItems( bibtexFile ) )
+                Settings * settings = Settings::self( m_bibtexfile );
+                for ( QValueList<BibTeX::Entry*>::Iterator it = list.begin(); it != list.end(); ++it )
                 {
-                    Settings * settings = Settings::self( m_bibtexfile );
-                    settings->addToCompletion( bibtexFile );
-                    slotModified();
+                    if ( m_editMode == emList )
+                        m_listViewElements->insertItem( new BibTeX::Entry( *it ) );
+                    else
+                    {
+                        QBuffer buffer;
+                        buffer.open( IO_WriteOnly );
+                        bool result = exporter->save( &buffer, *it );
+                        buffer.close();
+                        if ( result )
+                        {
+                            buffer.open( IO_ReadOnly );
+                            QTextStream textStream( &buffer );
+                            textStream.setEncoding( QTextStream::UnicodeUTF8 );
+                            QString text = textStream.read();
+                            buffer.close();
+                            QStringList lines = QStringList::split( '\n', text );
+                            for ( QStringList::Iterator it = lines.begin(); it != lines.end(); ++it )
+                                m_sourceView->insertLines( *it );
+                            m_sourceView->insertLines( "" );
+                        }
+
+                    }
+                    settings->addToCompletion( *it );
                 }
 
-                if ( bibtexFile != NULL )
-                    delete bibtexFile;
+                slotModified();
             }
+            delete exporter;
         }
     }
 
@@ -1062,6 +1192,19 @@ namespace KBibTeX
         }
     }
 
+    QString DocumentWidget::nextNewEntry()
+    {
+        QString name = QString( i18n( "May only contain ASCII characters, in case of doubt keep English form", "NewEntry%1" ) ).arg( m_newElementCounter++ );
+
+        while ( m_bibtexfile->containsKey( name ) != NULL )
+        {
+            ++m_newElementCounter;
+            name = QString( i18n( "May only contain ASCII characters, in case of doubt keep English form", "NewEntry%1" ) ).arg( m_newElementCounter++ );
+        }
+
+        return name;
+    }
+
     KURL::List DocumentWidget::getEntryURLs( BibTeX::Entry *entry )
     {
         QStringList urls = entry->urls();
@@ -1090,6 +1233,57 @@ namespace KBibTeX
         emit listViewSelectionChanged( numSelected );
     }
 
+    BibTeX::FileImporter *DocumentWidget::fileImporterFactory( const QString &fileName )
+    {
+        BibTeX::FileImporter * importer = NULL;
+        BibTeX::File::FileFormat format = BibTeX::File::formatUndefined;
+
+        if ( fileName.endsWith( ".bib", FALSE ) )
+            format = BibTeX::File::formatBibTeX;
+#ifdef HAVE_LIBBIBUTILS
+        else if ( fileName.endsWith( ".ref", FALSE ) || fileName.endsWith( ".refer", FALSE ) || fileName.endsWith( ".txt", FALSE ) || fileName.endsWith( ".rfr", FALSE ) )
+            format = BibTeX::File::formatEndNote;
+        else if ( fileName.endsWith( ".isi", FALSE ) || fileName.endsWith( ".cgi", FALSE ) )
+            format = BibTeX::File::formatISI;
+#endif // HAVE_LIBBIBUTILS
+        else if ( fileName.endsWith( ".ris", FALSE ) )
+            format = BibTeX::File::formatRIS;
+
+        if ( format != BibTeX::File::formatUndefined )
+        {
+            switch ( format )
+            {
+            case BibTeX::File::formatBibTeX:
+                {
+                    Settings * settings = Settings::self( NULL );
+                    importer = new BibTeX::FileImporterBibTeX( settings->editing_FirstNameFirst );
+                }
+                break;
+#ifdef HAVE_LIBBIBUTILS
+            case BibTeX::File::formatEndNote:
+                importer = new BibTeX::FileImporterBibUtils( BibTeX::FileImporterBibUtils::ifEndNote );
+                break;
+            case BibTeX::File::formatRIS:
+                importer = new BibTeX::FileImporterBibUtils( BibTeX::FileImporterBibUtils::ifRIS );
+                break;
+            case BibTeX::File::formatISI:
+                importer = new BibTeX::FileImporterBibUtils( BibTeX::FileImporterBibUtils::ifISI );
+                break;
+#else
+            case BibTeX::File::formatRIS:
+                importer = new BibTeX::FileImporterRIS( );
+                break;
+#endif
+            default:
+                {
+                    KMessageBox::sorry( this, QString( i18n( "The file '%1' does not contain a known type of bibliography." ) ).arg( fileName ), i18n( "Unknown file format" ) );
+                }
+            }
+        }
+
+        return importer;
+    }
+
     void DocumentWidget::slotPreviewElement( QListViewItem * item )
     {
         if ( item == NULL )
@@ -1168,6 +1362,13 @@ namespace KBibTeX
                 m_sourceView->setBibTeXFile( m_bibtexfile );
             }
             m_editMode = emSource;
+
+            if ( m_actionEditCut && m_actionEditCopy && m_actionEditPaste )
+            {
+                m_actionEditCut->setEnabled( TRUE );
+                m_actionEditCopy->setEnabled( TRUE );
+                m_actionEditPaste->setEnabled( TRUE );
+            }
         }
         else if ( tab == m_container && m_editMode != emList )
         {
@@ -1185,6 +1386,7 @@ namespace KBibTeX
                     m_sideBar->refreshLists( m_bibtexfile );
                 }
                 m_listViewElements->setBibTeXFile( m_bibtexfile );
+                emit listViewSelectionChanged( 0 );
             }
             m_editMode = emList;
         }
@@ -1196,14 +1398,14 @@ namespace KBibTeX
         else if ( tab == m_container )
             m_listViewElements->setFocus();
 
-        if ( m_find && m_findNext )
+        if ( m_actionEditFind && m_actionEditFindNext )
         {
-            m_find->setEnabled( m_editMode == emSource );
-            m_findNext->setEnabled( m_editMode == emSource );
+            m_actionEditFind->setEnabled( m_editMode == emSource );
+            m_actionEditFindNext->setEnabled( m_editMode == emSource );
         }
+        if ( m_actionEditCopyRef )
+            m_actionEditCopyRef->setEnabled( m_editMode == emList );
 
-        if ( m_copyReference )
-            m_copyReference->setEnabled( m_editMode == emList );
 
         emit undoChanged( m_editMode == emSource );
     }
@@ -1219,10 +1421,19 @@ namespace KBibTeX
 
     void DocumentWidget::slotFileGotDirty( const QString& path )
     {
-        m_dirWatch.stopScan();
-        if ( path == m_filename && KMessageBox::questionYesNo( this, QString( i18n( "File '%1' has been modified. Reload file to import changes or ignore changes?" ) ).arg( m_filename ), i18n( "Reload file?" ), KGuiItem( i18n( "Reload" ), "reload" ), KGuiItem( i18n( "Ignore" ), "ignore" ) ) == KMessageBox::Yes )
+        if ( path == m_filename )
+        {
+            m_dirWatch.removeFile( m_filename );
+            QTimer::singleShot( 100, this, SLOT( slotRefreshDirtyFile( ) ) );
+        }
+    }
+
+    void DocumentWidget::slotRefreshDirtyFile()
+    {
+        if ( KMessageBox::questionYesNo( this, QString( i18n( "File '%1' has been modified. Reload file to import changes or ignore changes?" ) ).arg( m_filename ), i18n( "Reload file?" ), KGuiItem( i18n( "Reload" ), "reload" ), KGuiItem( i18n( "Ignore" ), "ignore" ) ) == KMessageBox::Yes )
             open( m_filename, FALSE );
-        m_dirWatch.startScan();
+        else
+            m_dirWatch.addFile( m_filename );
     }
 
     void DocumentWidget::slotViewDocument( int id )
