@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -31,9 +31,13 @@ namespace BibTeX
         unsigned int unicode;
         const char *latex;
     }
-    charmappingdataxml[] = {
-                               {"&amp;", 0x0026, "&amp;"},
-                           };
+    charmappingdataxml[] =
+    {
+        {"&quot;", 0x0022, "&quot;"},
+        {"&amp;", 0x0026, "&amp;"},
+        {"&lt;", 0x003C, "&lt;"},
+        {"&gt;", 0x003E, "&gt;"}
+    };
     static const int charmappingdataxmlcount = sizeof( charmappingdataxml ) / sizeof( charmappingdataxml[ 0 ] ) ;
 
     EncoderXML::EncoderXML()
@@ -55,6 +59,34 @@ namespace BibTeX
         for ( QValueList<CharMappingItem>::ConstIterator it = m_charMapping.begin(); it != m_charMapping.end(); ++it )
             result.replace(( *it ).regExp, ( *it ).unicode );
 
+        /**
+          * Find and replace all characters written as hexadecimal number
+          */
+        int p = -1;
+        while (( p = result.find( "&#x", p + 1 ) ) >= 0 )
+        {
+            int p2 = result.find( ";", p + 1 );
+            if ( p2 < 0 ) break;
+            bool ok = FALSE;
+            int hex = result.mid( p + 3, p2 - p - 3 ).toInt( &ok, 16 );
+            if ( ok && hex > 0 )
+                result.replace( result.mid( p, p2 - p + 1 ), QChar( hex ) );
+        }
+
+        /**
+          * Find and replace all characters written as decimal number
+          */
+        p = -1;
+        while (( p = result.find( "&#", p + 1 ) ) >= 0 )
+        {
+            int p2 = result.find( ";", p + 1 );
+            if ( p2 < 0 ) break;
+            bool ok = FALSE;
+            int dec = result.mid( p + 2, p2 - p - 2 ).toInt( &ok, 10 );
+            if ( ok && dec > 0 )
+                result.replace( result.mid( p, p2 - p + 1 ), QChar( dec ) );
+        }
+
         return result;
     }
 
