@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2006 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -46,9 +46,11 @@ namespace BibTeX
     Entry::Entry( const QString& entryTypeString, const QString& id ) : Element( ), m_entryTypeString( entryTypeString ), m_id( id )
     {
         m_entryType = entryTypeFromString( entryTypeString );
+        if ( m_entryType != etUnknown )
+            m_entryTypeString = entryTypeToString( m_entryType );
     }
 
-    Entry::Entry( Entry *other )
+    Entry::Entry( const Entry *other )
     {
         copyFrom( other );
     }
@@ -61,6 +63,25 @@ namespace BibTeX
         }
     }
 
+    Element* Entry::clone()
+    {
+        return new Entry( this );
+    }
+
+    QString Entry::text() const
+    {
+        QString result = "Id: ";
+        result.append( m_id ).append( "  (" ).append( entryTypeString() ).append( ")\n" );
+
+        for ( EntryFields::const_iterator it = m_fields.begin(); it != m_fields.end(); it++ )
+        {
+            result.append(( *it )->fieldTypeName() ).append( ": " );
+            result.append(( *it )->value()->text() ).append( "\n" );
+        }
+
+        return result;
+    }
+
     void Entry::setEntryType( const EntryType entryType )
     {
         m_entryType = entryType;
@@ -78,7 +99,7 @@ namespace BibTeX
         return m_entryType;
     }
 
-    QString Entry::entryTypeString()
+    QString Entry::entryTypeString() const
     {
         return m_entryTypeString;
     }
@@ -93,31 +114,59 @@ namespace BibTeX
         return m_id;
     }
 
-    bool Entry::containsPattern( const QString & pattern, EntryField::FieldType fieldType, bool caseSensitive )
+    bool Entry::containsPattern( const QString & pattern, EntryField::FieldType fieldType, BibTeX::Element::FilterType filterType, bool caseSensitive ) const
     {
-        bool result = fieldType == EntryField::ftUnknown && m_id.contains( pattern, caseSensitive );
+        if ( filterType == ftExact )
+        {
+            /** check for exact match */
+            bool result = fieldType == EntryField::ftUnknown && m_id.contains( pattern, caseSensitive );
 
-        for ( EntryFields::iterator it = m_fields.begin(); !result && it != m_fields.end(); it++ )
-            if ( fieldType == EntryField::ftUnknown || ( *it ) ->fieldType() == fieldType )
-                result |= ( *it ) ->value() ->containsPattern( pattern, caseSensitive );
+            for ( EntryFields::ConstIterator it = m_fields.begin(); !result && it != m_fields.end(); it++ )
+                if ( fieldType == EntryField::ftUnknown || ( *it ) ->fieldType() == fieldType )
+                    result |= ( *it ) ->value() ->containsPattern( pattern, caseSensitive );
 
-        return result;
+            return result;
+        }
+        else
+        {
+            /** for each word in the search pattern ... */
+            QStringList words = QStringList::split( QRegExp( "\\s+" ), pattern );
+            bool *hits = new bool[words.count()];
+            int i = 0;
+            for ( QStringList::Iterator wit = words.begin(); wit != words.end(); ++wit, ++i )
+            {
+                hits[i] = fieldType == EntryField::ftUnknown && m_id.contains( *wit, caseSensitive );
+
+                /** check if word is contained in any field */
+                for ( EntryFields::ConstIterator fit = m_fields.begin(); fit != m_fields.end(); ++fit )
+                    if ( fieldType == EntryField::ftUnknown || ( *fit ) ->fieldType() == fieldType )
+                        hits[i] |= ( *fit ) ->value() ->containsPattern( *wit, caseSensitive );
+            }
+
+            unsigned int hitCount = 0;
+            for ( i = words.count() - 1; i >= 0; --i )
+                if ( hits[i] ) ++hitCount;
+            delete[] hits;
+
+            /** return success depending on filter type and number of hits */
+            return ( filterType == ftAnyWord && hitCount > 0 || filterType == ftEveryWord && hitCount == words.count() );
+        }
     }
 
-    QStringList Entry::urls()
+    QStringList Entry::urls() const
     {
         QStringList result;
-        QString fieldNames[] = {"localfile", "pdf", "ps", "postscript", "doi", "url", "howpublished", "ee"};
+        QString fieldNames[] = {"localfile", "pdf", "ps", "postscript", "doi", "url", "howpublished", "ee", "biburl"};
 
-        for ( int i = 0; i < 8; i++ )
+        for ( unsigned int i = 0; i < sizeof( fieldNames ) / sizeof( fieldNames[0] ); i++ )
         {
             EntryField * field = getField( fieldNames[ i ] );
-            if ( ( field && !field->value() ->isEmpty() ) )
+            if (( field && !field->value() ->items.isEmpty() ) )
             {
-                Value * value = field->value();
-                if ( value->count() == 1 && !value->first() ->isStringKey() )
+                PlainText *plainText = dynamic_cast<PlainText*>( field->value()->items.first() );
+                if ( plainText != NULL )
                 {
-                    QString plain = value->plainString();
+                    QString plain = plainText->text();
                     int urlPos = plain.find( "\\url{", 0, FALSE );
                     if ( urlPos > -1 )
                     {
@@ -144,22 +193,22 @@ namespace BibTeX
         return TRUE;
     }
 
-    EntryField* Entry::getField( const EntryField::FieldType fieldType )
+    EntryField* Entry::getField( const EntryField::FieldType fieldType ) const
     {
         EntryField * result = NULL;
 
-        for ( EntryFields::iterator it = m_fields.begin(); ( it != m_fields.end() ) && ( result == NULL ); it++ )
-            if ( ( *it ) ->fieldType() == fieldType ) result = *it;
+        for ( EntryFields::ConstIterator it = m_fields.begin(); ( it != m_fields.end() ) && ( result == NULL ); it++ )
+            if (( *it ) ->fieldType() == fieldType ) result = *it;
 
         return result;
     }
 
-    EntryField* Entry::getField( const QString & fieldName )
+    EntryField* Entry::getField( const QString & fieldName ) const
     {
         EntryField * result = NULL;
 
-        for ( EntryFields::iterator it = m_fields.begin(); ( it != m_fields.end() ) && ( result == NULL ); it++ )
-            if ( ( *it ) ->fieldTypeName().lower() == fieldName.lower() )
+        for ( EntryFields::ConstIterator it = m_fields.begin(); ( it != m_fields.end() ) && ( result == NULL ); it++ )
+            if (( *it ) ->fieldTypeName().lower() == fieldName.lower() )
                 result = *it;
 
         return result;
@@ -167,11 +216,11 @@ namespace BibTeX
 
     bool Entry::deleteField( const QString & fieldName )
     {
-        for ( EntryFields::iterator it = m_fields.begin(); it != m_fields.end(); it++ )
-            if ( ( *it ) ->fieldTypeName().lower() == fieldName.lower() )
+        for ( EntryFields::ConstIterator it = m_fields.begin(); it != m_fields.end(); it++ )
+            if (( *it ) ->fieldTypeName().lower() == fieldName.lower() )
             {
                 delete( *it );
-                m_fields.remove( it );
+                m_fields.remove( *it );
                 return TRUE;
             }
 
@@ -181,7 +230,7 @@ namespace BibTeX
     bool Entry::deleteField( const EntryField::FieldType fieldType )
     {
         for ( EntryFields::iterator it = m_fields.begin(); it != m_fields.end(); it++ )
-            if ( ( *it ) ->fieldType() == fieldType )
+            if (( *it ) ->fieldType() == fieldType )
             {
                 delete( *it );
                 m_fields.remove( it );
@@ -191,14 +240,14 @@ namespace BibTeX
         return FALSE;
     }
 
-    Entry::EntryFields::iterator Entry::begin()
+    Entry::EntryFields::const_iterator Entry::begin() const
     {
-        return m_fields.begin();
+        return m_fields.constBegin();
     }
 
-    Entry::EntryFields::iterator Entry::end()
+    Entry::EntryFields::const_iterator Entry::end() const
     {
-        return m_fields.end();
+        return m_fields.constEnd();
     }
 
     int Entry::getFieldCount() const
@@ -213,16 +262,38 @@ namespace BibTeX
         m_fields.clear();
     }
 
-    void Entry::copyFrom( Entry *other )
+    void Entry::copyFrom( const Entry *other )
     {
+        if ( other == NULL ) return;
+
         m_entryType = other->m_entryType;
         m_entryTypeString = other->m_entryTypeString;
         m_id = other->m_id;
         clearFields();
-        for ( EntryFields::iterator it = other->m_fields.begin(); it != other->m_fields.end(); it++ )
+        for ( EntryFields::const_iterator it = other->m_fields.begin(); it != other->m_fields.end(); it++ )
             m_fields.append( new EntryField( *it ) );
     }
 
+    void Entry::merge( BibTeX::Entry *other, bool forceAdding )
+    {
+        for ( EntryFields::iterator it = other->m_fields.begin(); it != other->m_fields.end(); it++ )
+        {
+            EntryField *otherField = new EntryField( *it );
+            EntryField::FieldType otherFieldType = otherField->fieldType();
+            QString otherFieldTypeName = otherField->fieldTypeName();
+            EntryField *thisField = otherFieldType != EntryField::ftUnknown ? getField( otherFieldType ) : getField( otherFieldTypeName );
+
+            if ( thisField != NULL && !forceAdding ) continue;
+
+            if ( thisField != NULL )
+            {
+                otherFieldTypeName.prepend( "OPT" );
+                otherField->setFieldType( EntryField::ftUnknown, otherFieldTypeName );
+            }
+            m_fields.append( otherField );
+        }
+    }
+
     QString Entry::entryTypeToString( const EntryType entryType )
     {
         switch ( entryType )
@@ -273,13 +344,13 @@ namespace BibTeX
             return etBooklet;
         else if ( entryTypeStringLower == "collection" )
             return etCollection;
-        else if ( ( entryTypeStringLower == "electronic" ) || ( entryTypeStringLower == "online" ) || ( entryTypeStringLower == "internet" ) || ( entryTypeStringLower == "webpage" ) )
+        else if (( entryTypeStringLower == "electronic" ) || ( entryTypeStringLower == "online" ) || ( entryTypeStringLower == "internet" ) || ( entryTypeStringLower == "webpage" ) )
             return etElectronic;
         else if ( entryTypeStringLower == "inbook" )
             return etInBook;
         else if ( entryTypeStringLower == "incollection" )
             return etInCollection;
-        else if ( ( entryTypeStringLower == "inproceedings" ) || ( entryTypeStringLower == "conference" ) )
+        else if (( entryTypeStringLower == "inproceedings" ) || ( entryTypeStringLower == "conference" ) )
             return etInProceedings;
         else if ( entryTypeStringLower == "manual" )
             return etManual;
@@ -322,7 +393,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etBook:
             switch ( fieldType )
@@ -345,7 +417,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etBooklet:
             switch ( fieldType )
@@ -364,7 +437,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etElectronic:
             switch ( fieldType )
@@ -380,7 +454,8 @@ namespace BibTeX
             case EntryField::ftKey:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etInBook:
             switch ( fieldType )
@@ -406,7 +481,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etInCollection:
             switch ( fieldType )
@@ -432,7 +508,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etInProceedings:
             switch ( fieldType )
@@ -461,7 +538,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etManual:
             switch ( fieldType )
@@ -480,7 +558,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etMastersThesis:
             switch ( fieldType )
@@ -498,7 +577,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etMisc:
             switch ( fieldType )
@@ -514,7 +594,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etPhDThesis:
             switch ( fieldType )
@@ -533,7 +614,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etCollection:
         case etProceedings:
@@ -557,7 +639,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etTechReport:
             switch ( fieldType )
@@ -576,8 +659,10 @@ namespace BibTeX
             case EntryField::ftKey:
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
+            case EntryField::ftISSN:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         case etUnpublished:
             switch ( fieldType )
@@ -593,7 +678,8 @@ namespace BibTeX
             case EntryField::ftURL:
             case EntryField::ftLocalFile:
                 return Entry::frsOptional;
-            default: return Entry::frsIgnored;
+            default:
+                return Entry::frsIgnored;
             }
         default:
             return Entry::frsOptional;
