@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -38,8 +38,8 @@
 namespace KBibTeX
 {
 
-    EntryWidgetExternal::EntryWidgetExternal( BibTeX::Entry *entry, BibTeX::File *bibtexfile, bool isReadOnly, QWidget *parent, const char *name )
-            : EntryWidgetTab( entry, bibtexfile, isReadOnly, parent, name )
+    EntryWidgetExternal::EntryWidgetExternal( BibTeX::File *bibtexfile, bool isReadOnly, QWidget *parent, const char *name )
+            : EntryWidgetTab( bibtexfile, isReadOnly, parent, name )
     {
         setupGUI();
     }
@@ -64,40 +64,40 @@ namespace KBibTeX
 
         enableWidget = enableAll || BibTeX::Entry::getRequireStatus( entryType, BibTeX::EntryField::ftLocalFile ) != BibTeX::Entry::frsIgnored;
         m_fieldLineEditLocalFile->setEnabled( enableWidget );
-        m_pushButtonBrowseLocalFile->setEnabled( enableWidget );
+        m_pushButtonBrowseLocalFile->setEnabled( !m_isReadOnly && enableWidget );
 
         updateGUI();
     }
 
-    void EntryWidgetExternal::apply( )
+    void EntryWidgetExternal::apply( BibTeX::Entry *entry )
     {
         BibTeX::Value * value = m_fieldLineEditURL->value();
-        setValue( m_entry, BibTeX::EntryField::ftURL, value );
+        setValue( entry, BibTeX::EntryField::ftURL, value );
         delete value;
 
         value = m_fieldLineEditDoi->value();
-        setValue( m_entry, BibTeX::EntryField::ftDoi, value );
+        setValue( entry, BibTeX::EntryField::ftDoi, value );
         delete value;
 
         value = m_fieldLineEditLocalFile->value();
-        setValue( m_entry, BibTeX::EntryField::ftLocalFile, value );
+        setValue( entry, BibTeX::EntryField::ftLocalFile, value );
         delete value;
     }
 
-    void EntryWidgetExternal::reset( )
+    void EntryWidgetExternal::reset( BibTeX::Entry *entry )
     {
         disconnect( m_fieldLineEditURL, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );
         disconnect( m_fieldLineEditDoi, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );
         disconnect( m_fieldLineEditLocalFile, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );
 
 
-        BibTeX::EntryField * field = m_entry->getField( BibTeX::EntryField::ftURL );
+        BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftURL );
         m_fieldLineEditURL->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftDoi );
+        field = entry->getField( BibTeX::EntryField::ftDoi );
         m_fieldLineEditDoi->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftLocalFile );
+        field = entry->getField( BibTeX::EntryField::ftLocalFile );
         m_fieldLineEditLocalFile->setValue( field != NULL ? field->value() : NULL );
 
         updateGUI();
@@ -184,6 +184,7 @@ namespace KBibTeX
         QLabel *label = new QLabel( QString( "%1:" ).arg( i18n( "URL" ) ), this );
         gridLayout->addWidget( label, 0, 0 );
         m_fieldLineEditURL = new KBibTeX::FieldLineEdit( i18n( "URL" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditURL" );
+        m_fieldLineEditURL->setFieldType( BibTeX::EntryField::ftURL );
         gridLayout->addWidget( m_fieldLineEditURL, 0, 1 );
         label->setBuddy( m_fieldLineEditURL );
         connect( m_fieldLineEditURL, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );
@@ -200,6 +201,7 @@ namespace KBibTeX
         urllabel->setFocusPolicy( QWidget::NoFocus );
         gridLayout->addWidget( urllabel, 1, 0 );
         m_fieldLineEditDoi = new KBibTeX::FieldLineEdit( i18n( "DOI" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditDoi" );
+        m_fieldLineEditDoi->setFieldType( BibTeX::EntryField::ftDoi );
         gridLayout->addWidget( m_fieldLineEditDoi, 1, 1 );
         urllabel->setBuddy( m_fieldLineEditDoi );
         connect( m_fieldLineEditDoi, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );
@@ -209,12 +211,13 @@ namespace KBibTeX
         gridLayout->addWidget( m_pushButtonOpenDoi, 1, 2 );
         m_pushButtonOpenDoi->setIconSet( QIconSet( SmallIcon( "fileopen" ) ) );
         m_pushButtonOpenDoi->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
-        QToolTip::add( m_pushButtonOpenDoi, QString( i18n( "Open %1" ) ).arg( i18n( "Doi" ) ) );
+        QToolTip::add( m_pushButtonOpenDoi, i18n( "Open DOI" ) );
         connect( m_pushButtonOpenDoi, SIGNAL( clicked() ), this, SLOT( openDoi() ) );
 
         label = new QLabel( QString( "%1:" ).arg( i18n( "Local File" ) ), this );
         gridLayout->addWidget( label, 2, 0 );
         m_fieldLineEditLocalFile = new KBibTeX::FieldLineEdit( i18n( "Local File" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditLocalFile" );
+        m_fieldLineEditLocalFile->setFieldType( BibTeX::EntryField::ftLocalFile );
         gridLayout->addWidget( m_fieldLineEditLocalFile, 2, 1 );
         label->setBuddy( m_fieldLineEditLocalFile );
         connect( m_fieldLineEditLocalFile, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );
