@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2006 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -19,10 +19,13 @@
 ***************************************************************************/
 #include <qlayout.h>
 #include <qlabel.h>
+#include <qregexp.h>
 #include <qpushbutton.h>
 #include <qpopupmenu.h>
-#include <kiconloader.h>
+#include <qtooltip.h>
 
+#include <kapplication.h>
+#include <kiconloader.h>
 #include <kdialog.h>
 #include <klocale.h>
 
@@ -33,8 +36,8 @@
 namespace KBibTeX
 {
 
-    EntryWidgetPublication::EntryWidgetPublication( BibTeX::Entry *entry, BibTeX::File *bibtexfile, bool isReadOnly, QWidget *parent, const char *name )
-            : EntryWidgetTab( entry, bibtexfile, isReadOnly, parent, name )
+    EntryWidgetPublication::EntryWidgetPublication( BibTeX::File *bibtexfile, bool isReadOnly, QWidget *parent, const char *name )
+            : EntryWidgetTab( bibtexfile, isReadOnly, parent, name )
     {
         setupGUI();
     }
@@ -84,13 +87,15 @@ namespace KBibTeX
 
         enableWidget = enableAll || BibTeX::Entry::getRequireStatus( entryType, BibTeX::EntryField::ftMonth ) != BibTeX::Entry::frsIgnored;
         m_fieldLineEditMonth->setEnabled( enableWidget );
-        m_pushButtonMonths->setEnabled( enableWidget );
+        m_pushButtonMonths->setEnabled( enableWidget && !m_isReadOnly );
 
         enableWidget = enableAll || BibTeX::Entry::getRequireStatus( entryType, BibTeX::EntryField::ftYear ) != BibTeX::Entry::frsIgnored;
         m_fieldLineEditYear->setEnabled( enableWidget );
 
         enableWidget = enableAll || BibTeX::Entry::getRequireStatus( entryType, BibTeX::EntryField::ftISBN ) != BibTeX::Entry::frsIgnored;
         m_fieldLineEditISBN->setEnabled( enableWidget );
+        int len = isbn().length();
+        m_pushButtonISBN->setEnabled( len == 10 || len == 13 );
 
         enableWidget = enableAll || BibTeX::Entry::getRequireStatus( entryType, BibTeX::EntryField::ftISSN ) != BibTeX::Entry::frsIgnored;
         m_fieldLineEditISSN->setEnabled( enableWidget );
@@ -108,147 +113,150 @@ namespace KBibTeX
         m_fieldLineEditPages->setEnabled( enableWidget );
     }
 
-    void EntryWidgetPublication::apply( )
+    void EntryWidgetPublication::apply( BibTeX::Entry *entry )
     {
         BibTeX::Value * value = m_fieldLineEditHowPublished->value();
-        setValue( m_entry, BibTeX::EntryField::ftHowPublished, value );
+        setValue( entry, BibTeX::EntryField::ftHowPublished, value );
         delete value;
 
         value = m_fieldLineEditPublisher->value();
-        setValue( m_entry, BibTeX::EntryField::ftPublisher, value );
+        setValue( entry, BibTeX::EntryField::ftPublisher, value );
         delete value;
 
         value = m_fieldLineEditOrganization->value();
-        setValue( m_entry, BibTeX::EntryField::ftOrganization, value );
+        setValue( entry, BibTeX::EntryField::ftOrganization, value );
         delete value;
 
         value = m_fieldLineEditInstitution->value();
-        setValue( m_entry, BibTeX::EntryField::ftInstitution, value );
+        setValue( entry, BibTeX::EntryField::ftInstitution, value );
         delete value;
 
         value = m_fieldLineEditSchool->value();
-        setValue( m_entry, BibTeX::EntryField::ftSchool, value );
+        setValue( entry, BibTeX::EntryField::ftSchool, value );
         delete value;
 
         value = m_fieldLineEditLocation->value();
-        setValue( m_entry, BibTeX::EntryField::ftLocation, value );
+        setValue( entry, BibTeX::EntryField::ftLocation, value );
         delete value;
 
         value = m_fieldLineEditAddress->value();
-        setValue( m_entry, BibTeX::EntryField::ftAddress, value );
+        setValue( entry, BibTeX::EntryField::ftAddress, value );
         delete value;
 
         value = m_fieldLineEditJournal->value();
-        setValue( m_entry, BibTeX::EntryField::ftJournal, value );
+        setValue( entry, BibTeX::EntryField::ftJournal, value );
         delete value;
 
         value = m_fieldLineEditPages->value();
-        if ( value != NULL && value->count() > 0 )
-            for ( QValueList<BibTeX::ValueItem*>::const_iterator it = value->begin(); it != value->end(); ++it )
-            {
-                BibTeX::ValueItem *item = *it;
-                if ( !item->isStringKey() )
-                    item->setText( item->text().replace( "--", QChar( 0x2013 ) ).replace( "-", QChar( 0x2013 ) ) );
-            }
-        setValue( m_entry, BibTeX::EntryField::ftPages, value );
+        if ( value != NULL && value->items.first() != NULL )
+        {
+            BibTeX::PlainText *plainText = dynamic_cast<BibTeX::PlainText*>( value->items.first() );
+            if ( plainText != NULL )
+                plainText->setText( plainText->text().replace( QRegExp( "\\s*--?\\s*" ), QChar( 0x2013 ) ) );
+        }
+        setValue( entry, BibTeX::EntryField::ftPages, value );
         delete value;
 
         value = m_fieldLineEditEdition->value();
-        setValue( m_entry, BibTeX::EntryField::ftEdition, value );
+        setValue( entry, BibTeX::EntryField::ftEdition, value );
         delete value;
 
         value = m_fieldLineEditChapter->value();
-        setValue( m_entry, BibTeX::EntryField::ftChapter, value );
+        setValue( entry, BibTeX::EntryField::ftChapter, value );
         delete value;
 
         value = m_fieldLineEditVolume->value();
-        setValue( m_entry, BibTeX::EntryField::ftVolume, value );
+        setValue( entry, BibTeX::EntryField::ftVolume, value );
         delete value;
 
         value = m_fieldLineEditNumber->value();
-        setValue( m_entry, BibTeX::EntryField::ftNumber, value );
+        setValue( entry, BibTeX::EntryField::ftNumber, value );
         delete value;
 
         value = m_fieldLineEditMonth->value();
-        setValue( m_entry, BibTeX::EntryField::ftMonth, value );
+        setValue( entry, BibTeX::EntryField::ftMonth, value );
         delete value;
 
         value = m_fieldLineEditYear->value();
-        setValue( m_entry, BibTeX::EntryField::ftYear, value );
+        setValue( entry, BibTeX::EntryField::ftYear, value );
         delete value;
 
         value = m_fieldLineEditISBN->value();
-        setValue( m_entry, BibTeX::EntryField::ftISBN, value );
+        setValue( entry, BibTeX::EntryField::ftISBN, value );
         delete value;
 
         value = m_fieldLineEditISSN->value();
-        setValue( m_entry, BibTeX::EntryField::ftISSN, value );
+        setValue( entry, BibTeX::EntryField::ftISSN, value );
         delete value;
 
         value = m_fieldLineEditCrossRef->value();
-        setValue( m_entry, BibTeX::EntryField::ftCrossRef, value );
+        setValue( entry, BibTeX::EntryField::ftCrossRef, value );
         delete value;
     }
 
-    void EntryWidgetPublication::reset( )
+    void EntryWidgetPublication::reset( BibTeX::Entry *entry )
     {
-        BibTeX::EntryField * field = m_entry->getField( BibTeX::EntryField::ftHowPublished );
+        BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftHowPublished );
         m_fieldLineEditHowPublished->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftPublisher );
+        field = entry->getField( BibTeX::EntryField::ftPublisher );
         m_fieldLineEditPublisher->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftOrganization );
+        field = entry->getField( BibTeX::EntryField::ftOrganization );
         m_fieldLineEditOrganization->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftInstitution );
+        field = entry->getField( BibTeX::EntryField::ftInstitution );
         m_fieldLineEditInstitution->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftSchool );
+        field = entry->getField( BibTeX::EntryField::ftSchool );
         m_fieldLineEditSchool->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftLocation );
+        field = entry->getField( BibTeX::EntryField::ftLocation );
         m_fieldLineEditLocation->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftAddress );
+        field = entry->getField( BibTeX::EntryField::ftAddress );
         m_fieldLineEditAddress->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftJournal );
+        field = entry->getField( BibTeX::EntryField::ftJournal );
         m_fieldLineEditJournal->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftPages );
+        field = entry->getField( BibTeX::EntryField::ftPages );
         m_fieldLineEditPages->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftEdition );
+        field = entry->getField( BibTeX::EntryField::ftEdition );
         m_fieldLineEditEdition->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftChapter );
+        field = entry->getField( BibTeX::EntryField::ftChapter );
         m_fieldLineEditChapter->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftVolume );
+        field = entry->getField( BibTeX::EntryField::ftVolume );
         m_fieldLineEditVolume->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftNumber );
+        field = entry->getField( BibTeX::EntryField::ftNumber );
         m_fieldLineEditNumber->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftMonth );
+        field = entry->getField( BibTeX::EntryField::ftMonth );
         m_fieldLineEditMonth->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftYear );
+        field = entry->getField( BibTeX::EntryField::ftYear );
         m_fieldLineEditYear->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftISBN );
+        field = entry->getField( BibTeX::EntryField::ftISBN );
         m_fieldLineEditISBN->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftISSN );
+        field = entry->getField( BibTeX::EntryField::ftISSN );
         m_fieldLineEditISSN->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftCrossRef );
+        field = entry->getField( BibTeX::EntryField::ftCrossRef );
         m_fieldLineEditCrossRef->setValue( field != NULL ? field->value() : NULL );
+        slotSetCrossRefEntry();
     }
 
     void EntryWidgetPublication::updateWarnings( BibTeX::Entry::EntryType entryType, QListView *listViewWarnings )
     {
+        bool crossRefValid = FALSE;
+        BibTeX::Value *value = NULL;
+
         addMissingWarning( entryType, BibTeX::EntryField::ftHowPublished, m_fieldLineEditHowPublished->caption(), !m_fieldLineEditHowPublished->isEmpty(), m_fieldLineEditHowPublished, listViewWarnings );
 
         addMissingWarning( entryType, BibTeX::EntryField::ftPublisher, m_fieldLineEditPublisher->caption(), !m_fieldLineEditPublisher->isEmpty(), m_fieldLineEditPublisher, listViewWarnings );
@@ -267,13 +275,22 @@ namespace KBibTeX
 
         addMissingWarning( entryType, BibTeX::EntryField::ftEdition, m_fieldLineEditEdition->caption(), !m_fieldLineEditEdition->isEmpty(), m_fieldLineEditEdition, listViewWarnings );
 
-        addMissingWarning( entryType, BibTeX::EntryField::ftVolume, m_fieldLineEditVolume->caption(), !m_fieldLineEditVolume->isEmpty(), m_fieldLineEditVolume, listViewWarnings );
+        crossRefValid = m_crossRefEntry != NULL && m_crossRefEntry->getField( BibTeX::EntryField::ftVolume ) != NULL;
+        addMissingWarning( entryType, BibTeX::EntryField::ftVolume, m_fieldLineEditVolume->caption(), !m_fieldLineEditVolume->isEmpty() || crossRefValid, m_fieldLineEditVolume, listViewWarnings );
+        if ( crossRefValid && m_fieldLineEditVolume->isEmpty() )
+            addCrossRefInfo( m_fieldLineEditVolume->caption(), m_fieldLineEditVolume, listViewWarnings );
 
-        addMissingWarning( entryType, BibTeX::EntryField::ftNumber, m_fieldLineEditNumber->caption(), !m_fieldLineEditNumber->isEmpty(), m_fieldLineEditNumber, listViewWarnings );
+        crossRefValid = m_crossRefEntry != NULL && m_crossRefEntry->getField( BibTeX::EntryField::ftNumber ) != NULL;
+        addMissingWarning( entryType, BibTeX::EntryField::ftNumber, m_fieldLineEditNumber->caption(), !m_fieldLineEditNumber->isEmpty() || crossRefValid, m_fieldLineEditNumber, listViewWarnings );
+        if ( crossRefValid && m_fieldLineEditNumber->isEmpty() )
+            addCrossRefInfo( m_fieldLineEditNumber->caption(), m_fieldLineEditNumber, listViewWarnings );
 
         addMissingWarning( entryType, BibTeX::EntryField::ftMonth, m_fieldLineEditMonth->caption(), !m_fieldLineEditMonth->isEmpty(), m_fieldLineEditMonth, listViewWarnings );
 
-        addMissingWarning( entryType, BibTeX::EntryField::ftYear, m_fieldLineEditYear->caption(), !m_fieldLineEditYear->isEmpty(), m_fieldLineEditYear, listViewWarnings );
+        crossRefValid = m_crossRefEntry != NULL && m_crossRefEntry->getField( BibTeX::EntryField::ftYear ) != NULL;
+        addMissingWarning( entryType, BibTeX::EntryField::ftYear, m_fieldLineEditYear->caption(), !m_fieldLineEditYear->isEmpty() || crossRefValid, m_fieldLineEditYear, listViewWarnings );
+        if ( crossRefValid && m_fieldLineEditYear->isEmpty() )
+            addCrossRefInfo( m_fieldLineEditYear->caption(), m_fieldLineEditYear, listViewWarnings );
 
         addMissingWarning( entryType, BibTeX::EntryField::ftISBN, m_fieldLineEditISBN->caption(), !m_fieldLineEditISBN->isEmpty(), m_fieldLineEditISBN, listViewWarnings );
 
@@ -284,7 +301,7 @@ namespace KBibTeX
         if ( entryType == BibTeX::Entry::etInBook )
         {
             if ( m_fieldLineEditChapter->isEmpty() && m_fieldLineEditPages->isEmpty() )
-                new EntryWidgetWarningsItem( EntryWidgetWarningsItem::wlWarning, i18n( "The fields ‘Chapter’ or ‘Pages’ are required, but both are missing" ), m_fieldLineEditPages, listViewWarnings, "warning" );
+                new EntryWidgetWarningsItem( EntryWidgetWarningsItem::wlWarning, i18n( "The fields 'Chapter' or 'Pages' are required, but both are missing" ), m_fieldLineEditPages, listViewWarnings, "warning" );
         }
         else
         {
@@ -292,12 +309,17 @@ namespace KBibTeX
 
             addMissingWarning( entryType, BibTeX::EntryField::ftChapter, m_fieldLineEditChapter->caption(), !m_fieldLineEditChapter->isEmpty(), m_fieldLineEditChapter, listViewWarnings );
         }
-        BibTeX::Value *value = m_fieldLineEditCrossRef->value();
-        if ( value != NULL )
+
+        value = m_fieldLineEditJournal->value();
+        if ( value != NULL && !value->items.isEmpty() )
         {
-            QString crossRefText = m_fieldLineEditCrossRef->value() ->plainString();
-            if ( !crossRefText.isEmpty() && !m_bibtexfile->containsKey( crossRefText ) )
-                new EntryWidgetWarningsItem( EntryWidgetWarningsItem::wlWarning, i18n( "The field ‘CrossRef’ refers to a not existing entry" ), m_fieldLineEditCrossRef, listViewWarnings, "warning" );
+            BibTeX::MacroKey *key = dynamic_cast<BibTeX::MacroKey*>( value->items[0] );
+            if ( key != NULL )
+            {
+                QString jourText = key->text();
+                if ( !jourText.isEmpty() && !m_bibtexfile->containsKey( jourText ) )
+                    new EntryWidgetWarningsItem( EntryWidgetWarningsItem::wlWarning, i18n( "The field 'Journal' refers to a not existing entry" ), m_fieldLineEditJournal, listViewWarnings, "warning" );
+            }
         }
 
         addFieldLineEditWarning( m_fieldLineEditHowPublished, m_fieldLineEditHowPublished->caption(), listViewWarnings );
@@ -336,17 +358,36 @@ namespace KBibTeX
 
         addFieldLineEditWarning( m_fieldLineEditCrossRef, m_fieldLineEditCrossRef->caption(), listViewWarnings );
 
+        /** misusing this function to enable/disable ISBN button */
+        int len = isbn().length();
+        m_pushButtonISBN->setEnabled( len == 10 || len == 13 );
     }
 
     void EntryWidgetPublication::slotSetMonth( int month )
     {
+        BibTeX::MacroKey *item = new BibTeX::MacroKey( BibTeX::MonthsTriple[ month ] );
         BibTeX::Value * value = new BibTeX::Value();
-        BibTeX::ValueItem *item = new BibTeX::ValueItem( BibTeX::MonthsTriple[ month ], TRUE );
-        value->add( item );
+        value->items.append( item );
         m_fieldLineEditMonth->setValue( value );
         delete value;
     }
 
+    void EntryWidgetPublication::slotSetCrossRefEntry()
+    {
+        m_crossRefEntry = NULL;
+        if ( m_fieldLineEditCrossRef->value() != NULL )
+        {
+            QString crossRefText = m_fieldLineEditCrossRef->value() ->text();
+            if ( !crossRefText.isEmpty() )
+                m_crossRefEntry = dynamic_cast<BibTeX::Entry*>( m_bibtexfile->containsKey( crossRefText ) );
+        }
+    }
+
+    void EntryWidgetPublication::slotOpenISBN()
+    {
+        kapp->invokeBrowser( QString( i18n( "http://en.wikipedia.org/w/index.php?title=Special:Booksources&isbn=%1" ) ).arg( isbn() ) );
+    }
+
     /**
      *
      */
@@ -384,6 +425,7 @@ namespace KBibTeX
         m_pushButtonMonths = new QPushButton( this, "m_pushButtonMonths" );
         m_pushButtonMonths->setIconSet( QIconSet( SmallIcon( "vcalendar" ) ) );
         m_pushButtonMonths->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum );
+        m_pushButtonMonths->setEnabled( !m_isReadOnly );
         innerLayout->addWidget( m_pushButtonMonths );
         m_fieldLineEditMonth = new KBibTeX::FieldLineEdit( i18n( "Month" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditMonth" );
         m_fieldLineEditMonth->setFieldType( BibTeX::EntryField::ftMonth );
@@ -424,6 +466,7 @@ namespace KBibTeX
         m_fieldLineEditCrossRef->setFieldType( BibTeX::EntryField::ftCrossRef );
         label->setBuddy( m_fieldLineEditCrossRef );
         gridLayout->addWidget( m_fieldLineEditCrossRef, 8, 1 );
+        connect( m_fieldLineEditCrossRef, SIGNAL( textChanged() ), this, SLOT( slotSetCrossRefEntry() ) );
 
         label = new QLabel( QString( "%1:" ).arg( i18n( "Organization" ) ), this );
         gridLayout->addWidget( label, 0, 3 );
@@ -469,10 +512,18 @@ namespace KBibTeX
 
         label = new QLabel( QString( "%1:" ).arg( i18n( "ISBN" ) ), this );
         gridLayout->addWidget( label, 6, 3 );
+        innerLayout = new QHBoxLayout();
+        gridLayout->addLayout( innerLayout, 6, 4 );
+        m_pushButtonISBN = new QPushButton( this, "m_pushButtonMonths" );
+        m_pushButtonISBN->setIconSet( QIconSet( SmallIcon( "find" ) ) );
+        m_pushButtonISBN->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum );
+        QToolTip::add( m_pushButtonISBN, i18n( "Lookup ISBN number in Wikipedia" ) );
+        innerLayout->addWidget( m_pushButtonISBN );
         m_fieldLineEditISBN = new KBibTeX::FieldLineEdit( i18n( "ISBN" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditISBN" );
         m_fieldLineEditISBN ->setFieldType( BibTeX::EntryField::ftISBN );
         label->setBuddy( m_fieldLineEditISBN );
-        gridLayout->addWidget( m_fieldLineEditISBN, 6, 4 );
+        innerLayout->addWidget( m_fieldLineEditISBN );
+        connect( m_pushButtonISBN, SIGNAL( clicked() ), this, SLOT( slotOpenISBN() ) );
 
         label = new QLabel( QString( "%1:" ).arg( i18n( "ISSN" ) ), this );
         gridLayout->addWidget( label, 7, 3 );
@@ -495,5 +546,13 @@ namespace KBibTeX
         connect( menuMonths, SIGNAL( activated( int ) ), this, SLOT( slotSetMonth( int ) ) );
     }
 
+    QString EntryWidgetPublication::isbn() const
+    {
+        BibTeX::Value *value = m_fieldLineEditISBN->value();
+        if ( value == NULL )
+            return "";
+
+        return value->text().replace( QRegExp( "[^0-9X]" ), "" );
+    }
 }
 #include "entrywidgetpublication.moc"
