@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -23,6 +23,7 @@
 #include <qheader.h>
 #include <qtimer.h>
 #include <qtooltip.h>
+#include <qwhatsthis.h>
 #include <qpushbutton.h>
 #include <qcheckbox.h>
 
@@ -41,8 +42,8 @@
 namespace KBibTeX
 {
 
-    FieldListView::FieldListView( const QString& caption, bool isReadOnly, QWidget *parent, const char *name )
-            : QWidget( parent, name ), m_value( new BibTeX::Value() ), m_caption( caption ), m_isReadOnly( isReadOnly ), m_enabled( TRUE ), m_isComplex( FALSE ), m_isModified( FALSE ), m_newValueCounter( 0 ), m_fieldType( BibTeX::EntryField::ftUnknown )
+    FieldListView::FieldListView( const QString& caption, const QString& prefixNew, bool isReadOnly, QWidget *parent, const char *name )
+            : QWidget( parent, name ), m_value( new BibTeX::Value() ), m_caption( caption ), m_prefixNew( prefixNew ), m_isReadOnly( isReadOnly ), m_enabled( TRUE ), m_isComplex( FALSE ), m_isModified( FALSE ), m_newValueCounter( 0 ), m_fieldType( BibTeX::EntryField::ftUnknown )
     {
         setupGUI();
         m_listViewElements->installEventFilter( this );
@@ -96,6 +97,8 @@ namespace KBibTeX
 
         Settings * settings = Settings::self();
         m_listViewElements->renameLineEdit() ->setCompletionObject( settings->completion( m_fieldType ) );
+        QToolTip::add( m_listViewElements, QString( i18n( "BibTeX field '%1'" ) ).arg( BibTeX::EntryField::fieldTypeToString( fieldType ) ) );
+        QWhatsThis::add( m_listViewElements, QString( i18n( "BibTeX field '%1'" ) ).arg( BibTeX::EntryField::fieldTypeToString( fieldType ) ) );
 
         m_value->items.clear();
     }
@@ -198,7 +201,7 @@ namespace KBibTeX
     {
         if ( isSimple() )
         {
-            KListViewItem * item = new KListViewItem( m_listViewElements, m_listViewElements->lastItem(), QString( i18n( "New%1%2" ) ).arg( m_caption ).arg( m_newValueCounter++ ) );
+            KListViewItem * item = new KListViewItem( m_listViewElements, m_listViewElements->lastItem(), QString( "%1%2" ).arg( m_prefixNew ).arg( ++m_newValueCounter ) );
             m_listViewElements->setSelected( item, TRUE );
             updateGUI();
             QTimer::singleShot( 100, this, SLOT( slotEdit() ) );
@@ -315,37 +318,37 @@ namespace KBibTeX
         layout->addWidget( m_pushButtonAdd, 0, 1 );
         m_pushButtonAdd->setIconSet( QIconSet( SmallIcon( "add" ) ) );
         connect( m_pushButtonAdd, SIGNAL( clicked() ), this, SLOT( slotAdd() ) );
-        QToolTip::add( m_pushButtonAdd, QString( i18n( "Add new ”%1“ item (Ctrl+A)" ) ).arg( m_caption ) );
+        QToolTip::add( m_pushButtonAdd, QString( i18n( "Add new '%1' item (Ctrl+A)" ) ).arg( m_caption ) );
 
         m_pushButtonEdit = new QPushButton( i18n( "Edit" ), this );
         layout->addWidget( m_pushButtonEdit, 1, 1 );
         m_pushButtonEdit->setIconSet( QIconSet( SmallIcon( "edit" ) ) );
         connect( m_pushButtonEdit, SIGNAL( clicked() ), this, SLOT( slotEdit() ) );
-        QToolTip::add( m_pushButtonEdit, QString( i18n( "Edit current ”%1“ item (F2)" ) ).arg( m_caption ) );
+        QToolTip::add( m_pushButtonEdit, QString( i18n( "Edit current '%1' item (F2)" ) ).arg( m_caption ) );
 
         m_pushButtonDelete = new QPushButton( i18n( "Delete" ), this );
         layout->addWidget( m_pushButtonDelete, 2, 1 );
         m_pushButtonDelete->setIconSet( QIconSet( SmallIcon( "editdelete" ) ) );
         connect( m_pushButtonDelete, SIGNAL( clicked() ), this, SLOT( slotDelete() ) );
-        QToolTip::add( m_pushButtonDelete, QString( i18n( "Delete current ”%1“ item (Del)" ) ).arg( m_caption ) );
+        QToolTip::add( m_pushButtonDelete, QString( i18n( "Delete current '%1' item (Del)" ) ).arg( m_caption ) );
 
         m_pushButtonUp = new QPushButton( i18n( "Up" ), this );
         layout->addWidget( m_pushButtonUp, 3, 1 );
         m_pushButtonUp->setIconSet( QIconSet( SmallIcon( "up" ) ) );
         connect( m_pushButtonUp, SIGNAL( clicked() ), this, SLOT( slotUp() ) );
-        QToolTip::add( m_pushButtonUp, QString( i18n( "Move current ”%1“ item up (Ctrl+Up)" ) ).arg( m_caption ) );
+        QToolTip::add( m_pushButtonUp, QString( i18n( "Move current '%1' item up (Ctrl+Up)" ) ).arg( m_caption ) );
 
         m_pushButtonDown = new QPushButton( i18n( "Down" ), this );
         layout->addWidget( m_pushButtonDown, 4, 1 );
         m_pushButtonDown->setIconSet( QIconSet( SmallIcon( "down" ) ) );
         connect( m_pushButtonDown, SIGNAL( clicked() ), this, SLOT( slotDown() ) );
-        QToolTip::add( m_pushButtonDown, QString( i18n( "Move current ”%1“ item down (Ctrl+Down)" ) ).arg( m_caption ) );
+        QToolTip::add( m_pushButtonDown, QString( i18n( "Move current '%1' item down (Ctrl+Down)" ) ).arg( m_caption ) );
 
         m_pushButtonComplexEdit = new QPushButton( i18n( "Complex..." ), this );
         layout->addWidget( m_pushButtonComplexEdit, 6, 1 );
         m_pushButtonComplexEdit->setIconSet( QIconSet( SmallIcon( "leftjust" ) ) );
         connect( m_pushButtonComplexEdit, SIGNAL( clicked() ), this, SLOT( slotComplex() ) );
-        QToolTip::add( m_pushButtonComplexEdit, QString( i18n( "Edit current ”%1“ item as a concatenated string (Ctrl+Alt+C)" ) ).arg( m_caption ) );
+        QToolTip::add( m_pushButtonComplexEdit, QString( i18n( "Edit current '%1' item as a concatenated string (Ctrl+Alt+C)" ) ).arg( m_caption ) );
 
         m_checkBoxEtAl = new QCheckBox( i18n( "... and others (et al.)" ), this );
         layout->addMultiCellWidget( m_checkBoxEtAl, 7, 7, 0, 1 );
@@ -356,7 +359,6 @@ namespace KBibTeX
     {
         QStringList elements;
         Settings * settings = Settings::self();
-        KCompletion *completion = settings->completion( m_fieldType );
 
         m_value->items.clear();
 
@@ -364,7 +366,6 @@ namespace KBibTeX
         {
             QString text = it.current() ->text( 0 );
             elements.append( text );
-            completion->addItem( text );
         }
 
         if ( elements.count() > 0 )
@@ -378,7 +379,10 @@ namespace KBibTeX
             case BibTeX::EntryField::ftEditor:
                 {
                     for ( QStringList::ConstIterator it = elements.constBegin(); it != elements.constEnd(); ++it )
-                        container->persons.append( new BibTeX::Person( *it, settings->editing_FirstNameFirst ) );
+                    {
+                        BibTeX::Person *person = new BibTeX::Person( *it, settings->editing_FirstNameFirst );
+                        container->persons.append( person );
+                    }
                 }
                 break;
             default:
@@ -392,6 +396,8 @@ namespace KBibTeX
                 m_value->items.append( container );
             else
                 delete container;
+
+            settings->addToCompletion( m_value, m_fieldType );
         }
     }
 
