@@ -41,14 +41,14 @@ namespace BibTeX
     class FileExporter;
 
     static const QString Months[] =
-        {
-            QString( "January" ), QString( "February" ), QString( "March" ), QString( "April" ), QString( "May" ), QString( "June" ), QString( "July" ), QString( "August" ), QString( "September" ), QString( "October" ), QString( "November" ), QString( "December" )
-        };
+    {
+        QString( "January" ), QString( "February" ), QString( "March" ), QString( "April" ), QString( "May" ), QString( "June" ), QString( "July" ), QString( "August" ), QString( "September" ), QString( "October" ), QString( "November" ), QString( "December" )
+    };
 
     static const QString MonthsTriple[] =
-        {
-            QString( "jan" ), QString( "feb" ), QString( "mar" ), QString( "apr" ), QString( "may" ), QString( "jun" ), QString( "jul" ), QString( "aug" ), QString( "sep" ), QString( "oct" ), QString( "nov" ), QString( "dec" )
-        };
+    {
+        QString( "jan" ), QString( "feb" ), QString( "mar" ), QString( "apr" ), QString( "may" ), QString( "jun" ), QString( "jul" ), QString( "aug" ), QString( "sep" ), QString( "oct" ), QString( "nov" ), QString( "dec" )
+    };
 
     class File : public QObject
     {
@@ -63,7 +63,7 @@ namespace BibTeX
     public:
         typedef QValueList<Element*> ElementList;
 
-        enum FileFormat { formatUndefined = 0, formatBibTeX = 1, formatXML = 2, formatHTML = 3, formatPDF = 4, formatPS = 5, formatRTF = 6, formatRIS = 7, formatEndNote = 8 };
+        enum FileFormat { formatUndefined = 0, formatBibTeX = 1, formatXML = 2, formatHTML = 3, formatPDF = 4, formatPS = 5, formatRTF = 6, formatRIS = 7, formatEndNote = 8, formatISI = 9 };
         enum Encoding {encImplicit = 0, encLaTeX = 1, encUTF8 = 2};
         enum StringProtection { spNone, spParanthesis, spQuote, spBoth };
 
@@ -78,7 +78,9 @@ namespace BibTeX
         static Element* cloneElement( Element *element );
 
         Element *containsKey( const QString &key );
+        const Element *containsKeyConst( const QString &key ) const;
         QStringList allKeys();
+        QString text();
 
         ElementList::iterator begin();
         ElementList::iterator end();
@@ -88,7 +90,8 @@ namespace BibTeX
         QStringList getAllValuesAsStringList( const EntryField::FieldType fieldType ) const;
         QMap<QString, int> getAllValuesAsStringListWithCount( const EntryField::FieldType fieldType ) const;
         void replaceValue( const QString& oldText, const QString& newText, const EntryField::FieldType fieldType );
-        void completeReferencedFields( BibTeX::Entry *entry );
+        BibTeX::Entry *completeReferencedFieldsConst( const BibTeX::Entry *entry ) const;
+        void completeReferencedFields( BibTeX::Entry *entry ) const;
 
         QString fileName;
 
