@@ -44,12 +44,13 @@ namespace BibTeX
         FileExporterBibTeX();
         ~FileExporterBibTeX();
 
-        bool save( QIODevice* iodevice, File* bibtexfile, QStringList *errorLog = NULL );
-        bool save( QIODevice* iodevice, Element* element, QStringList *errorLog = NULL );
+        bool save( QIODevice* iodevice, const File* bibtexfile, QStringList *errorLog = NULL );
+        bool save( QIODevice* iodevice, const Element* element, QStringList *errorLog = NULL );
 
         void setStringDelimiter( const QChar& stringOpenDelimiter, const QChar& stringCloseDelimiter );
         void setKeywordCasing( const KeywordCasing keywordCasing );
         void setEncoding( File::Encoding encoding );
+        void setEnclosingCurlyBrackets( bool protectCasing );
 
     public slots:
         void cancel();
@@ -59,18 +60,20 @@ namespace BibTeX
         QChar m_stringCloseDelimiter;
         KeywordCasing m_keywordCasing;
         File::Encoding m_encoding;
+        bool m_protectCasing;
         bool cancelFlag;
 
-        bool writeEntry( QTextStream &stream, Entry* entry );
-        bool writeMacro( QTextStream &stream, Macro *macro );
-        bool writeComment( QTextStream &stream, Comment *comment );
-        bool writePreamble( QTextStream &stream, Preamble* preamble );
+        bool writeEntry( QTextStream &stream, const Entry* entry );
+        bool writeMacro( QTextStream &stream, const Macro *macro );
+        bool writeComment( QTextStream &stream, const Comment *comment );
+        bool writePreamble( QTextStream &stream, const  Preamble* preamble );
 
         QString valueToString( const Value *value, const EntryField::FieldType fieldType = EntryField::ftUnknown );
 
         void escapeLaTeXChars( QString &text );
         QString applyKeywordCasing( const QString &keyword );
         bool requiresPersonQuoting( const QString &text, bool isLastName );
+        void addProtectiveCasing( QString &text );
     };
 
 }
