@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -40,7 +40,7 @@ namespace BibTeX
         delete wc;
     }
 
-    bool FileExporterExternal::save( QIODevice* iodevice, File* bibtexfile, QStringList *errorLog )
+    bool FileExporterExternal::save( QIODevice* iodevice, const File* bibtexfile, QStringList *errorLog )
     {
         QBuffer buffer;
         buffer.open( IO_WriteOnly );
@@ -55,7 +55,7 @@ namespace BibTeX
         return result;
     }
 
-    bool FileExporterExternal::save( QIODevice* iodevice, Element* element, QStringList *errorLog )
+    bool FileExporterExternal::save( QIODevice* iodevice, const Element* element, QStringList *errorLog )
     {
         QBuffer buffer;
         buffer.open( IO_WriteOnly );
