@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -41,7 +41,7 @@ namespace BibTeX
         // nothing
     }
 
-    bool FileExporterPS::save( QIODevice* iodevice, File* bibtexfile, QStringList *errorLog )
+    bool FileExporterPS::save( QIODevice* iodevice, const File* bibtexfile, QStringList *errorLog )
     {
         bool result = FALSE;
 
@@ -60,7 +60,7 @@ namespace BibTeX
         return result;
     }
 
-    bool FileExporterPS::save( QIODevice* iodevice, Element* element, QStringList *errorLog )
+    bool FileExporterPS::save( QIODevice* iodevice, const Element* element, QStringList *errorLog )
     {
         bool result = FALSE;
 
@@ -91,7 +91,7 @@ namespace BibTeX
 
     bool FileExporterPS::generatePS( QIODevice* iodevice, QStringList *errorLog )
     {
-        QStringList cmdLines = QStringList::split( '|', "latex bibtex-to-ps.tex|bibtex bibtex-to-ps|latex bibtex-to-ps.tex|latex bibtex-to-ps.tex|dvips -o bibtex-to-ps.ps bibtex-to-ps.dvi" );
+        QStringList cmdLines = QStringList::split( '|', "latex -halt-on-error bibtex-to-ps.tex|bibtex bibtex-to-ps|latex -halt-on-error bibtex-to-ps.tex|latex -halt-on-error bibtex-to-ps.tex|dvips -o bibtex-to-ps.ps bibtex-to-ps.dvi" );
 
         if ( writeLatexFile( laTeXFilename ) && runProcesses( cmdLines, errorLog ) && writeFileToIODevice( outputFilename, iodevice ) )
             return TRUE;
