@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -86,10 +86,10 @@ namespace BibTeX
 
             result = m_process->normalExit() && counter < 400;
             if ( !result )
-                errorLog->append( QString( "Process \"%1\" failed." ).arg( args.join( " " ) ) );
+                errorLog->append( QString( "Process '%1' failed." ).arg( args.join( " " ) ) );
         }
         else
-            errorLog->append( QString( "Process \"%1\" was not started." ).arg( args.join( " " ) ) );
+            errorLog->append( QString( "Process '%1' was not started." ).arg( args.join( " " ) ) );
 
 
         disconnect( m_process, SIGNAL( readyReadStdout() ), this, SLOT( slotReadProcessOutput() ) );
