@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -42,7 +42,7 @@ namespace BibTeX
         // nothing
     }
 
-    bool FileExporterXML::save( QIODevice* iodevice, File* bibtexfile, QStringList * /*errorLog*/ )
+    bool FileExporterXML::save( QIODevice* iodevice, const File* bibtexfile, QStringList * /*errorLog*/ )
     {
         bool result = TRUE;
         m_cancelFlag = FALSE;
@@ -52,7 +52,7 @@ namespace BibTeX
         stream << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
         stream << "<bibliography>" << endl;
 
-        for ( File::ElementList::iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !m_cancelFlag; it++ )
+        for ( File::ElementList::const_iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !m_cancelFlag; it++ )
         {
             Element *element = *it;
             write( stream, element, bibtexfile );
@@ -63,7 +63,7 @@ namespace BibTeX
         return result && !m_cancelFlag;
     }
 
-    bool FileExporterXML::save( QIODevice* iodevice, Element* element, QStringList * /*errorLog*/ )
+    bool FileExporterXML::save( QIODevice* iodevice, const Element* element, QStringList * /*errorLog*/ )
     {
         QTextStream stream( iodevice );
         stream.setEncoding( QTextStream::UnicodeUTF8 );
@@ -77,17 +77,16 @@ namespace BibTeX
         m_cancelFlag = TRUE;
     }
 
-    bool FileExporterXML::write( QTextStream&stream, Element *element, File* bibtexfile )
+    bool FileExporterXML::write( QTextStream&stream, const Element *element, const File* bibtexfile )
     {
         bool result = FALSE;
 
-        Entry *entry = dynamic_cast<Entry*>( element );
+        const Entry *entry = dynamic_cast<const Entry*>( element );
         if ( entry != NULL )
         {
             if ( bibtexfile != NULL )
             {
-                entry = new BibTeX::Entry( entry );
-                bibtexfile->completeReferencedFields( entry );
+                entry = bibtexfile->completeReferencedFieldsConst( entry );
             }
             result |= writeEntry( stream, entry );
             if ( bibtexfile != NULL )
@@ -95,12 +94,12 @@ namespace BibTeX
         }
         else
         {
-            Macro * macro = dynamic_cast<Macro*>( element );
+            const Macro * macro = dynamic_cast<const Macro*>( element );
             if ( macro != NULL )
                 result |= writeMacro( stream, macro );
             else
             {
-                Comment * comment = dynamic_cast<Comment*>( element );
+                const Comment * comment = dynamic_cast<const Comment*>( element );
                 if ( comment != NULL )
                     result |= writeComment( stream, comment );
                 else
@@ -113,10 +112,10 @@ namespace BibTeX
         return result;
     }
 
-    bool FileExporterXML::writeEntry( QTextStream &stream, Entry* entry )
+    bool FileExporterXML::writeEntry( QTextStream &stream, const Entry* entry )
     {
         stream << " <entry id=\"" << entry->id() << "\" type=\"" << entry->entryTypeString().lower() << "\">" << endl;
-        for ( Entry::EntryFields::iterator it = entry->begin(); it != entry->end(); it++ )
+        for ( Entry::EntryFields::const_iterator it = entry->begin(); it != entry->end(); it++ )
         {
             EntryField *field = *it;
             switch ( field->fieldType() )
@@ -135,18 +134,39 @@ namespace BibTeX
             case EntryField::ftMonth:
                 {
                     stream << "  <month";
-                    QString monthText = field->value()->text();
                     bool ok = FALSE;
 
-                    for ( int i = 0; i < 12 && !ok; i++ )
-                        if ( QString::compare( monthText, MonthsTriple[ i ] ) == 0 )
+                    int month = -1;
+                    QString tag = "";
+                    QString content = "";
+                    for ( QValueList<ValueItem*>::ConstIterator it = field->value()->items.begin(); it != field->value()->items.end(); ++it )
+                    {
+                        if ( dynamic_cast<MacroKey*>( *it ) != NULL )
                         {
-                            stream << " tag=\"" << MonthsTriple[ i ] << "\">" << Months[ i ];
-                            ok = TRUE;
+                            for ( int i = 0; i < 12; i++ )
+                                if ( QString::compare(( *it )->text(), MonthsTriple[ i ] ) == 0 )
+                                {
+                                    if ( month < 1 )
+                                    {
+                                        tag = MonthsTriple[ i ];
+                                        month = i + 1;
+                                    }
+                                    content.append( Months[ i ] );
+                                    ok = TRUE;
+                                    break;
+                                }
                         }
+                        else
+                            content.append( EncoderXML::currentEncoderXML() ->encode(( *it )->text() ) );
+                    }
 
                     if ( !ok )
-                        stream << '>' << EncoderXML::currentEncoderXML() ->encode( field->value()->simplifiedText() ) ;
+                        content = EncoderXML::currentEncoderXML() ->encode( field->value()->simplifiedText() ) ;
+                    if ( !tag.isEmpty() )
+                        stream << " tag=\"" << tag << "\"";
+                    if ( month > 0 )
+                        stream << " month=\"" << month << "\"";
+                    stream << '>' << content;
                     stream << "</month>" << endl;
                 }
                 break;
@@ -164,7 +184,7 @@ namespace BibTeX
         return TRUE;
     }
 
-    bool FileExporterXML::writeMacro( QTextStream &stream, Macro *macro )
+    bool FileExporterXML::writeMacro( QTextStream &stream, const Macro *macro )
     {
         stream << " <string key=\"" << macro->key() << "\">";
         stream << EncoderXML::currentEncoderXML() ->encode( valueToString( macro->value() ) );
@@ -173,7 +193,7 @@ namespace BibTeX
         return TRUE;
     }
 
-    bool FileExporterXML::writeComment( QTextStream &stream, Comment *comment )
+    bool FileExporterXML::writeComment( QTextStream &stream, const Comment *comment )
     {
         stream << " <comment>" ;
         stream << EncoderXML::currentEncoderXML() ->encode( comment->text() );
