@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -35,7 +35,7 @@
 
 namespace BibTeX
 {
-    const QString extraAlphaNumChars = QString( "?'`-_:.+/$\\\"" );
+    const QString extraAlphaNumChars = QString( "?'`-_:.+/$\\\"&" );
 
     FileImporterBibTeX::FileImporterBibTeX( bool personFirstNameFirst ) : FileImporter(), m_personFirstNameFirst( personFirstNameFirst ), m_currentChar( ' ' ), m_ignoreComments( FALSE )
     {
@@ -97,7 +97,7 @@ namespace BibTeX
     bool FileImporterBibTeX::guessCanDecode( const QString & rawText )
     {
         QString text = EncoderLaTeX::currentEncoderLaTeX() ->decode( rawText );
-        return text.contains( QRegExp( "@\\w+\\{.+\\}" ) );
+        return text.find( QRegExp( "@\\w+\\{.+\\}" ) ) >= 0;
     }
 
     void FileImporterBibTeX::setIgnoreComments( bool ignoreComments )
@@ -123,8 +123,13 @@ namespace BibTeX
                 return readMacroElement();
             else if ( elementType.lower() == "preamble" )
                 return readPreambleElement();
-            else
+            else if ( !elementType.isEmpty() )
                 return readEntryElement( elementType );
+            else
+            {
+                qDebug( "ElementType is empty" );
+                return NULL;
+            }
         }
         else if ( token == tUnknown )
             return readPlainCommentElement();
@@ -173,7 +178,7 @@ namespace BibTeX
         QString key = readSimpleString();
         if ( nextToken() != tAssign )
         {
-            qDebug( "Error in parsing macro \"%s\": Assign symbol (=) expected", key.latin1() );
+            qDebug( "Error in parsing macro '%s': Assign symbol (=) expected", key.latin1() );
             return NULL;
         }
 
@@ -247,7 +252,7 @@ namespace BibTeX
                 break;
             else if ( token != tComma )
             {
-                qDebug( "Error in parsing entry \"%s\": Comma symbol (,) expected", key.latin1() );
+                qDebug( "Error in parsing entry '%s': Comma symbol (,) expected", key.latin1() );
                 delete entry;
                 return NULL;
             }
@@ -261,7 +266,7 @@ namespace BibTeX
             }
             else if ( token != tAssign )
             {
-                qDebug( "Error in parsing entry \"%s\": Assign symbol (=) expected", key.latin1() );
+                qDebug( "Error in parsing entry '%s': Assign symbol (=) expected", key.latin1() );
                 delete entry;
                 return NULL;
             }
@@ -404,7 +409,7 @@ namespace BibTeX
     QString FileImporterBibTeX::readLine()
     {
         QString result;
-        while ( m_currentChar != '\n' )
+        while ( !m_textStream->atEnd() && m_currentChar != '\n' )
         {
             result.append( m_currentChar );
             *m_textStream >> m_currentChar;
@@ -472,6 +477,8 @@ namespace BibTeX
                     }
                 }
                 break;
+            case EntryField::ftPages:
+                text.replace( QRegExp( "\\s*--?\\s*" ), QChar( 0x2013 ) );
             default:
                 {
                     if ( isStringKey )
