@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -69,7 +69,7 @@ namespace BibTeX
 
     bool FileImporterRIS::guessCanDecode( const QString & text )
     {
-        return text.contains( "TY  - " );
+        return text.find( "TY  - " ) >= 0;
     }
 
     void FileImporterRIS::cancel()
@@ -128,7 +128,7 @@ namespace BibTeX
                 if ( date.isEmpty() )
                     date = ( *it ).value;
             }
-            else if (( *it ).key == "N1" || ( *it ).key == "AB" )
+            else if (( *it ).key == "N1" || ( *it ).key == "N2" )
             {
                 BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftNote );
                 if ( field == NULL )
@@ -140,7 +140,7 @@ namespace BibTeX
                 value->items.append( new BibTeX::PlainText(( *it ).value ) );
                 field->setValue( value );
             }
-            else if (( *it ).key == "N2" )
+            else if (( *it ).key == "AB" )
             {
                 BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftAbstract );
                 if ( field == NULL )
