@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -64,15 +64,6 @@
 #include <fileexporterbibtex.h>
 #include <fileexporterxml.h>
 #include <fileexporterxslt.h>
-#include <webquery.h>
-#include <webquerypubmed.h>
-#include <webquerybibsonomy.h>
-// #include <webquerygooglescholar.h>
-#include <webqueryamatex.h>
-#include <webqueryspireshep.h>
-#include <webquerycitebase.h>
-#include <webquerydblp.h>
-#include <webqueryzmath.h>
 #include <settings.h>
 #include <encoderlatex.h>
 #include "kbibtex_part.h"
@@ -97,7 +88,7 @@ KBibTeXPart::KBibTeXPart( QWidget *parentWidget, const char *widgetName,
     setupGUI( parentWidget, widgetName );
 
     // create our actions
-    setupActions( parentWidget );
+    setupActions();
 
     // we are read-write by default
     setReadWrite( TRUE );
@@ -124,12 +115,12 @@ void KBibTeXPart::setReadWrite( bool rw )
     if ( rw )
     {
         connect( m_documentWidget, SIGNAL( modified( ) ), this, SLOT( setModified( ) ) );
-        m_actionEditElement->setText( "&Edit" );
+        m_actionEditElement->setText( i18n( "&Edit" ) );
     }
     else
     {
         disconnect( m_documentWidget, SIGNAL( modified( ) ), this, SLOT( setModified( ) ) );
-        m_actionEditElement->setText( "&View" );
+        m_actionEditElement->setText( i18n( "&View" ) );
     }
     ReadWritePart::setReadWrite( rw );
     m_documentWidget->setReadOnly( !rw );
@@ -162,7 +153,7 @@ bool KBibTeXPart::queryClose()
     if ( docName.isEmpty() ) docName = i18n( "Untitled" );
 
     int res = KMessageBox::warningYesNoCancel( widget(),
-              i18n( "The document \"%1\" has been modified.\n"
+              i18n( "The document '%1' has been modified.\n"
                     "Do you want to save your changes or discard them?" ).arg( docName ),
               i18n( "Close Document" ), KStdGuiItem::save(), KStdGuiItem::discard() );
 
@@ -234,14 +225,14 @@ bool KBibTeXPart::saveAs()
     QString startDir = ! url().isEmpty() ? url().url() : QDir::currentDirPath();
     KURL saveURL = KFileDialog::getSaveURL( startDir, "*.bib|" + i18n( "BibTeX (*.bib)" ) + "\n*.ris|" + i18n( "Reference Manager (*.ris)" )
 #ifdef HAVE_LIBBIBUTILS
-                                            + "\n*.enl|" + i18n( "EndNote (text only) (*.enl)" )
+                                            + "\n*.ref *.refer *.rfr *.txt|" + i18n( "EndNote (Refer format) (*.ref *.refer *.rfr *.txt)" ) + "\n*.isi *.cgi|" + i18n( "ISI Web of Knowledge (*.isi *.cgi)" )
 #endif
                                             , widget() );
 
     if ( saveURL.isValid() && !saveURL.isEmpty() )
     {
 
-        if ( KIO::NetAccess::exists( saveURL, FALSE, widget() ) && KMessageBox::warningContinueCancel( widget(), i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" ).arg( saveURL.filename() ), QString::null, i18n( "Overwrite" ) ) != KMessageBox::Continue )
+        if ( KIO::NetAccess::exists( saveURL, FALSE, widget() ) && KMessageBox::warningContinueCancel( widget(), i18n( "A file named '%1' already exists. Are you sure you want to overwrite it?" ).arg( saveURL.filename() ), QString::null, i18n( "Overwrite" ) ) != KMessageBox::Continue )
             return result;
 
         if ( ReadWritePart::saveAs( saveURL ) )
@@ -259,33 +250,35 @@ void KBibTeXPart::slotFileExport()
     QString startDir = ! url().isEmpty() ? url().url() : QDir::currentDirPath();
     KURL exportURL = KFileDialog::getSaveURL( startDir, "*.pdf|" + i18n( "Portable Document File (*.pdf)" ) + "\n*.ps|" + i18n( "PostScript (*.ps)" ) + "\n*.ris|" + i18n( "Reference Manager (*.ris)" )
 #ifdef HAVE_LIBBIBUTILS
-                     + "\n*.enl|" + i18n( "EndNote (text only) (*.enl)" )
+                     + "\n*.ref *.refer *.rfr *.txt|" + i18n( "EndNote (Refer format) (*.ref *.refer *.rfr *.txt)" ) + "\n*.isi *.cgi|" + i18n( "ISI Web of Knowledge (*.isi *.cgi)" )
 #endif
                      + "\n*.xml|" + i18n( "eXtended Markup Language (*.xml)" ) + "\n*.html|" + i18n( "Hypertext Markup Langage (*.html)" ) + "\n*.rtf|" + i18n( "Rich Text Format (*.rtf)" ), widget() );
 
     if ( exportURL.isValid() && !exportURL.isEmpty() )
     {
-        if ( KIO::NetAccess::exists( exportURL, FALSE, widget() ) && KMessageBox::warningContinueCancel( widget(), i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" ).arg( exportURL.filename() ), QString::null, i18n( "Overwrite" ) ) != KMessageBox::Continue )
+        if ( KIO::NetAccess::exists( exportURL, FALSE, widget() ) && KMessageBox::warningContinueCancel( widget(), i18n( "A file named '%1' already exists. Are you sure you want to overwrite it?" ).arg( exportURL.filename() ), QString::null, i18n( "Overwrite" ) ) != KMessageBox::Continue )
             return ;
 
         KTempFile tempBibFile = KTempFile( locateLocal( "tmp", "bibexport" ) );
         tempBibFile.setAutoDelete( TRUE );
         BibTeX::File::FileFormat format = BibTeX::File::formatBibTeX;
-        if ( exportURL.fileName().endsWith( ".rtf" ) )
+        if ( exportURL.fileName().endsWith( ".rtf", FALSE ) )
             format = BibTeX::File::formatRTF;
-        else if ( exportURL.fileName().endsWith( ".pdf" ) )
+        else if ( exportURL.fileName().endsWith( ".pdf", FALSE ) )
             format = BibTeX::File::formatPDF;
-        else if ( exportURL.fileName().endsWith( ".ps" ) )
+        else if ( exportURL.fileName().endsWith( ".ps", FALSE ) )
             format = BibTeX::File::formatPS;
 #ifdef HAVE_LIBBIBUTILS
-        else if ( exportURL.fileName().endsWith( ".enl" ) )
+        else if ( exportURL.fileName().endsWith( ".ref", FALSE ) || exportURL.fileName().endsWith( ".refer", FALSE ) || exportURL.fileName().endsWith( ".txt", FALSE ) || exportURL.fileName().endsWith( ".rfr", FALSE ) )
             format = BibTeX::File::formatEndNote;
+        else if ( exportURL.fileName().endsWith( ".isi", FALSE ) || exportURL.fileName().endsWith( ".cgi", FALSE ) )
+            format = BibTeX::File::formatISI;
 #endif
-        else if ( exportURL.fileName().endsWith( ".ris" ) )
+        else if ( exportURL.fileName().endsWith( ".ris", FALSE ) )
             format = BibTeX::File::formatRIS;
-        else if ( exportURL.fileName().endsWith( ".xml" ) )
+        else if ( exportURL.fileName().endsWith( ".xml", FALSE ) )
             format = BibTeX::File::formatXML;
-        else if ( exportURL.fileName().endsWith( ".html" ) )
+        else if ( exportURL.fileName().endsWith( ".html", FALSE ) )
             format = BibTeX::File::formatHTML;
 
         QStringList errorLog;
@@ -301,10 +294,10 @@ void KBibTeXPart::slotFileExport()
         {
 #if (KDE_VERSION_MINOR>=4) && (KDE_VERSION_MAJOR>=3)
             if ( !errorLog.empty() )
-                KMessageBox::errorList( widget(), QString( i18n( "File could not be exported in \"%1\"." ) ).arg( exportURL.prettyURL() ).append( i18n( "\n\nSee the following error log for details." ) ), errorLog, i18n( "Exporting failed" ) );
+                KMessageBox::errorList( widget(), QString( i18n( "File could not be exported in '%1'." ) ).arg( exportURL.prettyURL() ).append( i18n( "\n\nSee the following error log for details." ) ), errorLog, i18n( "Exporting failed" ) );
             else
 #endif
-                KMessageBox::error( widget(), QString( i18n( "File could not be exported to \"%1\"." ) ).arg( exportURL.prettyURL() ), i18n( "Exporting failed" ) );
+                KMessageBox::error( widget(), QString( i18n( "File could not be exported to '%1'." ) ).arg( exportURL.prettyURL() ), i18n( "Exporting failed" ) );
         }
 
     }
@@ -321,29 +314,35 @@ void KBibTeXPart::slotFileMerge()
     QString startDir = ! url().isEmpty() ? url().url() : QDir::currentDirPath();
     KURL mergeURL = KFileDialog::getOpenURL( startDir, QString( "*.bib *.ris" ) +
 #ifdef HAVE_LIBBIBUTILS
-                    " *.enl" +
+                    " *.ref *.refer *.rfr *.txt *.isi *.cgi" +
 #endif
                     "|" + i18n( "Supported Bibliographies" ) + "\n*.bib|" + i18n( "BibTeX (*.bib)" ) + "\n*.ris|" + i18n( "Reference Manager (*.ris)" )
 #ifdef HAVE_LIBBIBUTILS
-                    + "\n*.enl|" + i18n( "EndNote (text only) (*.enl)" )
+                    + "\n*.ref *.refer *.rfr *.txt|" + i18n( "EndNote (Refer format) (*.ref *.refer *.rfr *.txt)" ) + "\n*.isi *.cgi|" + i18n( "ISI Web of Knowledge (*.isi *.cgi)" )
 #endif
                     , widget() );
 
     if ( !mergeURL.isValid() || mergeURL.isEmpty() )
         return;
 
+    QString extension = mergeURL.fileName();
+    int extPos = extension.find( '.' );
+    if ( extPos < 0 )
+        return;
+    extension = extension.mid( extPos );
+
     if ( !KIO::NetAccess::exists( mergeURL, TRUE, widget() ) )
     {
         KMessageBox::error( widget(), i18n( "The given file could not be read, check if it exists or if it is readable for the current user." ) );
         return ;
     }
 
-    KTempFile tempBibFile = KTempFile( locateLocal( "tmp", "bibmerge" ) );
+    KTempFile tempBibFile = KTempFile( locateLocal( "tmp", "bibmerge" ), extension );
     tempBibFile.setAutoDelete( TRUE );
     bool error = !KIO::NetAccess::file_copy( mergeURL, KURL( tempBibFile.name() ),  -1, TRUE, FALSE, widget() );
 
     if ( !error )
-        error = !m_documentWidget->open( tempBibFile.file(), TRUE );
+        error = !m_documentWidget->open( tempBibFile.name(), TRUE );
 
     tempBibFile.close();
 
@@ -374,7 +373,7 @@ void KBibTeXPart::setupGUI( QWidget *parentWidget, const char *name )
     setWidget( m_documentWidget );
 }
 
-void KBibTeXPart::setupActions( QWidget *parentWidget )
+void KBibTeXPart::setupActions()
 {
     if ( isReadWrite() )
     {
@@ -389,14 +388,19 @@ void KBibTeXPart::setupActions( QWidget *parentWidget )
 
     m_actionEditCut = KStdAction::cut( m_documentWidget, SLOT( cutElements() ), actionCollection() );
     m_actionEditCut->setToolTip( i18n( "Cut the selected BibTeX elements to the clipboard" ) );
+    m_actionEditCut->setEnabled( FALSE );
     m_actionEditCopy = KStdAction::copy( m_documentWidget, SLOT( copyElements() ), actionCollection() );
     m_actionEditCopy->setToolTip( i18n( "Copy the selected BibTeX elements to the clipboard" ) );
-    m_actionEditUndo = new KAction( i18n( "Undo" ), CTRL + Key_Z, m_documentWidget, SLOT( undoEditor() ), actionCollection(), "edit_undo" );
+    m_actionEditCopy->setEnabled( FALSE );
+    m_actionEditUndo =  KStdAction::undo( m_documentWidget, SLOT( undoEditor() ), actionCollection() );
     connect( m_documentWidget, SIGNAL( undoChanged( bool ) ), this, SLOT( slotUndoChanged( bool ) ) );
     m_actionEditCopyRef = new KAction( i18n( "Copy &Reference" ), CTRL + SHIFT + Key_C, m_documentWidget, SLOT( copyRefElements() ), actionCollection(), "edit_copyref" );
     m_actionEditCopyRef->setToolTip( i18n( "Copy a reference of the selected BibTeX elements to the clipboard" ) );
+    m_actionEditCopyRef->setEnabled( FALSE );
     m_actionEditPaste = KStdAction::paste( m_documentWidget, SLOT( pasteElements() ), actionCollection() );
     m_actionEditPaste->setToolTip( i18n( "Paste BibTeX elements from the clipboard" ) );
+    m_actionEditSelectAll = KStdAction::selectAll( m_documentWidget, SLOT( selectAll() ), actionCollection() );
+    m_actionEditSelectAll->setToolTip( i18n( "Select all BibTeX elements" ) );
     m_actionElementSendToLyX = new KAction( i18n( "Send Reference to &LyX" ), CTRL + SHIFT + Key_L, m_documentWidget, SLOT( sendSelectedToLyx() ), actionCollection(), "element_sendtolyx" );
     m_actionElementSendToLyX->setToolTip( i18n( "Send a reference of the selected BibTeX elements to LyX" ) );
     m_actionElementSendToLyX->setEnabled( FALSE );
@@ -407,39 +411,14 @@ void KBibTeXPart::setupActions( QWidget *parentWidget )
     m_actionEditFind->setEnabled( FALSE );
     m_actionEditFindNext->setEnabled( FALSE );
 
+    m_actionSearchOnlineDatabases = new KAction( i18n( "Search Online Databases" ), SmallIconSet( "network" ), CTRL + SHIFT + Key_S, m_documentWidget, SLOT( onlineSearch() ), actionCollection(), "search_onlinedatabases" );
+    m_actionSearchOnlineDatabases->setToolTip( i18n( "Search online databases to import bibliography data" ) );
+
     m_actionMenuSearchWebsites = new KActionMenu( i18n( "Search Document Online" ), actionCollection(), "search_document_online" );
     m_actionMenuSearchWebsites->setToolTip( i18n( "Search websites for the selected BibTeX element" ) );
     connect( m_actionMenuSearchWebsites->popupMenu(), SIGNAL( activated( int ) ), this, SLOT( slotSearchWebsites( int ) ) );
     m_actionMenuSearchWebsites->setEnabled( FALSE );
 
-    m_actionMenuSearchOnlineDatabases = new KActionMenu( i18n( "Search Online Databases" ), SmallIconSet( "network" ), actionCollection(), "search_onlinedatabases" );
-    m_actionMenuSearchOnlineDatabases->setToolTip( i18n( "Search online databases to import bibliography data" ) );
-    connect( m_actionMenuSearchOnlineDatabases->popupMenu(), SIGNAL( activated( int ) ), this, SLOT( slotSearchOnlineDatabases( int ) ) );
-    KBibTeX::WebQuery *webQuery = new KBibTeX::WebQueryPubMed( parentWidget );
-    webQueryList.append( webQuery );
-    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
-    webQuery = new KBibTeX::WebQueryBibSonomy( parentWidget );
-    webQueryList.append( webQuery );
-    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
-    webQuery = new KBibTeX::WebQuerySpiresHep( parentWidget );
-    webQueryList.append( webQuery );
-    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
-    webQuery = new KBibTeX::WebQueryZMATH( parentWidget );
-    webQueryList.append( webQuery );
-    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
-    webQuery = new KBibTeX::WebQueryCitebase( parentWidget );
-    webQueryList.append( webQuery );
-    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
-    webQuery = new KBibTeX::WebQueryDBLP( parentWidget );
-    webQueryList.append( webQuery );
-    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
-//     webQuery = new KBibTeX::WebQueryGoogleScholar( parentWidget );
-//     webQueryList.append( webQuery );
-//     m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
-    webQuery = new KBibTeX::WebQueryAmatex( parentWidget );
-    webQueryList.append( webQuery );
-    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
-
     m_actionViewShowColumns = new KActionMenu( i18n( "&Show Columns" ), actionCollection(), "view_showcolumns" );
     m_actionViewShowColumns->setToolTip( i18n( "Select columns to show in the main list" ) );
     m_actionViewDocument = new KActionMenu( i18n( "View Document" ), actionCollection(), "view_document" );
@@ -452,9 +431,9 @@ void KBibTeXPart::setupActions( QWidget *parentWidget )
     m_actionPreferences = KStdAction::preferences( this, SLOT( slotPreferences() ), actionCollection() );
     m_actionPreferences->setToolTip( i18n( "Configure KBibTeX" ) );
     KAction *actionSourceEditor = new KAction( i18n( "Configure Source View..." ), 0, m_documentWidget, SLOT( configureEditor() ), actionCollection(), "options_configure_editor" );
-    actionSourceEditor ->setToolTip( i18n( "Configure the Source Vditor" ) );
+    actionSourceEditor ->setToolTip( i18n( "Configure the Source Editor" ) );
 
-    ( new KAction( i18n( "Article" ), "element_new_entry_article", CTRL + Key_A, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_article" ) ) ->setToolTip( i18n( "Add an article to the BibTeX file" ) );
+    ( new KAction( i18n( "Article" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_article" ) ) ->setToolTip( i18n( "Add an article to the BibTeX file" ) );
     ( new KAction( i18n( "Book" ), "element_new_entry_book", CTRL + Key_B, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_book" ) ) ->setToolTip( i18n( "Add a book to the BibTeX file" ) );
     ( new KAction( i18n( "Booklet" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_booklet" ) ) ->setToolTip( i18n( "Add a booklet to the BibTeX file" ) );
     ( new KAction( i18n( "Electronic (IEEE)" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_electronic" ) ) ->setToolTip( i18n( "Add an electronic reference (IEEE) to the BibTeX file" ) );
@@ -584,7 +563,7 @@ void KBibTeXPart::slotUseInPipe()
 {
     if ( m_inPipe == NULL && QFile::exists( inPipeFilename ) )
     {
-        KMessageBox::error( widget(), QString( i18n( "Some other KBibTeX instance is using the pipe.\nIf this assumption is wrong, please delete \"%1\"." ) ).arg( inPipeFilename ), i18n( "Pipe already in use" ) );
+        KMessageBox::error( widget(), QString( i18n( "Some other KBibTeX instance is using the pipe.\nIf this assumption is wrong, please delete '%1'." ) ).arg( inPipeFilename ), i18n( "Pipe already in use" ) );
     }
     else
     {
@@ -595,7 +574,7 @@ void KBibTeXPart::slotUseInPipe()
             m_inPipe->remove();
             delete m_inPipe;
             m_inPipe = NULL;
-            QFile::remove ( inPipeFilename );
+            QFile::remove( inPipeFilename );
         }
         else
         {
@@ -609,11 +588,11 @@ void KBibTeXPart::slotUseInPipe()
                     m_inPipe = NULL;
                 }
                 if ( m_inPipe == NULL )
-                    QFile::remove ( inPipeFilename );
+                    QFile::remove( inPipeFilename );
             }
 
             if ( m_inPipe == NULL )
-                KMessageBox::error( widget(), QString( i18n( "Could not create pipe at \"%1\"." ) ).arg( inPipeFilename ), i18n( "Error creating pipe" ) );
+                KMessageBox::error( widget(), QString( i18n( "Could not create pipe at '%1'." ) ).arg( inPipeFilename ), i18n( "Error creating pipe" ) );
         }
     }
 
@@ -626,17 +605,6 @@ void KBibTeXPart::slotSearchWebsites( int id )
     m_documentWidget->searchWebsites( settings->searchURLs[ id - 1 ] ->url, settings->searchURLs[ id - 1 ] ->includeAuthor );
 }
 
-void KBibTeXPart::slotSearchOnlineDatabases( int id )
-{
-    for ( QValueList<KBibTeX::WebQuery*>::ConstIterator it = webQueryList.begin(); it != webQueryList.end(); ++it )
-        if (( *it ) ->uniqueId() == id )
-        {
-            m_documentWidget->doWebQuery( *it );
-            break;
-        }
-
-}
-
 void KBibTeXPart::slotToggleShowSpecialElements()
 {
     KBibTeX::Settings * settings = KBibTeX::Settings::self();
@@ -659,7 +627,6 @@ void KBibTeXPart::slotDeferredInitialization()
     if ( factory() != NULL )
     {
         m_documentWidget->setFactory( factory(), this );
-        m_documentWidget->setOnlineDatabaseSearchMenu( m_actionMenuSearchOnlineDatabases->popupMenu() );
         m_documentWidget->deferredInitialization();
         m_initializationDone = TRUE;
     }
@@ -667,15 +634,12 @@ void KBibTeXPart::slotDeferredInitialization()
     {
         m_defInitCounter++;
 
-        if ( m_defInitCounter > 50 )
+        if ( m_defInitCounter > 5 )
         {
-            KGuiItem retry = KGuiItem( i18n( "Try again" ), "reload" );
-            if ( KMessageBox::warningContinueCancel( widget(), i18n( "The KBibTeX part has problems to initialize itself." ), i18n( "KBibTeX Part" ), retry ) == KMessageBox::Cancel )
-                return ;
-            else
-                m_defInitCounter = 0;
+            KMessageBox::sorry( widget(), i18n( "The KBibTeX part has problems to initialize itself. Only limited functionality will be available." ), i18n( "KBibTeX Part" ) );
+            return ;
         }
-        QTimer::singleShot( 100, this, SLOT( slotDeferredInitialization() ) );
+        QTimer::singleShot( 250, this, SLOT( slotDeferredInitialization() ) );
     }
 }
 
@@ -719,7 +683,7 @@ KInstance* KBibTeXPartFactory::instance()
 {
     if ( !s_instance )
     {
-        s_about = new KAboutData( "kbibtexpart", I18N_NOOP( "KBibTeXPart" ), "0.2" );
+        s_about = new KAboutData( "kbibtexpart", I18N_NOOP( "KBibTeXPart" ), "0.2.1" );
         s_about->addAuthor( "Thomas Fischer", 0, "fischer@unix-ag.uni-kl.de" );
         s_instance = new KInstance( s_about );
     }
