@@ -83,7 +83,6 @@ protected slots:
 private slots:
     bool slotNewElement();
     void slotSearchWebsites( int id );
-    void slotSearchOnlineDatabases( int id );
     void slotToggleShowSpecialElements();
     void slotDeferredInitialization();
     void slotUpdateMenu( int numSelectedItems );
@@ -116,13 +115,14 @@ private:
     KAction *m_actionEditCopy;
     KAction *m_actionEditCopyRef;
     KAction *m_actionEditPaste;
+    KAction *m_actionEditSelectAll;
     KAction *m_actionEditFind;
     KAction *m_actionEditFindNext;
     KAction *m_actionPreferences;
+    KAction *m_actionSearchOnlineDatabases;
     KActionMenu *m_actionViewDocument;
     KActionMenu *m_actionAssignKeywords;
     KActionMenu *m_actionMenuSearchWebsites;
-    KActionMenu *m_actionMenuSearchOnlineDatabases;
     KActionMenu *m_actionViewShowColumns;
     KAction *m_actionSearchNCBI;
     KToggleAction *m_actionUseInPipe;
@@ -137,7 +137,7 @@ private:
     void writeSettings();
 
     void setupGUI( QWidget *parentWidget, const char *name );
-    void setupActions( QWidget *parentWidget );
+    void setupActions();
 
 signals:
     void progressCancel();
