@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -17,6 +17,9 @@
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
+#include <qregexp.h>
+#include <qstringlist.h>
+
 #include "macro.h"
 
 namespace BibTeX
@@ -44,12 +47,12 @@ namespace BibTeX
         m_key = key;
     }
 
-    QString Macro::key()
+    QString Macro::key() const
     {
         return m_key;
     }
 
-    Value *Macro::value()
+    Value *Macro::value() const
     {
         return m_value;
     }
@@ -67,14 +70,30 @@ namespace BibTeX
         }
     }
 
-    bool Macro::containsPattern( const QString& pattern, EntryField::FieldType fieldType, bool caseSensitive )
+    bool Macro::containsPattern( const QString& pattern, EntryField::FieldType fieldType, FilterType filterType, bool caseSensitive ) const
     {
-        if ( fieldType != EntryField::ftUnknown )
-            return FALSE;
+        QString text = QString( m_key ).append( m_value->simplifiedText() );
 
-        bool result = m_key.contains( pattern, caseSensitive ) || m_value->simplifiedText().contains( pattern, caseSensitive );
+        if ( filterType == ftExact )
+        {
+            /** check for exact match */
+            return fieldType == EntryField::ftUnknown && text.contains( pattern, caseSensitive );
+        }
+        else
+        {
+            /** for each word in the search pattern ... */
+            QStringList words = QStringList::split( QRegExp( "\\s+" ), pattern );
+            unsigned int hits = 0;
+            for ( QStringList::Iterator it = words.begin(); it != words.end(); ++it )
+            {
+                /** check if word is contained in text */
+                if ( fieldType == EntryField::ftUnknown && text.contains( *it, caseSensitive ) )
+                    ++hits;
+            }
 
-        return result;
+            /** return success depending on filter type and number of hits */
+            return ( filterType == ftAnyWord && hits > 0 || filterType == ftEveryWord && hits == words.count() );
+        }
     }
 
     Element* Macro::clone()
@@ -89,7 +108,7 @@ namespace BibTeX
         m_value = new Value( other->m_value );
     }
 
-    QString Macro::text()
+    QString Macro::text() const
     {
         return m_key + "=" + m_value->text();
     }
