@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2006 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -22,6 +22,9 @@
 #include <qstringlist.h>
 #include <qfont.h>
 #include <qregexp.h>
+#include <qwaitcondition.h>
+#include <qprocess.h>
+#include <qapplication.h>
 
 #include <kconfig.h>
 #include <kstandarddirs.h>
@@ -29,6 +32,7 @@
 #include <klocale.h>
 #include <kbibtex_part.h>
 #include <kdebug.h>
+#include <kio/netaccess.h>
 
 #include <entryfield.h>
 #include <element.h>
@@ -38,6 +42,9 @@
 
 namespace KBibTeX
 {
+    Settings* Settings::staticSettings = new Settings();;
+    QStringList Settings::m_lyxRcFileNames = QStringList::split( '|', QDir::home().canonicalPath() + "/.lyx/lyxrc" + '|' + QDir::home().canonicalPath() + "/.lyx/preferences" );
+
     Settings::Settings()
     {
         checkExternalToolsAvailable();
@@ -54,6 +61,10 @@ namespace KBibTeX
             m_completion[ i ] = new KCompletion();
             m_completion[ i ] ->setIgnoreCase( TRUE );
         }
+        completionMacro = new KCompletion();
+        completionMacro->setIgnoreCase( FALSE );
+
+        currentBibTeXFile = NULL;
     }
 
     Settings::~Settings()
@@ -62,31 +73,37 @@ namespace KBibTeX
         for ( int i = 0; i < numCompletions;i++ )
             delete m_completion[ i ];
         delete[] m_completion;
+        delete completionMacro;
 
         for ( QValueList<SearchURL*>::ConstIterator it = searchURLs.begin(); it != searchURLs.end(); ++it )
             delete *it;
     }
 
-    Settings* Settings::self()
+    Settings* Settings::self( BibTeX::File *bibtexFile )
     {
-        static Settings * result = new Settings( );
-        return result;
+        if ( bibtexFile != NULL )
+            staticSettings->currentBibTeXFile = bibtexFile;
+        return staticSettings;
     }
 
     void Settings::load( KConfig * config )
     {
         config->setGroup( "FileIO" );
-        fileIO_Encoding = ( BibTeX::File::Encoding ) ( config->readNumEntry( "Encoding", ( int ) ( BibTeX::File::encLaTeX - 1 ) ) + 1 );
+        fileIO_Encoding = ( BibTeX::File::Encoding )( config->readNumEntry( "Encoding", ( int )( BibTeX::File::encLaTeX - 1 ) ) + 1 );
         fileIO_ExportLanguage = config->readEntry( "ExportLanguage", "english" );
         fileIO_ExportBibliographyStyle = config->readEntry( "ExportBibliographyStyle", "plain" );
         fileIO_ExporterHTML = ( BibTeX::FileExporterExternal::Exporter ) config->readNumEntry( "ExporterHTML", ( int ) BibTeX::FileExporterExternal::exporterNone );
         fileIO_BibtexStringOpenDelimiter = config->readEntry( "BibtexStringOpenDelimiter", "\"" ).at( 0 );
         fileIO_BibtexStringCloseDelimiter = config->readEntry( "BibtexStringCloseDelimiter", "\"" ).at( 0 );
+        fileIO_KeywordCasing = ( BibTeX::FileExporterBibTeX::KeywordCasing )config->readNumEntry( "KeywordCasing", ( int )BibTeX::FileExporterBibTeX::kcCamelCase );
+        fileIO_EmbedFiles = config->readBoolEntry( "EmbedFiles", FALSE );
+        fileIO_EnclosingCurlyBrackets = config->readBoolEntry( "EnclosingCurlyBrackets", FALSE );
 
         config->setGroup( "Editing" );
+        editing_SearchBarClearField = config->readBoolEntry( "SearchBarClearField", false );
         editing_EnableAllFields = config->readBoolEntry( "EnableAllFields", false );
         editing_MainListDoubleClickAction = config->readNumEntry( "MainListDoubleClickAction", 0 );
-        editing_MainListSortingColumn = config->readNumEntry( "MainListSortingColumn", -1 );
+        editing_MainListSortingColumn = config->readNumEntry( "MainListSortingColumn", 1 );
         editing_MainListSortingOrder = config->readNumEntry( "MainListSortingOrder", 1 );
         editing_MainListColumnsWidth = config->readIntListEntry( "MainListColumnsWidth" );
         editing_MainListColumnsIndex = config->readIntListEntry( "MainListColumnsIndex" );
@@ -98,6 +115,9 @@ namespace KBibTeX
         QFont defaultFont = KGlobalSettings::generalFont();
         editing_SpecialFont = config->readFontEntry( "SpecialFont", &defaultFont );
         editing_UseSpecialFont = config->readBoolEntry( "UseSpecialFont", FALSE );
+        editing_FirstNameFirst = config->readBoolEntry( "FirstNameFirst", FALSE );
+        editing_DocumentSearchPath = config->readEntry( "DocumentSearchPath", "" );
+        editing_DragAction = (DragAction)config->readNumEntry( "DragAction", ( int )COPYREFERENCE );
 
         if ( editing_MainListColumnsWidth.isEmpty() || editing_MainListColumnsIndex.isEmpty() )
         {
@@ -113,7 +133,7 @@ namespace KBibTeX
 
             for ( int i = 0; i <= ( int ) BibTeX::EntryField::ftYear - ( int ) BibTeX::EntryField::ftAbstract; i++ )
             {
-                BibTeX::EntryField::FieldType fieldType = ( BibTeX::EntryField::FieldType ) ( i + ( int ) BibTeX::EntryField::ftAbstract );
+                BibTeX::EntryField::FieldType fieldType = ( BibTeX::EntryField::FieldType )( i + ( int ) BibTeX::EntryField::ftAbstract );
                 if ( fieldType == BibTeX::EntryField::ftAuthor || fieldType == BibTeX::EntryField::ftTitle )
                     editing_MainListColumnsWidth.append( 0xffff );
                 else
@@ -128,11 +148,13 @@ namespace KBibTeX
         {
             QString descr = config->readEntry( QString( "SearchURLDescription%1" ).arg( i ), QString::null );
             QString url = config->readEntry( QString( "SearchURL%1" ).arg( i ), QString::null );
+            bool includeAuthor = config->readBoolEntry( QString( "IncludeAuthor%1" ).arg( i ), FALSE );
             if ( descr != QString::null && url != QString::null )
             {
                 SearchURL * searchURL = new SearchURL;
                 searchURL->description = descr;
                 searchURL->url = url;
+                searchURL->includeAuthor = includeAuthor;
                 searchURLs.append( searchURL );
             }
             else
@@ -141,6 +163,45 @@ namespace KBibTeX
 
         if ( searchURLs.count() == 0 )
             restoreDefaultSearchURLs();
+
+        config->setGroup( "Keyword" );
+        keyword_GlobalList = config->readListEntry( "GlobalList" );
+        keyword_GlobalList.sort();
+
+        config->setGroup( "OnlineSearchDatabase" );
+        webQuery_LastEngine = config->readNumEntry( "LastEngine", 0 );
+        webQuery_LastSearchTerm = config->readEntry( "LastSearchTerm", "" );
+        webQuery_LastNumberOfResults = config->readNumEntry( "LastNumberOfResults", 10 );
+        webQuery_ImportAll = config->readBoolEntry( "ImportAll", FALSE );
+
+        config->setGroup( "IdSuggestions" );
+        idSuggestions_formatStrList = config->readListEntry( "FormatStrList" );
+        if ( idSuggestions_formatStrList.count() == 0 )
+        {
+            idSuggestions_formatStrList = QStringList::split( ',', "A,A2|y,A3|y,A4|y|\":|T5,al|\":|T,al|y,al|Y,Al\"-|\"-|y,Al\"+|Y,al|y|T,al|Y|T3,al|Y|T3l,a|\":|Y|\":|T1,a|y,A|\":|Y" );
+            idSuggestions_default = -1;
+            idSuggestions_forceDefault = FALSE;
+        }
+        else
+        {
+            idSuggestions_default = config->readNumEntry( "Default", -1 );
+            idSuggestions_forceDefault = config->readBoolEntry( "ForceDefault", FALSE );
+        }
+
+        config->setGroup( "UserDefinedInputFields" );
+        QStringList names = config->readListEntry( "Names" );
+        QStringList labels = config->readListEntry( "Labels" );
+        QStringList inputtypes = config->readListEntry( "InputTypes" );
+
+        userDefinedInputFields.clear();
+        for ( unsigned int i = 0; i < names.size();++i )
+        {
+            UserDefinedInputFields *udif = new UserDefinedInputFields();
+            udif->name = names[i];
+            udif->label = labels[i];
+            udif->inputType = ( inputtypes[i] == "single" ? FieldLineEdit::itSingleLine : FieldLineEdit::itMultiLine );
+            userDefinedInputFields << udif;
+        }
     }
 
     void Settings::save( KConfig * config )
@@ -152,8 +213,12 @@ namespace KBibTeX
         config->writeEntry( "ExporterHTML", ( int ) fileIO_ExporterHTML );
         config->writeEntry( "BibtexStringOpenDelimiter", QString( fileIO_BibtexStringOpenDelimiter ) );
         config->writeEntry( "BibtexStringCloseDelimiter", QString( fileIO_BibtexStringCloseDelimiter ) );
+        config->writeEntry( "KeywordCasing", ( int )fileIO_KeywordCasing );
+        config->writeEntry( "EmbedFiles", fileIO_EmbedFiles );
+        config->writeEntry( "EnclosingCurlyBrackets", fileIO_EnclosingCurlyBrackets );
 
         config->setGroup( "Editing" );
+        config->writeEntry( "SearchBarClearField", editing_SearchBarClearField );
         config->writeEntry( "EnableAllFields", editing_EnableAllFields );
         config->writeEntry( "MainListDoubleClickAction", editing_MainListDoubleClickAction );
         config->writeEntry( "MainListSortingColumn", editing_MainListSortingColumn );
@@ -167,14 +232,44 @@ namespace KBibTeX
         config->writeEntry( "VerticalSplitterSizes", editing_VertSplitterSizes );
         config->writeEntry( "SpecialFont", editing_SpecialFont );
         config->writeEntry( "UseSpecialFont", editing_UseSpecialFont );
+        config->writeEntry( "FirstNameFirst", editing_FirstNameFirst );
+        config->writeEntry( "DocumentSearchPath", editing_DocumentSearchPath );
+        config->writeEntry( "DragAction", editing_DragAction );
 
         config->setGroup( "SearchURLs" );
         int i = 1;
-        for ( QValueList<SearchURL*>::ConstIterator it = searchURLs.begin(); it != searchURLs.end(); ++it )
+        for ( QValueList<SearchURL*>::ConstIterator it = searchURLs.begin(); it != searchURLs.end(); ++it, ++i )
         {
             config->writeEntry( QString( "SearchURLDescription%1" ).arg( i ), ( *it ) ->description );
             config->writeEntry( QString( "SearchURL%1" ).arg( i ), ( *it ) ->url );
+            config->writeEntry( QString( "IncludeAuthor%1" ).arg( i ), ( *it ) ->includeAuthor );
         }
+
+        config->setGroup( "Keyword" );
+        config->writeEntry( "GlobalList", keyword_GlobalList );
+
+        config->setGroup( "OnlineSearchDatabase" );
+        config->writeEntry( "LastEngine", webQuery_LastEngine );
+        config->writeEntry( "LastSearchTerm", webQuery_LastSearchTerm );
+        config->writeEntry( "LastNumberOfResults", webQuery_LastNumberOfResults );
+        config->writeEntry( "ImportAll", webQuery_ImportAll );
+
+        config->setGroup( "IdSuggestions" );
+        config->writeEntry( "FormatStrList", idSuggestions_formatStrList );
+        config->writeEntry( "Default", idSuggestions_default );
+        config->writeEntry( "ForceDefault", idSuggestions_forceDefault );
+
+        config->setGroup( "UserDefinedInputFields" );
+        QStringList names, labels, inputtype;
+        for ( QValueList<UserDefinedInputFields*>::iterator it = userDefinedInputFields.begin(); it != userDefinedInputFields.end();++it )
+        {
+            names << ( *it )->name;
+            labels << ( *it )->label;
+            inputtype << (( *it )->inputType == FieldLineEdit::itMultiLine ? "multi" : "single" );
+        }
+        config->writeEntry( "Names", names );
+        config->writeEntry( "Labels", labels );
+        config->writeEntry( "InputTypes", inputtype );
     }
 
     void Settings::checkExternalToolsAvailable()
@@ -203,6 +298,65 @@ namespace KBibTeX
         return FALSE;
     }
 
+    QString Settings::detectLyXInPipe()
+    {
+        QString result = "";
+        for ( QStringList::Iterator it = m_lyxRcFileNames.begin(); result.isEmpty() && it != m_lyxRcFileNames.end(); ++it )
+        {
+            QString lyxRcFileName = *it;
+            QFile lyxRcFile( lyxRcFileName );
+
+            kdDebug() << "detectLyXInPipe:  looking for " << lyxRcFileName << endl;
+            if ( lyxRcFile.exists() && lyxRcFile.open( IO_ReadOnly ) )
+            {
+                QTextStream input( &lyxRcFile );
+                while ( result.isEmpty() && !input.atEnd() )
+                {
+                    QString line = input.readLine();
+                    if ( line.startsWith( "\\serverpipe ", FALSE ) )
+                    {
+                        QStringList cols = QStringList::split( QRegExp( "\\s+\"|\"" ), line );
+                        if ( cols.size() >= 2 )
+                        {
+                            result = cols[ 1 ] + ".in";
+                            kdDebug() << "detectLyXInPipe:  testing " << result << " from config file" << endl;
+                            if ( !QFile::exists( result ) )
+                                kdDebug() << "LyX in pipe '" << result << "' from " << lyxRcFileName << " does not exist" << endl;
+                        }
+                    }
+                }
+                lyxRcFile.close();
+            }
+        }
+
+        if ( result.isEmpty() )
+        {
+            result = QDir::home().canonicalPath() + "/.lyx/lyxpipe.in";
+            kdDebug() << "detectLyXInPipe:  testing " << result << endl;
+            if ( !QFile::exists( result ) )
+                result = "";
+        }
+
+        if ( result.isEmpty() )
+        {
+            result = QDir::home().canonicalPath() + "/.lyx/.lyxpipe.in";
+            kdDebug() << "detectLyXInPipe:  testing " << result << endl;
+            if ( !QFile::exists( result ) )
+                result = "";
+        }
+
+        if ( result.isEmpty() )
+        {
+            result = QDir::home().canonicalPath() + "/.lyxpipe.in";
+            kdDebug() << "detectLyXInPipe:  testing " << result << endl;
+            if ( !QFile::exists( result ) )
+                result = "";
+        }
+
+        kdDebug() << "detectLyXInPipe:  using " << ( result.isEmpty() ? "<EMPTY>" : result ) << endl;
+        return result;
+    }
+
     void Settings::addToCompletion( BibTeX::File *file )
     {
         for ( BibTeX::File::ElementList::iterator it = file->begin(); it != file->end(); it++ )
@@ -212,43 +366,44 @@ namespace KBibTeX
     void Settings::addToCompletion( BibTeX::Element *element )
     {
         BibTeX::Entry * entry = dynamic_cast<BibTeX::Entry*>( element );
+        BibTeX::Macro * macro = dynamic_cast<BibTeX::Macro*>( element );
         if ( entry != NULL )
         {
-            for ( BibTeX::Entry::EntryFields::const_iterator ite = entry->begin(); ite != entry->end(); ite++ )
+            for ( BibTeX::Entry::EntryFields::ConstIterator ite = entry->begin(); ite != entry->end(); ite++ )
             {
                 BibTeX::EntryField::FieldType fieldType = ( *ite ) ->fieldType();
-                int index = completionFieldTypeToIndex( fieldType );
-
                 BibTeX::Value *value = ( *ite ) ->value();
-                for ( QValueList<BibTeX::ValueItem*>::const_iterator itv = value->begin(); itv != value->end(); itv++ )
-                {
-                    BibTeX::ValueItem *item = *itv;
-                    if ( !item->isStringKey() )
-                    {
-                        if ( ( *ite ) ->fieldType() == BibTeX::EntryField::ftAuthor || ( *ite ) ->fieldType() == BibTeX::EntryField::ftEditor )
-                        {
-                            QStringList elements = QStringList::split( QRegExp( "\\s+(,|and|&)\\s+", FALSE ), item->text() );
-                            for ( QStringList::Iterator it = elements.begin(); it != elements.end(); it++ )
-                                m_completion[ index ] ->addItem( *it );
-                        }
-                        else
-                            m_completion[ index ] ->addItem( item->text() );
-                    }
-                }
+                addToCompletion( value, fieldType );
             }
         }
-        else
-        {
+        else if ( macro != NULL )
+            completionMacro->addItem( macro->key() );
+    }
 
-            BibTeX::Macro * macro = dynamic_cast<BibTeX::Macro*>( element );
-            if ( macro != NULL )
+    void Settings::addToCompletion( BibTeX::Value *value, BibTeX::EntryField::FieldType fieldType )
+    {
+        int index = completionFieldTypeToIndex( fieldType );
+
+        for ( QValueList<BibTeX::ValueItem*>::ConstIterator itv = value->items.begin(); itv != value->items.end(); itv++ )
+        {
+            BibTeX::PlainText *plainText = dynamic_cast<BibTeX::PlainText*>( *itv );
+            if ( plainText != NULL )
+                m_completion[ index ] ->addItem( plainText->text() );
+            else
             {
-                m_completion[ 0 ] ->addItem( macro->key() );
-                BibTeX::Value *value = macro->value() ;
-                for ( QValueList<BibTeX::ValueItem*>::const_iterator it = value->begin(); it != value->end(); it++ )
+                BibTeX::PersonContainer *personContainer = dynamic_cast<BibTeX::PersonContainer*>( *itv );
+                if ( personContainer != NULL )
+                    for ( QValueList<BibTeX::Person*>::Iterator itp = personContainer->persons.begin();itp != personContainer->persons.end(); ++itp )
+                    {
+                        m_completion[ index ] ->addItem(( *itp )->text( TRUE ) );
+                        m_completion[ index ] ->addItem(( *itp )->text( FALSE ) );
+                    }
+                else
                 {
-                    BibTeX::ValueItem *item = *it;
-                    m_completion[ 0 ] ->addItem( item->text() );
+                    BibTeX::KeywordContainer *keywordContainer = dynamic_cast<BibTeX::KeywordContainer*>( *itv );
+                    if ( keywordContainer != NULL )
+                        for ( QValueList<BibTeX::Keyword*>::Iterator itk = keywordContainer->keywords.begin();itk != keywordContainer->keywords.end(); ++itk )
+                            m_completion[ index ] ->addItem(( *itk )->text() );
                 }
             }
         }
@@ -256,7 +411,7 @@ namespace KBibTeX
 
     KCompletion *Settings::completion( BibTeX::EntryField::FieldType fieldType )
     {
-        return m_completion[ completionFieldTypeToIndex( fieldType ) ];
+        return m_completion[ completionFieldTypeToIndex( fieldType )];
     }
 
     int Settings::completionFieldTypeToIndex( BibTeX::EntryField::FieldType fieldType )
@@ -265,7 +420,7 @@ namespace KBibTeX
         if ( fieldType == BibTeX::EntryField::ftEditor )
             fieldType = BibTeX::EntryField::ftAuthor;
         if ( fieldType != BibTeX::EntryField::ftUnknown )
-            index = ( int ) ( fieldType - BibTeX::EntryField::ftAbstract ) + 1;
+            index = ( int )( fieldType - BibTeX::EntryField::ftAbstract ) + 1;
         return index;
     }
 
@@ -291,7 +446,7 @@ namespace KBibTeX
         QValueList<SearchURL*> toDelete;
         for ( QValueList<SearchURL*>::ConstIterator it = searchURLs.begin(); it != searchURLs.end(); ++it )
         {
-            if ( defaultDescriptions.contains( ( *it ) ->description ) )
+            if ( defaultDescriptions.contains(( *it ) ->description ) )
             {
                 toDelete.append( *it );
             }
@@ -306,76 +461,91 @@ namespace KBibTeX
         SearchURL *searchURL = new SearchURL;
         searchURL->description = "Google";
         searchURL->url = "http://www.google.com/search?q=%1&ie=UTF-8&oe=UTF-8";
+        searchURL->includeAuthor = TRUE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "Google Scholar";
         searchURL->url = "http://scholar.google.com/scholar?q=%1&ie=UTF-8&oe=UTF-8";
+        searchURL->includeAuthor = TRUE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "Google .bib Search";
         searchURL->url = "http://www.google.com/search?q=%1%20filetype%3Abib&ie=UTF-8&oe=UTF-8";
+        searchURL->includeAuthor = TRUE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "Google Document Search";
         searchURL->url = "http://www.google.com/search?q=%1%20filetype%3Apdf%20OR%20filetype%3Aps&ie=UTF-8&oe=UTF-8";
+        searchURL->includeAuthor = TRUE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "PubMed";
         searchURL->url = "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=pubmed&cmd=search&pmfilter_Fulltext=on&pmfilter_Relevance=on&term=%1&search=Find+Articles";
+        searchURL->includeAuthor = FALSE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "PubMed Central";
         searchURL->url = "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=pubmed&cmd=search&term=%1";
+        searchURL->includeAuthor = FALSE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "Amatex (US)";
         searchURL->url = "http://www.2ndminute.org:8080/amatex/search.do?querry=%1&suchart=kwd&lang=DE";
+        searchURL->includeAuthor = FALSE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "DBLP (Computer Science)";
         searchURL->url = "http://www.informatik.uni-trier.de/ley/dbbin/dblpquery.cgi?title=%1";
+        searchURL->includeAuthor = FALSE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "CiteSeer";
         searchURL->url = "http://citeseer.ist.psu.edu/cis?q=%1&submit=Search+Documents";
+        searchURL->includeAuthor = FALSE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "citebase";
         searchURL->url = "http://www.citebase.org/search?type=metadata&author=&title=%1&publication=&yearfrom=&yearuntil=&order=DESC&rank=paperimpact&submitted=Search";
+        searchURL->includeAuthor = FALSE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "BASE";
         searchURL->url = "http://digital.ub.uni-bielefeld.de/index.php?q=%1&s=free";
+        searchURL->includeAuthor = FALSE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "Forschungsportal.Net";
         searchURL->url = "http://www.forschungsportal.net/fpj/q/?q=%1&pp=5&art=dok&html=1&pdf=1&ps=1&dvi=1";
+        searchURL->includeAuthor = FALSE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "scirus";
         searchURL->url = "http://www.scirus.com/srsapp/search?q=%1&ds=jnl&ds=nom&ds=web&g=s&t=all";
+        searchURL->includeAuthor = FALSE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "ScientificCommons";
         searchURL->url = "http://en.scientificcommons.org/#search_string=%1";
+        searchURL->includeAuthor = FALSE;
         searchURLs.append( searchURL );
 
         searchURL = new SearchURL;
         searchURL->description = "SpringerLink";
         searchURL->url = "http://www.springerlink.com/content/?k=%1";
+        searchURL->includeAuthor = FALSE;
         searchURLs.append( searchURL );
     }
 
@@ -415,6 +585,8 @@ namespace KBibTeX
             return QString( i18n( "Journal" ) );
         case BibTeX::EntryField::ftKey:
             return QString( i18n( "Key" ) );
+        case BibTeX::EntryField::ftKeywords:
+            return QString( i18n( "Keywords" ) );
         case BibTeX::EntryField::ftLocalFile:
             return QString( i18n( "Local File" ) );
         case BibTeX::EntryField::ftLocation:
@@ -463,5 +635,59 @@ namespace KBibTeX
         return KURL();
     }
 
-}
+    bool Settings::kpsewhich( const QString& filename )
+    {
+        bool result = FALSE;
+        int counter = 0;
 
+        QWaitCondition waitCond;
+        QProcess kpsewhich;
+        kpsewhich.addArgument( "kpsewhich" );
+        kpsewhich.addArgument( filename );
+        if ( kpsewhich.start() )
+        {
+            qApp->processEvents();
+            while ( kpsewhich.isRunning() )
+            {
+                waitCond.wait( 250 );
+                qApp->processEvents();
+
+                counter++;
+                if ( counter > 50 )
+                    kpsewhich.tryTerminate();
+            }
+
+            result = kpsewhich.exitStatus() == 0 && counter < 50;
+        }
+
+        return result;
+    }
+
+    KURL Settings::locateFile( const QString& filename, const QString& bibTeXFileName, QWidget *window )
+    {
+        KURL url( filename );
+        if ( url.isValid() && ( !url.isLocalFile() || KIO::NetAccess::exists( url, TRUE, window ) ) )
+            return url;
+
+        if ( bibTeXFileName != QString::null )
+        {
+            QString path = KURL( bibTeXFileName ).directory( FALSE, FALSE );
+            url = KURL( path + "/" + filename );
+            if ( url.isValid() && KIO::NetAccess::exists( url, TRUE, window ) )
+                return url;
+        }
+
+        Settings* settings = self( NULL );
+        url = KURL( settings->editing_DocumentSearchPath + "/" + filename );
+        if ( url.isValid() && KIO::NetAccess::exists( url, TRUE, window ) )
+            return url;
+
+        return KURL();
+    }
+
+    KURL Settings::locateFile( const QString& filename, QWidget *window )
+    {
+        return locateFile( filename, currentBibTeXFile == NULL ? QString::null : currentBibTeXFile->fileName, window );
+    }
+
+}
