@@ -25,6 +25,7 @@
 #include <kcompletion.h>
 
 #include <file.h>
+#include <fieldlineedit.h>
 #include <fileexporterbibtex.h>
 #include <fileexporterexternal.h>
 #include <entryfield.h>
@@ -49,6 +50,13 @@ namespace KBibTeX
             bool includeAuthor;
         };
 
+        struct UserDefinedInputFields
+        {
+            QString name;
+            QString label;
+            FieldLineEdit::InputType inputType;
+        };
+
         static Settings* self( BibTeX::File *bibtexFile = NULL );
 
         void load( KConfig * config );
@@ -61,6 +69,8 @@ namespace KBibTeX
         KURL locateFile( const QString& filename, QWidget *window = NULL );
 
     public:
+        enum DragAction {COPYREFERENCE = 1, COPYBIBTEX = 2};
+
         // FileIO section
         BibTeX::File::Encoding fileIO_Encoding;
         QString fileIO_ExportLanguage;
@@ -70,8 +80,10 @@ namespace KBibTeX
         BibTeX::FileExporterBibTeX::KeywordCasing fileIO_KeywordCasing;
         BibTeX::FileExporterExternal::Exporter fileIO_ExporterHTML;
         bool fileIO_EmbedFiles;
+        bool fileIO_EnclosingCurlyBrackets;
 
         // Editing section
+        bool editing_SearchBarClearField;
         bool editing_EnableAllFields;
         int editing_MainListDoubleClickAction;
         int editing_MainListSortingColumn;
@@ -87,10 +99,14 @@ namespace KBibTeX
         bool editing_UseSpecialFont;
         bool editing_FirstNameFirst;
         QString editing_DocumentSearchPath;
+        DragAction editing_DragAction;
 
         // SearchURLs section
         QValueList<SearchURL*> searchURLs;
 
+        // UserDefinedInputFields section
+        QValueList<UserDefinedInputFields*> userDefinedInputFields;
+
         // External section
         QString external_XSLTStylesheetHTML;
         bool external_bibconvAvailable;
@@ -100,16 +116,25 @@ namespace KBibTeX
 
         // Id Suggestions
         QStringList idSuggestions_formatStrList;
+        int idSuggestions_default;
+        bool idSuggestions_forceDefault;
 
         // Auto-completion for user fields (non-persistant)
         void addToCompletion( BibTeX::File *file );
         void addToCompletion( BibTeX::Element *element );
+        void addToCompletion( BibTeX::Value *value, BibTeX::EntryField::FieldType fieldType );
         KCompletion* completion( BibTeX::EntryField::FieldType fieldType );
         KCompletion* completionMacro;
 
         // Keyword list
         QStringList keyword_GlobalList;
 
+        // Online Database Search;
+        int webQuery_LastEngine;
+        QString webQuery_LastSearchTerm;
+        int webQuery_LastNumberOfResults;
+        bool webQuery_ImportAll;
+
         // Current BibTeX file
         BibTeX::File *currentBibTeXFile;
 
