@@ -38,13 +38,14 @@ namespace KBibTeX
     {
         Q_OBJECT
     public:
-        SideBar( QWidget *parent = 0, const char *name = 0 );
+        SideBar( bool isReadOnly, QWidget *parent = 0, const char *name = 0 );
         ~SideBar();
 
         void restoreState();
+        void setReadOnly( bool isReadOnly );
 
     signals:
-        void selected( const QString& text, BibTeX::EntryField::FieldType fieldType );
+        void selected( const QString& text, BibTeX::Element::FilterType, BibTeX::EntryField::FieldType fieldType );
         void valueRenamed();
 
     public slots:
@@ -59,6 +60,7 @@ namespace KBibTeX
         QComboBox *m_listTypeList;
         KListView *m_listAvailableItems;
         static const BibTeX::EntryField::FieldType importantFields[ 6 ];
+        bool m_isReadOnly;
         KPopupMenu *m_popupMenu;
         QString m_oldText;
 
