@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -270,10 +270,16 @@ namespace BibTeX
 
     QString Person::text() const
     {
+        return text( m_firstNameFirst );
+    }
+
+    QString Person::text( bool firstNameFirst ) const
+    {
+
         if ( m_firstName.isEmpty() )
             return m_lastName;
         else
-            return m_firstNameFirst ? m_firstName + " " + m_lastName : m_lastName + ", " + m_firstName;
+            return firstNameFirst ? m_firstName + " " + m_lastName : m_lastName + ", " + m_firstName;
     }
 
     QString Person::firstName()
@@ -349,7 +355,7 @@ namespace BibTeX
     void PersonContainer::setText( const QString& text )
     {
         ValueTextInterface::setText( text );
-        qDebug( "You cannot set a text (\"%s\") to a Value object", text.latin1() );
+        qDebug( "You cannot set a text ('%s') to a Value object", text.latin1() );
     }
 
     QString PersonContainer::text() const
@@ -430,22 +436,15 @@ namespace BibTeX
     void Value::setText( const QString& text )
     {
         ValueTextInterface::setText( text );
-        qDebug( "You cannot set a text (\"%s\") to a Value object", text.latin1() );
+        qDebug( "You cannot set a text ('%s') to a Value object", text.latin1() );
     }
 
     QString Value::text() const
     {
         QString result;
-        bool first = TRUE;
 
         for ( QValueList<ValueItem*>::ConstIterator it = items.begin(); it != items.end(); ++it )
-        {
-            if ( !first )
-                result.append( " " );
-            else
-                first = FALSE;
             result.append(( *it )->text() );
-        }
 
         return result;
     }
