@@ -30,10 +30,13 @@
 #include "entry.h"
 
 class QWidget;
+class QFrame;
 class QListView;
+class QCheckBox;
 class QSpinBox;
 class KLineEdit;
 class KComboBox;
+class KProgressDialog;
 
 namespace KBibTeX
 {
@@ -45,46 +48,69 @@ namespace KBibTeX
     {
         Q_OBJECT
     public:
-        WebQuery( QWidget *parent = 0, const char *name = 0 );
-        ~WebQuery();
+        WebQuery( QWidget *parent );
+        virtual ~WebQuery();
 
-        virtual BibTeX::File* query( ) = 0;
+        virtual void query( const QString&, int ) { m_aborted = FALSE; };
+        virtual void cancelQuery();
 
         virtual QString title() = 0;
-        virtual int uniqueId() = 0;
+        virtual QString disclaimer() = 0;
+        virtual QString disclaimerURL() = 0;
 
+    signals:
+        void foundEntry( BibTeX::Entry* );
+        void endSearch( bool );
+        void setProgress( int progress );
+        void setTotalSteps( int totalSteps );
+
+    public slots:
+        void slotCancelQuery();
+
+    protected:
+        bool m_aborted;
+        QWidget *m_parent;
     };
 
-    class WebQueryWizard: public KDialogBase
+    class WebQueryWizard: public QWidget
     {
         Q_OBJECT
     public:
-        WebQueryWizard( const QString &caption, const QString &disclaimer, const QString&disclainerURL, bool showListOfMirrors, bool showNumberOfResults, QWidget* parent, const char* name = 0 );
         ~WebQueryWizard();
 
-        BibTeX::File *result();
+        static int execute( QWidget *parent, QValueList<BibTeX::Entry*> &results );
 
-    public slots:
-        void accept();
-
-    protected slots:
-        void virtual startSearch() = 0;
-
-    private slots:
-        void listViewResultsSelectionChanged( );
+    signals:
+        void changeButtonOK( bool state );
 
     protected:
-        BibTeX::File *m_bibtexFile;
-        QWidget *m_toolbarContainer;
-        KComboBox *m_comboBoxMirrors;
+        QFrame *m_toolbarContainer;
         QListView *m_listViewResults;
+        KComboBox *m_comboBoxEngines;
         KLineEdit *m_lineEditQuery;
         QSpinBox *m_spinBoxMaxHits;
+        KURLLabel *m_disclaimerLabel;
+        QCheckBox *m_checkBoxImportAll;
+        QValueList<WebQuery*> m_webQueries;
+        KDialogBase *m_dlg;
+        KProgressDialog *m_progressDialog;
+
+        static KDialogBase *singletonDlg;
+        static WebQueryWizard *singletonWiz;
+        WebQueryWizard( KDialogBase *dlg, const char* name = 0 );
+
+    private slots:
+        void importEnableChanging( );
+        void otherEngineSelected( int index );
+        void startSearch();
+        void endSearch( bool );
+        void addHit( BibTeX::Entry* );
+        void queryTextChanged( const QString& );
 
     private:
         QPushButton *m_pushButtonSearch;
 
-        void setupGUI( const QString &disclaimer, const QString&disclainerURL, bool showListOfMirrors, bool showNumberOfResults );
+        void setupGUI( );
     };
 
     class ResultsListViewItem: public QListViewItem
