@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2008 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -21,16 +21,13 @@
 #include <qdom.h>
 #include <qapplication.h>
 #include <qstringlist.h>
-#include <qspinbox.h>
 #include <qcstring.h>
 #include <qregexp.h>
 
 #include <kdialog.h>
 #include <klocale.h>
-#include <klineedit.h>
 #include <kurl.h>
 #include <kmessagebox.h>
-#include <kdebug.h>
 #include <kio/netaccess.h>
 
 #include <entryfield.h>
@@ -41,27 +38,14 @@
 namespace KBibTeX
 {
 
-    WebQueryPubMed::WebQueryPubMed( QWidget* parent, const char* name ) : WebQuery( parent, name )
+    WebQueryPubMed::WebQueryPubMed( QWidget *parent ) : WebQuery( parent )
     {
-        wizard = new PubMed::PubMedWizard( title(), parent, name );
+        // nothing
     }
 
     WebQueryPubMed::~WebQueryPubMed()
     {
-        delete wizard;
-    }
-
-    BibTeX::File* WebQueryPubMed::query( )
-    {
-        if ( wizard->exec() == QDialog::Accepted )
-            return wizard->result();
-        else
-            return NULL;
-    }
-
-    int WebQueryPubMed::uniqueId()
-    {
-        return 0x013ac4b8;
+        // nothing
     }
 
     QString WebQueryPubMed::title()
@@ -69,36 +53,40 @@ namespace KBibTeX
         return i18n( "NCBI (PubMed)" );
     }
 
-    PubMed::PubMedWizard::PubMedWizard( const QString& caption, QWidget* parent, const char* name ) : WebQueryWizard( caption, i18n( "NCBI's Disclaimer and Copyright" ), "http://eutils.ncbi.nlm.nih.gov/About/disclaimer.html", FALSE, TRUE, parent, name )
+    QString WebQueryPubMed::disclaimer()
     {
-// nothing
+        return i18n( "NCBI's Disclaimer and Copyright" );
     }
 
-    PubMed::PubMedWizard::~PubMedWizard()
+    QString WebQueryPubMed::disclaimerURL()
     {
-// nothing
+        return "http://eutils.ncbi.nlm.nih.gov/About/disclaimer.html";
     }
 
-    void PubMed::PubMedWizard::startSearch()
+    void WebQueryPubMed::query( const QString& searchTerm, int numberOfResults )
     {
-        setEnabled( FALSE );
-        QApplication::setOverrideCursor( Qt::waitCursor );
-        m_listViewResults->clear();
+        WebQuery::query( searchTerm, numberOfResults );
+
+        emit setTotalSteps( 2 );
+
+        QString term = searchTerm;
+        KURL url = KURL( QString( "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&term=%1&retmax=%2&tool=KBibTeX&email=kbibtex@unix-ag.uni-kl.de" ).arg( term.replace( "%", "%25" ).replace( " ", "+" ).replace( "?", "%3F" ).replace( "&", "%26" ) ).arg( numberOfResults ) );
 
-        KURL url = KURL( QString( "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&term=%1&retmax=%2&tool=KBibTeX&email=kbibtex@unix-ag.uni-kl.de" ).arg( m_lineEditQuery->text().replace( "%", "%25" ).replace( " ", "+" ).replace("?","%3F" ).replace("&","%26" ) ).arg( m_spinBoxMaxHits->text() ) );
         QString tmpFile;
-        if ( KIO::NetAccess::download( url, tmpFile, NULL ) )
+        if ( KIO::NetAccess::download( url, tmpFile, m_parent ) && !m_aborted )
         {
             QFile inputFile( tmpFile );
             QValueList<int> intList;
             QXmlInputSource inputSource( &inputFile );
             QXmlSimpleReader reader;
-            StructureParserQuery handler( &intList );
+            WebQueryPubMedStructureParserQuery handler( &intList );
             reader.setContentHandler( &handler );
             reader.parse( &inputSource );
             inputFile.close();
             KIO::NetAccess::removeTempFile( tmpFile );
 
+            emit setProgress( 1 );
+
             QString ids;
             QValueList<int>::iterator it = intList.begin();
             if ( it != intList.end() )
@@ -113,55 +101,62 @@ namespace KBibTeX
             }
 
             url = KURL( QString( "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&retmode=xml&id=%1&tool=KBibTeX&email=kbibtex@unix-ag.uni-kl.de" ).arg( ids ) );
-            if ( KIO::NetAccess::download( url, tmpFile, NULL ) )
+            if ( KIO::NetAccess::download( url, tmpFile, m_parent ) && !m_aborted )
             {
                 QFile inputFile( tmpFile );
                 QDomDocument doc( "efetch'ed" );
                 doc.setContent( &inputFile );
                 QDomElement docElem = doc.documentElement();
-                ResultParser resultParser( m_listViewResults, doc.documentElement() );
+                emit setProgress( 2 );
+                WebQueryPubMedResultParser resultParser;
+                connect( &resultParser, SIGNAL( foundEntry( BibTeX::Entry* ) ), this, SIGNAL( foundEntry( BibTeX::Entry* ) ) );
+                resultParser.parse( doc.documentElement() );
                 inputFile.close();
                 KIO::NetAccess::removeTempFile( tmpFile );
-
-                setEnabled( TRUE );
-                QApplication::restoreOverrideCursor();
-                m_listViewResults->setFocus();
+                emit endSearch( false );
+            }
+            else if ( !m_aborted )
+            {
+                QString message = KIO::NetAccess::lastErrorString();
+                if ( message.isEmpty() )
+                    message.prepend( '\n' );
+                message.prepend( QString( i18n( "Querying database '%1' failed." ) ).arg( title() ) );
+                KMessageBox::error( m_parent, message );
+                emit endSearch( true );
             }
             else
-            {
-                setEnabled( TRUE );
-                QApplication::restoreOverrideCursor();
-                KMessageBox::error( this, QString( i18n( "Could not query NCBI's database:\n%1" ) ).arg( KIO::NetAccess::lastErrorString() ) );
-                return ;
-            }
-
+                emit endSearch( false );
+        }
+        else if ( !m_aborted )
+        {
+            QString message = KIO::NetAccess::lastErrorString();
+            if ( message.isEmpty() )
+                message.prepend( '\n' );
+            message.prepend( QString( i18n( "Querying database '%1' failed." ) ).arg( title() ) );
+            KMessageBox::error( m_parent, message );
+            emit endSearch( true );
         }
         else
-        {
-            setEnabled( TRUE );
-            QApplication::restoreOverrideCursor();
-            KMessageBox::error( this, QString( i18n( "Could not query NCBI's database:\n%1" ) ).arg( KIO::NetAccess::lastErrorString() ) );
-            return ;
-        }
+            emit endSearch( false );
     }
 
-    PubMed::StructureParserQuery::StructureParserQuery( QValueList<int> *intList ) : QXmlDefaultHandler(), m_intList( intList )
+    WebQueryPubMedStructureParserQuery::WebQueryPubMedStructureParserQuery( QValueList<int> *intList ) : QXmlDefaultHandler(), m_intList( intList )
     {
         m_intList->clear();
     }
 
-    PubMed::StructureParserQuery::~StructureParserQuery( )
+    WebQueryPubMedStructureParserQuery::~WebQueryPubMedStructureParserQuery( )
     {
         // nothing
     }
 
-    bool PubMed::StructureParserQuery::startElement( const QString & /*namespaceURI*/, const QString & /*localName*/, const QString & /*qName*/, const QXmlAttributes & /*atts*/ )
+    bool WebQueryPubMedStructureParserQuery::startElement( const QString & /*namespaceURI*/, const QString & /*localName*/, const QString & /*qName*/, const QXmlAttributes & /*atts*/ )
     {
         concatString = QString();
         return TRUE;
     }
 
-    bool PubMed::StructureParserQuery::endElement( const QString & /*namespaceURI*/, const QString & /*localName*/, const QString & qName )
+    bool WebQueryPubMedStructureParserQuery::endElement( const QString & /*namespaceURI*/, const QString & /*localName*/, const QString & qName )
     {
         if ( qName == "Id" )
         {
@@ -174,13 +169,18 @@ namespace KBibTeX
         return TRUE;
     }
 
-    bool PubMed::StructureParserQuery::characters( const QString & ch )
+    bool WebQueryPubMedStructureParserQuery::characters( const QString & ch )
     {
         concatString.append( ch );
         return TRUE;
     }
 
-    PubMed::ResultParser::ResultParser( QListView *listView, const QDomElement& rootElement )
+    WebQueryPubMedResultParser::WebQueryPubMedResultParser( ) : QObject()
+    {
+        // nothing
+    }
+
+    void WebQueryPubMedResultParser::parse( const QDomElement& rootElement )
     {
         if ( rootElement.tagName() == "PubmedArticleSet" )
             for ( QDomNode n = rootElement.firstChild(); !n.isNull(); n = n.nextSibling() )
@@ -190,17 +190,17 @@ namespace KBibTeX
                 {
                     BibTeX::Entry * entry = new BibTeX::Entry( BibTeX::Entry::etMisc, "PubMed" );
                     parsePubmedArticle( e, entry );
-                    new ResultsListViewItem( listView, entry );
+                    emit foundEntry( entry );
                 }
             }
     }
 
-    PubMed::ResultParser::~ResultParser()
+    WebQueryPubMedResultParser::~WebQueryPubMedResultParser()
     {
         // nothing
     }
 
-    void PubMed::ResultParser::parsePubmedArticle( const QDomElement& element, BibTeX::Entry *entry )
+    void WebQueryPubMedResultParser::parsePubmedArticle( const QDomElement& element, BibTeX::Entry *entry )
     {
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
@@ -210,7 +210,7 @@ namespace KBibTeX
         }
     }
 
-    void PubMed::ResultParser::parseMedlineCitation( const QDomElement& element, BibTeX::Entry *entry )
+    void WebQueryPubMedResultParser::parseMedlineCitation( const QDomElement& element, BibTeX::Entry *entry )
     {
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
@@ -243,14 +243,17 @@ namespace KBibTeX
 
     }
 
-    void PubMed::ResultParser::parseArticle( const QDomElement& element, BibTeX::Entry *entry )
+    void WebQueryPubMedResultParser::parseArticle( const QDomElement& element, BibTeX::Entry *entry )
     {
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
             QDomElement e = n.toElement();
 
             if ( e.tagName() == "Journal" )
+            {
                 parseJournal( e, entry );
+                entry->setEntryType( BibTeX::Entry::etArticle );
+            }
             else if ( e.tagName() == "ArticleTitle" )
             {
                 BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftTitle );
@@ -275,20 +278,17 @@ namespace KBibTeX
                     field->setValue( new BibTeX::Value( medlinePgn.text() ) );
                 }
             }
-            // TF: I AM NOT SURE ABOUT COPYRIGHTS, SO FETCHING ABSTRACTS IS DISABLED FOR NOW
-            /*            else if ( e.tagName() == "Abstract" )
-                        {
-                            QDomElement abstractText = e.firstChild().toElement();
-                            BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftAbstract );
-                            if ( field == NULL )
-                            {
-                                field = new BibTeX::EntryField( BibTeX::EntryField::ftAbstract );
-                                entry->addField( field );
-                            }
-                            BibTeX::Value *value = new BibTeX::Value ();;
-                            value->add( new BibTeX::ValueItem( abstractText.text(), FALSE ) );
-                            field->setValue( value );
-                        }*/
+            else if ( e.tagName() == "Abstract" )
+            {
+                QDomElement abstractText = e.firstChild().toElement();
+                BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftAbstract );
+                if ( field == NULL )
+                {
+                    field = new BibTeX::EntryField( BibTeX::EntryField::ftAbstract );
+                    entry->addField( field );
+                }
+                field->setValue( new BibTeX::Value( abstractText.text() ) );
+            }
             else if ( e.tagName() == "Affiliation" )
             {
                 BibTeX::EntryField * field = entry->getField( "affiliation" );
@@ -304,7 +304,7 @@ namespace KBibTeX
         }
     }
 
-    void PubMed::ResultParser::parseJournal( const QDomElement& element, BibTeX::Entry *entry )
+    void WebQueryPubMedResultParser::parseJournal( const QDomElement& element, BibTeX::Entry *entry )
     {
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
@@ -335,7 +335,7 @@ namespace KBibTeX
         }
     }
 
-    void PubMed::ResultParser::parseJournalIssue( const QDomElement& element, BibTeX::Entry *entry )
+    void WebQueryPubMedResultParser::parseJournalIssue( const QDomElement& element, BibTeX::Entry *entry )
     {
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
@@ -366,7 +366,7 @@ namespace KBibTeX
         }
     }
 
-    void PubMed::ResultParser::parsePubDate( const QDomElement& element, BibTeX::Entry *entry )
+    void WebQueryPubMedResultParser::parsePubDate( const QDomElement& element, BibTeX::Entry *entry )
     {
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
@@ -431,7 +431,7 @@ namespace KBibTeX
         }
     }
 
-    void PubMed::ResultParser::parseAuthorList( const QDomElement& element, BibTeX::Entry *entry )
+    void WebQueryPubMedResultParser::parseAuthorList( const QDomElement& element, BibTeX::Entry *entry )
     {
         if ( element.attribute( "CompleteYN", "Y" ) == "Y" )
         {
@@ -452,7 +452,7 @@ namespace KBibTeX
                     }
                     QString name = lastName;
                     if ( !firstName.isEmpty() )
-                        name.prepend( " " ).prepend( firstName );
+                        name.prepend( "|" ).prepend( firstName );
                     authorList.append( name );
                 }
             }
@@ -468,7 +468,12 @@ namespace KBibTeX
             BibTeX::PersonContainer *personContainer = new BibTeX::PersonContainer( settings->editing_FirstNameFirst );
             value->items.append( personContainer );
             for ( QStringList::Iterator sli = authorList.begin(); sli != authorList.end(); ++sli )
-                personContainer->persons.append( new BibTeX::Person( *sli, settings->editing_FirstNameFirst ) );
+            {
+                QStringList nameParts = QStringList::split( '|', *sli );
+                QString firstName = nameParts.count() > 1 ? nameParts[0] : "";
+                QString lastName = nameParts[nameParts.count() - 1];
+                personContainer->persons.append( new BibTeX::Person( firstName, lastName, settings->editing_FirstNameFirst ) );
+            }
             field->setValue( value );
         }
     }
